/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.commons.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;

import org.apache.commons.lang.StringUtils;

/**
 * Calendar Util
 * 
 * @author yousuke
 * 
 */
public class CalendarUtil {

    /**
     * Get week of year.
     * 
     * @param year
     * @param month
     * @param dayOfMonth
     * @param firstDayOfWeek
     * @return
     */
    public static int getWeekOfYear(int year, int month, int dayOfMonth, int firstDayOfWeek) {
        Calendar calendar = new GregorianCalendar();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month - 1);
        calendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
        calendar.setFirstDayOfWeek(firstDayOfWeek);
        return calendar.get(Calendar.WEEK_OF_YEAR);
    }

    /**
     * Get target week of year calendar.
     * 
     * @param weekOfYear
     * @return
     */
    public static Calendar[] getTargetWeekOfYearCalenadar(int year, int weekOfYear, int firstDayOfWeek) {
        Calendar calendar = new GregorianCalendar();
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.WEEK_OF_YEAR, weekOfYear);
        calendar.setFirstDayOfWeek(firstDayOfWeek);

        if (firstDayOfWeek == 1) {
            calendar.setMinimalDaysInFirstWeek(7);
        }
        Calendar[] cals = new GregorianCalendar[7];
        for (int week = 0; week < cals.length; week++) {
            calendar.set(
                Calendar.DAY_OF_WEEK, week + firstDayOfWeek > 7 
                ? week + firstDayOfWeek - 7 
                : week + firstDayOfWeek);

            cals[week] = new GregorianCalendar();
            cals[week].set(Calendar.YEAR, calendar.get(Calendar.YEAR));
            cals[week].set(Calendar.MONTH, calendar.get(Calendar.MONTH));
            cals[week].set(Calendar.DAY_OF_MONTH, calendar.get(Calendar.DAY_OF_MONTH));
            cals[week].set(Calendar.DAY_OF_WEEK, calendar.get(Calendar.DAY_OF_WEEK));
            cals[week].set(Calendar.WEEK_OF_YEAR, weekOfYear);
            cals[week].setFirstDayOfWeek(firstDayOfWeek);
        }
        return cals;
    }

    /**
     * get GMT calendar.
     * 
     * @param timeZone
     * @return
     */
    public static Calendar getGMTCalendar(String timeZoneId) {
        TimeZone timeZone = TimeZone.getTimeZone(timeZoneId);

        Calendar calendar = new GregorianCalendar();
        calendar.setTimeZone(timeZone);

        DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmm");
        dateFormat.setTimeZone(timeZone);

        timeZone = TimeZone.getTimeZone("GMT");
        calendar.setTimeZone(timeZone);

        dateFormat.setTimeZone(timeZone);
        String calendarStr = dateFormat.format(calendar.getTime());
        Calendar result = 
            new GregorianCalendar(Integer.parseInt(
                calendarStr.substring(0, 4)), 
                Integer.parseInt(calendarStr.substring(4, 6)) - 1, 
                Integer.parseInt(calendarStr.substring(6, 8)), 
                Integer.parseInt(calendarStr.substring(8, 10)), 
                Integer.parseInt(calendarStr.substring(10, 12)));

        result.setTimeZone(timeZone);
        return result;
    }

    /**
     * get this week calendars.
     * 
     * @param timeZoneId
     * @return
     */
    public static Calendar[] getThisWeekCalendars(String timeZoneId,
            int firstDayOfWeek) {
        Calendar calendar = CalendarUtil.getGMTCalendar(timeZoneId);

        Calendar[] weekCalendar = 
            CalendarUtil.getTargetWeekOfYearCalenadar(
                calendar.get(Calendar.YEAR), 
                calendar.get(Calendar.WEEK_OF_YEAR), firstDayOfWeek);

        return weekCalendar;
    }

    /**
     * date serilalize.
     * 
     * @param calnedar
     * @return
     */
    public static String dateSerialize(Calendar calendar) {
        DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmm");
        return dateFormat.format(calendar.getTime());
    }

    /**
     * get date formatted calendar.
     * 
     * @param datetimeStr
     * @param timeFormat
     * @param timezoneStr
     * @param isGMTTime
     * @return
     */
    public static String dateFormattedCalendar(String datetimeStr, String timeFormat, String timezoneStr, boolean isGMTTime) {
        if (datetimeStr == null || datetimeStr.length() < 8) return null;

        int year = Integer.parseInt(datetimeStr.substring(0, 4));
        int month = Integer.parseInt(datetimeStr.substring(4, 6)) - 1;
        int dayOfMonth = Integer.parseInt(datetimeStr.substring(6, 8));
        int hour = 
            datetimeStr.length() > 8 
            ? Integer.parseInt(datetimeStr.substring(8, 10)) 
            : 0;
        int minute = 
            datetimeStr.length() > 10 
            ? Integer.parseInt(datetimeStr.substring(10, 12)) 
            : 0;

        Calendar calendar = new GregorianCalendar(Locale.US);
        calendar.set(Calendar.YEAR, year);
        calendar.set(Calendar.MONTH, month);
        calendar.set(Calendar.DAY_OF_MONTH, dayOfMonth);
        calendar.set(Calendar.HOUR_OF_DAY, hour);
        calendar.set(Calendar.MINUTE, minute);
        calendar.set(Calendar.SECOND, 0);

        if (isGMTTime) {
            DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmm");
            dateFormat.setTimeZone(TimeZone.getTimeZone("GMT"));
            calendar.setTimeZone(TimeZone.getTimeZone("GMT"));
            dateFormat.setTimeZone(TimeZone.getTimeZone(timezoneStr));

            return CalendarUtil
                    .dateFormattedCalendar(
                        dateFormat.format(calendar.getTime()), 
                        timeFormat, timezoneStr, false);
        }
        StringBuilder strBuf = new StringBuilder();
        DateFormat outputFormat = 
            DateFormat.getDateInstance(DateFormat.MEDIUM, Locale.US);

        strBuf.append(StringUtils.replace(
            outputFormat.format(calendar.getTime()), ",", ""));

        if (timeFormat != null && !"".equals(timeFormat)) {
            outputFormat = new SimpleDateFormat(timeFormat);
            strBuf.append(" ").append(outputFormat.format(calendar.getTime()));
            strBuf.append(" ").append(timezoneStr);
        }
        return strBuf.toString();
    }
}
