/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.commons.validator;

import java.text.MessageFormat;
import java.util.ResourceBundle;

import javax.faces.FacesException;
import javax.faces.application.FacesMessage;
import javax.faces.component.StateHolder;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import org.apache.commons.validator.GenericValidator;

/**
 * RegEx Validator
 * 
 * @author yousuke.hara
 * 
 */
public class RegExValidator implements Validator, StateHolder {
    static final String BUNDLE_NAME = "appMessages";

    String pattern_;

    boolean transient_ = false;

    public RegExValidator() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.validator.Validator#validate(javax.faces.context.FacesContext,
     *      javax.faces.component.UIComponent, java.lang.Object)
     */
    public void validate(FacesContext context, UIComponent component,
            Object value) throws FacesException {

        if (!GenericValidator.matchRegexp(this.pattern_, (String) value)) {
            throw new ValidatorException(
                new FacesMessage(this.getResourceMessage()));
        }
    }

    String messageId_;

    public String getMessageId() {
        return this.messageId_;
    }

    public void setMessageId(String messageId) {
        this.messageId_ = messageId;
    }

    Object[] messageArgs_;

    public Object[] getMessageArgs() {
        return this.messageArgs_;
    }

    public void setMessageArgs(Object[] messageArgs) {
        this.messageArgs_ = messageArgs;
    }

    /**
     * 
     * @return
     */
    public String getPattern() {
        return this.pattern_;
    }

    /**
     * 
     * @param pattern
     */
    public void setPattern(String pattern) {
        this.pattern_ = pattern;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.StateHolder#isTransient()
     */
    public boolean isTransient() {
        return this.transient_;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.StateHolder#setTransient(boolean)
     */
    public void setTransient(boolean newTransient) {
        this.transient_ = newTransient;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.StateHolder#saveState(javax.faces.context.FacesContext)
     */
    public Object saveState(FacesContext arg0) {
        Object values[] = new Object[3];
        values[0] = this.pattern_;
        values[1] = this.messageId_;
        values[2] = this.messageArgs_;
        return values;
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.faces.component.StateHolder#restoreState(javax.faces.context.FacesContext,
     *      java.lang.Object)
     */
    public void restoreState(FacesContext context, Object state) {
        Object values[] = (Object[]) state;
        this.pattern_ = values[0].toString();
        this.messageId_ = values[1].toString();
        this.messageArgs_ = (Object[]) values[2];
    }

    /**
     * Get Resource Message
     * 
     * @return
     */
    String getResourceMessage() {
        ResourceBundle bundle = ResourceBundle.getBundle(BUNDLE_NAME);
        String mess = bundle.getString(this.messageId_);
        return MessageFormat.format(mess, this.messageArgs_);
    }
}
