/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.dto;

import java.io.Serializable;
import java.util.Calendar;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.search.annotations.Index;
import org.seasar.tugboat.search.annotations.Searchable;
import org.seasar.tugboat.search.annotations.SearchableField;
import org.seasar.tugboat.search.annotations.SearchableId;
import org.seasar.tugboat.search.annotations.Store;

/**
 * Search result dto.
 */
@Searchable(keyName = "org.seasar.tugboat.gtd.container.dto.SearchDto")
public class SearchDto implements Serializable {
    static final long serialVersionUID = 1L;

    /* unique id */
    @SearchableId(name = "uniqueId", store = Store.YES, index = Index.UN_TOKENIZED)
    String uniqueId;

    public String getUniqueId() {
        return uniqueId;
    }

    public void setUniqueId(String uniqueId) {
        this.uniqueId = uniqueId;
    }

    /* id */
    @SearchableField(name = "id", store = Store.YES, index = Index.UN_TOKENIZED)
    int id;

    public int getId() {
        return id;
    }

    public void setId(int id) {
        this.id = id;
    }

    /* class id */
    @SearchableField(name = "class", store = Store.YES, index = Index.UN_TOKENIZED)
    int classId;

    public int getClassId() {
        return classId;
    }

    public void setClassId(int classId) {
        this.classId = classId;
    }

    /* user id */
    @SearchableField(name = "userId", store = Store.YES, index = Index.UN_TOKENIZED)
    int userId = -1;

    public int getUserId() {
        return userId;
    }

    public void setUserId(int userId) {
        this.userId = userId;
    }

    /* activity id */
    @SearchableField(name = "activityId", store = Store.YES, index = Index.UN_TOKENIZED)
    int activityId;

    public int getActivityId() {
        return activityId;
    }

    public void setActivityId(int activityId) {
        this.activityId = activityId;
    }

    /* title */
    @SearchableField(name = "title", store = Store.YES, index = Index.TOKENIZED)
    String title;

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    /* body */
    @SearchableField(name = "body", store = Store.YES, index = Index.TOKENIZED)
    String body;

    public String getBody() {
        return body;
    }

    public void setBody(String body) {
        this.body = body;
    }

    /* summary */
    String summary;

    public String getSummary() {
        return summary;
    }

    public void setSummary(String summary) {
        this.summary = summary;
    }

    /* classification id */
    @SearchableField(name = "classificationId", store = Store.YES, index = Index.UN_TOKENIZED)
    int classificationId;

    public int getClassificationId() {
        return classificationId;
    }

    public void setClassificationId(int classificationId) {
        this.classificationId = classificationId;
    }

    /* sub project id */
    @SearchableField(name = "subProjectId", store = Store.YES, index = Index.UN_TOKENIZED)
    int subProjectId = -1;

    public int getSubProjectId() {
        return subProjectId;
    }

    public void setSubProjectId(int subProjectId) {
        this.subProjectId = subProjectId;
    }

    /* context id */
    @SearchableField(name = "contextId", store = Store.YES, index = Index.UN_TOKENIZED)
    int contextId = -1;

    public int getContextId() {
        return contextId;
    }

    public void setContextId(int contextId) {
        this.contextId = contextId;
    }

    /* privacy */
    @SearchableField(name = "privacy", store = Store.YES, index = Index.UN_TOKENIZED)
    int privacy = -1;

    public int getPrivacy() {
        return privacy;
    }

    public void setPrivacy(int privacy) {
        this.privacy = privacy;
    }

    /* share */
    @SearchableField(name = "share", store = Store.YES, index = Index.UN_TOKENIZED)
    int share = -1;

    public int getShare() {
        return share;
    }

    public void setShare(int share) {
        this.share = share;
    }

    /* modified */
    @SearchableField(name = "modified", store = Store.YES, index = Index.UN_TOKENIZED)
    long modified;

    public long getModified() {
        return modified;
    }

    public void setModified(long modified) {
        this.modified = modified;
    }

    public Calendar getCalendarModified() {
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(modified);
        return calendar;
    }

    public void setCalendarModified(Calendar modified) {
        this.modified = modified.getTimeInMillis();
    }

    /* xml */
    String xml;

    public String getXml() {
        return xml;
    }

    public void setXml(String xml) {
        this.xml = xml;
    }

    /* modified */
    @SearchableField(name = "userName", store = Store.YES, index = Index.UN_TOKENIZED)
    String userName;

    public String getUserName() {
        return userName;
    }

    public void setUserName(String userName) {
        this.userName = userName;
    }

    /* data level-1 */
    @SearchableField(name = "dataLevel1", store = Store.YES, index = Index.UN_TOKENIZED)
    int dataLevel1;

    public int getDataLevel1() {
        return dataLevel1;
    }

    public void setDataLevel1(int dataLevel1) {
        this.dataLevel1 = dataLevel1;
    }

    /* data level-2 */
    @SearchableField(name = "dataLevel2", store = Store.YES, index = Index.UN_TOKENIZED)
    int dataLevel2;

    public int getDataLevel2() {
        return dataLevel2;
    }

    public void setDataLevel2(int dataLevel2) {
        this.dataLevel2 = dataLevel2;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
