/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.exception;

import java.text.MessageFormat;
import java.util.ResourceBundle;

/**
 * Application Runtime Exception.
 * 
 * @author yousuke.hara
 * 
 */
public abstract class AppRuntimeException extends RuntimeException {
    static final Object[] EMPTY_ARGS = new Object[0];
    static final String BUNDLE_NAME = "appMessages";

    String messageId_;
    Object[] args_;

    /**
     * @param messageId
     */
    public AppRuntimeException(String messageId) {
        this(messageId, EMPTY_ARGS);
    }

    /**
     * @param messageId
     * @param args
     */
    public AppRuntimeException(String messageId, Object[] args) {
        this(messageId, args, null);
    }

    /**
     * @param messageId
     * @param args
     * @param cause
     */
    public AppRuntimeException(String messageId, Object[] args, Throwable cause) {
        initCause(cause);
        this.messageId_ = messageId;
        this.args_ = args;
    }

    /**
     * Get Array
     * 
     * @return
     */
    public Object[] getArgs() {
        return args_;
    }

    /**
     * Get Message Id
     * 
     * @return
     */
    public String getMessageId() {
        return messageId_;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Throwable#getMessage()
     */
    public String getMessage() {
        ResourceBundle bundle = ResourceBundle.getBundle(BUNDLE_NAME);
        String pattern = bundle.getString(messageId_);
        return MessageFormat.format(pattern, args_);
    }
}
