/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.interceptor;

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;

import javax.servlet.http.HttpServletRequest;

import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.aop.interceptors.AbstractInterceptor;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;
import org.seasar.tugboat.gtd.container.annotation.ActionAnnotation;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.dto.SecurityDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * abstract tugboat interceptor.
 * @author yosuke
 *
 */
public abstract class AbstractTugboatInterceptor extends AbstractInterceptor {
    static final long serialVersionUID = 1L;
    public static final String REQ_ATRB_ERROR_MESSAGE = "ERROR_MESSAGE";
    
    /**
     * is exists referer.
     * 
     * @return
     */
    protected boolean isExistsReferer() {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        HttpServletRequest request = 
            (HttpServletRequest) container.getComponent(HttpServletRequest.class);

        String referer = request.getHeader("Referer");
        return referer != null;
    }
    
    /**
     * add error message to http request.
     * 
     * @param message
     */
    protected void addErrorMessageToHttpRequest(String message) {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        HttpServletRequest request = 
            (HttpServletRequest) container.getComponent(HttpServletRequest.class);

        request.setAttribute(REQ_ATRB_ERROR_MESSAGE, message);
    }
    
    /**
     * is exists session.
     * 
     * @return
     */
    protected boolean isExistsSession(boolean isAdminTool) {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        Object session = 
            isAdminTool 
            ? container.getComponent(UserDto.LOGIN_ADMIN_USER_SESSION_ATTRIBUTE_NAME)
            : container.getComponent(UserDto.LOGIN_USER_SESSION_ATTRIBUTE_NAME);

        boolean result = session != null;
        if (session != null) {
            result = 0 < ((UserDto) session).getUserId();
        }
        return result;
    }
    
    /**
     * get securuty dto.
     * 
     * @param invocation
     * @return
     * @throws ClassNotFoundException
     * @throws NoSuchMethodException
     */
    protected SecurityDto getSecurity(MethodInvocation invocation)
            throws ClassNotFoundException, NoSuchMethodException {

        SecurityDto securityDto = new SecurityDto();
        String className = invocation.getClass().getName();
        className = className.substring(0, className.indexOf("$$"));

        Annotation[] classAnnotations = 
            Class.forName(className).getAnnotations();

        if (classAnnotations != null && classAnnotations.length > 0) {
            FormAnnotation formAnnotation = 
                Class.forName(className).getAnnotation(FormAnnotation.class);

            securityDto.setFormId(formAnnotation.formId());
            securityDto.setDescription(formAnnotation.description());
            securityDto.setAuthorizationCheck(formAnnotation.isAuthorizationCheck());
            securityDto.setDirectAccessCheck(formAnnotation.isDirectAccessCheck());
            securityDto.setErrorPage(formAnnotation.errorPage());
            securityDto.setIsAdminTool(formAnnotation.isAdminTool());
        }
        // Get Method Annotations
        Method method = invocation.getMethod();
        Annotation[] methodAnnotations = method.getAnnotations();

        if (methodAnnotations != null && methodAnnotations.length > 0) {
            ActionAnnotation actionAnnotation = method
                    .getAnnotation(ActionAnnotation.class);

            securityDto.setAuthorizationCheck(actionAnnotation
                    .isAuthorizationCheck());
            securityDto.setErrorPage(actionAnnotation.errorPage());
        }
        return securityDto;
    }
}
