/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.service.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.seasar.tugboat.gtd.container.service.Scheduler;

/**
 * abstract scheduler.
 * 
 * @author yosuke
 * 
 */
public abstract class AbstractScheduler implements Scheduler {
    protected List<ScheduleInfo> startSchedule_;

    protected List<ScheduleInfo> stopSchedule_;

    protected long interval_;

    protected String startDateTime_;

    protected String stopDateTime_;

    protected boolean isStarted_;

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#addStartSchedule(int,
     *      int)
     */
    public void addStartSchedule(int hour, int minute) {
        if (startSchedule_ == null)
            startSchedule_ = new ArrayList<ScheduleInfo>();

        ScheduleInfo scheduleInfo = new ScheduleInfo();
        scheduleInfo.setHour(hour);
        scheduleInfo.setMinute(minute);
        startSchedule_.add(scheduleInfo);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#addStopSchedule(int,
     *      int)
     */
    public void addStopSchedule(int hour, int minute) {
        if (stopSchedule_ == null)
            stopSchedule_ = new ArrayList<ScheduleInfo>();

        ScheduleInfo scheduleInfo = new ScheduleInfo();
        scheduleInfo.setHour(hour);
        scheduleInfo.setMinute(minute);
        stopSchedule_.add(scheduleInfo);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#getStartDateTime()
     */
    public String getStartDateTime() {
        return this.startDateTime_;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#getStartSchedule()
     */
    public Date[] getStartSchedule() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#getStopDateTime()
     */
    public String getStopDateTime() {
        return this.stopDateTime_;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#getStopSchedule()
     */
    public Date[] getStopSchedule() {
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#init()
     */
    public void init() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#isStarted()
     */
    public boolean isStarted() {
        return isStarted_;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#setInterval(long)
     */
    public void setInterval(long interval) {
        this.interval_ = interval;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#start()
     */
    public void start() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.Scheduler#stop()
     */
    public void stop() {
    }

    /**
     * schedule info.
     * 
     */
    protected static class ScheduleInfo {
        // hour.
        private int hour_ = 0;

        public int getHour() {
            return hour_;
        }

        public void setHour(int hour) {
            hour_ = hour;
        }

        // minute.
        private int minute_ = 0;

        public int getMinute() {
            return minute_;
        }

        public void setMinute(int minute) {
            minute_ = minute;
        }

        // second
        private int second_ = 0;

        public int getSecond() {
            return second_;
        }

        public void setSecond(int second) {
            second_ = second;
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.lang.Object#toString()
         */
        public String toString() {
            return "hour:[" + hour_ + "],minute:[" + minute_ + "],second:[" + second_ + "]";
        }
    }
}
