/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.container.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.lang.StringUtils;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Sort;
import org.apache.lucene.search.SortField;
import org.apache.lucene.search.TermQuery;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.dto.SearchConditionDto;
import org.seasar.tugboat.gtd.container.dto.SearchDto;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.container.sao.SearchSao;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.container.service.TextSearchResultFormatter;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivitiesDao;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.task.entity.TaskCommentRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.search.SearchCondition;

/**
 * search service.
 * 
 */
public class SearchServiceImpl implements SearchService {
    Logger logger_ = Logger.getLogger(SearchServiceImpl.class);

    static final String FIELD_CLASS = "class";
    static final String FIELD_ID = "id";
    static final String FIELD_USER_ID = "userId";
    static final String FIELD_ACTIVITY_ID = "activityId";
    static final String FIELD_TITLE = "title";
    static final String FIELD_BODY = "body";
    static final String FIELD_CLASSIFICATION_ID = "classificationId";
    static final String FIELD_SUB_PROJECT_ID = "subProjectId";
    static final String FIELD_CONTEXT_ID = "contextId";
    static final String FIELD_PRIVACY = "privacy";
    static final String FIELD_SHARE = "share";
    static final String FIELD_MODIFIED = "modified";

    /* search sao */
    private SearchSao searchSao_;

    public void setSearchSao(SearchSao searchSao) {
        this.searchSao_ = searchSao;
    }

    /* search condition */
    private SearchCondition searchCondition_;

    public SearchCondition getSearchCondition() {
        return searchCondition_;
    }

    public void setSearchCondition(SearchCondition searchCondition) {
        this.searchCondition_ = searchCondition;
    }

    /* TextSearchResultFormatter */
    private TextSearchResultFormatter textSearchResultFormatter_;

    public void setTextSearchResultFormatter(
            TextSearchResultFormatter textSearchResultFormatter) {

        this.textSearchResultFormatter_ = textSearchResultFormatter;
    }

    /* activities dao */
    ActivitiesDao activitiesDao_;

    public void setActivitiesDao(ActivitiesDao activitiesDao) {
        this.activitiesDao_ = activitiesDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#initialize()
     */
    public void initialize() {
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#generateTaskIndex(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations)
     */
    public int generateTaskIndex(TaskRelation entity) {
        SearchDto searchDto = new SearchDto();
        searchDto.setUniqueId(
            SearchService.CLASS_TASK + ID_SEPARATOR + entity.getTaskId());

        searchDto.setId(entity.getTaskId());
        searchDto.setTitle(entity.getTitle());
        searchDto.setBody(entity.getDescription());
        searchDto.setUserId(entity.getUserId());
        searchDto.setActivityId(entity.getActivityId());
        searchDto.setClassId(SearchService.CLASS_TASK);
        searchDto.setUserName(entity.getUserLastName() + ","
                + entity.getUserFirstName());
        searchDto.setModified(System.currentTimeMillis());
        return searchSao_.save(searchDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#generateTaskCommentIndex(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskCommentRelation)
     */
    public int generateTaskCommentIndex(TaskCommentRelation entity) {
        SearchDto searchDto = new SearchDto();
        searchDto.setUniqueId(
            SearchService.CLASS_TASK_COMMENT + ID_SEPARATOR + entity.getId());

        searchDto.setDataLevel1(entity.getId());
        searchDto.setId(entity.getTaskId());
        searchDto.setTitle(entity.getComment());
        searchDto.setBody(entity.getComment());
        searchDto.setUserId(entity.getUserId());
        searchDto.setActivityId(entity.getActivityId());
        searchDto.setClassId(SearchService.CLASS_TASK_COMMENT);
        searchDto.setUserName(entity.getUserLastName() + ","
                + entity.getUserFirstName());
        searchDto.setModified(System.currentTimeMillis());

        return searchSao_.save(searchDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#generateMeetingIndex(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic)
     */
    public int generateMeetingIndex(MeetingBasic entity) {
        SearchDto searchDto = new SearchDto();
        searchDto.setUniqueId(
            SearchService.CLASS_MEETING + ID_SEPARATOR + entity.getId());

        searchDto.setId(entity.getId());
        searchDto.setTitle(entity.getTitle());
        searchDto.setBody(entity.getBody());
        searchDto.setUserId(entity.getPostedUserId());
        searchDto.setActivityId(entity.getActivityId());
        searchDto.setClassId(SearchService.CLASS_MEETING);
        searchDto.setUserName(entity.getLastName() + ","
                + entity.getFirstName());
        searchDto.setModified(System.currentTimeMillis());
        return searchSao_.save(searchDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#generateMeetingAgendaIndex(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda)
     */
    public int generateMeetingAgendaIndex(MeetingAgenda entity) {
        SearchDto searchDto = new SearchDto();
        searchDto.setUniqueId(
            SearchService.CLASS_MEETING_AGENDA + ID_SEPARATOR + entity.getId());

        searchDto.setDataLevel1(entity.getId());
        searchDto.setId(entity.getMeetingBasicId());
        searchDto.setTitle(entity.getBody());
        searchDto.setBody(entity.getBody());
        searchDto.setUserId(entity.getPostedUserId());
        searchDto.setActivityId(entity.getActivityId());
        searchDto.setClassId(SearchService.CLASS_MEETING_AGENDA);
        searchDto.setUserName(entity.getLastName() + ","
                + entity.getFirstName());
        searchDto.setModified(System.currentTimeMillis());

        return searchSao_.save(searchDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#generateMeetingAgendaDiscussionIndex(
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion)
     */
    public int generateMeetingAgendaDiscussionIndex(
            MeetingAgendaDiscussion entity) {
        SearchDto searchDto = new SearchDto();
        searchDto.setUniqueId(
            SearchService.CLASS_MEETING_COMMENT + ID_SEPARATOR + entity.getId());

        searchDto.setDataLevel2(entity.getId());
        searchDto.setDataLevel1(entity.getAgendaId());
        searchDto.setId(entity.getMeetingBasicId());
        searchDto.setTitle(entity.getBody());
        searchDto.setBody(entity.getBody());
        searchDto.setUserId(entity.getPostedUserId());
        searchDto.setActivityId(entity.getActivityId());
        searchDto.setClassId(SearchService.CLASS_MEETING_COMMENT);
        searchDto.setUserName(entity.getLastName() + ","
                + entity.getFirstName());
        searchDto.setModified(System.currentTimeMillis());

        return searchSao_.save(searchDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.SearchService#search(
     *      org.seasar.tugboat.gtd.container.dto.SearchConditionDto, int)
     */
    public SearchDto[] search(SearchConditionDto searchConditionDto, int userId) {
        long startDateTime = System.currentTimeMillis();

        Activity[] activities = activitiesDao_.selectOwnActivities(userId);
        List<String> activityIdList = new ArrayList<String>();
        for (Activity a : activities) {
            activityIdList.add(String.valueOf(a.getId()));
        }
        String[] activityIds = 
            activityIdList.toArray(new String[activityIdList.size()]);

        try {
            BooleanQuery keywordQuery = new BooleanQuery();
            keywordQuery.add(
                new QueryParser(
                    FIELD_BODY, 
                    searchCondition_.getAnalyzer()).parse(
                        searchConditionDto.getKeyword()), BooleanClause.Occur.SHOULD);

            keywordQuery.add(
                new QueryParser(
                    FIELD_TITLE, 
                    searchCondition_.getAnalyzer()).parse(
                        searchConditionDto.getKeyword()), BooleanClause.Occur.SHOULD);

            BooleanQuery booleanQuery = new BooleanQuery();
            booleanQuery.add(keywordQuery, BooleanClause.Occur.MUST);

            if (activityIds != null) {
                booleanQuery.add(createActivityQuery(activityIds), BooleanClause.Occur.MUST);
            }
            logger_.debug("***** Search.Query:" + booleanQuery);

            Sort sort = new Sort(
                new SortField[] {
                    new SortField(FIELD_CLASS, false), 
                    SortField.FIELD_SCORE,
                    new SortField(FIELD_MODIFIED, true) });

            Map<String, String> keywords = new HashMap<String, String>();
            keywords.put(FIELD_BODY, searchConditionDto.getKeyword());
            searchCondition_.setQuery(booleanQuery);
            searchCondition_.setSort(sort);
            searchCondition_.setOffset(searchConditionDto.getOffset());
            SearchDto[] searchDtos = searchSao_.find(searchCondition_);

            long endDateTime = System.currentTimeMillis();
            double procDateTime = endDateTime - startDateTime;

            logger_.debug("<b>Search for " + searchConditionDto.getKeyword()
                    + " </b>" + searchCondition_.getCount() + " couent" + "("
                    + Double.toString(procDateTime / 1000) + " sec)");

            // set results
            searchConditionDto.setOffset(searchCondition_.getOffset());
            searchConditionDto.setCount(searchCondition_.getCount());
            searchConditionDto.setTotalPage(searchCondition_.getTotalPage());
            searchConditionDto.setNext(searchCondition_.hasNextPage());
            searchConditionDto.setPrevious(searchCondition_.hasPreviousPage());
            return searchDtos;
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /**
     * create activity query.
     * 
     * @param activityIds
     * @return
     */
    private BooleanQuery createActivityQuery(String[] activityIds) {
        BooleanQuery activityQuery = new BooleanQuery();
        for (String activityId : activityIds) {
            TermQuery classificationIdQuery = 
                new TermQuery(
                    new Term(FIELD_ACTIVITY_ID, activityId));
            activityQuery.add(classificationIdQuery, BooleanClause.Occur.SHOULD);
        }
        logger_.debug("activityQuery:" + activityQuery.toString());
        return activityQuery;
    }

    /**
     * highlight term.
     * 
     * @param document
     * @param content
     * @return
     */
    String highlightTerm(String document, String content) {

        final String TOKEN_SPACE = " ";
        final String STR_PLUS = "+";
        final String STR_MINUS = "-";
        String result = "";

        if (content.indexOf(TOKEN_SPACE) > -1) {
            StringTokenizer contentToken = 
                new StringTokenizer(content, TOKEN_SPACE);
            result = document;

            while (contentToken.hasMoreTokens()) {
                String strContentToken = contentToken.nextToken();
                strContentToken = 
                    StringUtils.replace(strContentToken, STR_PLUS, "");

                if (strContentToken.startsWith(STR_MINUS)) continue;
                if (document.indexOf(strContentToken) == -1) continue;
                result = 
                    this.textSearchResultFormatter_.highlightTerm(result, strContentToken);
            }
        } else {
            result = 
                this.textSearchResultFormatter_.highlightTerm(document, content);
        }
        return result;
    }
}
