/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.dxo;

import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityRelation;

/**
 * Activity DXO
 * 
 * @author yousuke.hara
 * 
 */
public interface ActivityDxo {

    /**
     * convert activity dto to activity.
     * 
     * @param activityDto
     * @return
     */
    Activity convertActivity(ActivityDto activityDto);

    /**
     * convert activity dtos to activities.
     * 
     * @param activityDtos
     * @return
     */
    Activity[] convertActivities(ActivityDto[] activityDtos);

    /**
     * convert activities to activity dtos.
     * 
     * @param activities
     * @return
     */
    ActivityDto[] convertActivityDtos(Activity[] activities);

    /**
     * convert activities to activity dtos.
     * 
     * @param activities
     * @param members
     * @return
     */
    ActivityDto[] convertActivityDtos(Activity[] activities, ActivityMember[] members);

    /**
     * convert activity to activity dto.
     * 
     * @param activity
     * @param members
     * @return
     */
    ActivityDto convertActivityDto(Activity activity, ActivityMember[] members);

    /**
     * convert activity member dtos to activity members.
     * 
     * @param memberDtos
     * @return
     */
    ActivityMember[] convertActivityMembers(ActivityMemberDto[] memberDtos);

    /**
     * convert activity member dto to activity member.
     * 
     * @param memberDto
     * @return
     */
    ActivityMember convertActivityMember(ActivityMemberDto memberDto);

    /**
     * convert activity members to member dtos.
     * 
     * @param members
     * @return
     */
    ActivityMemberDto[] convertMemberDtos(ActivityMember[] members);

    /**
     * convert activity member to member dto.
     * 
     * @param member
     * @return
     */
    ActivityMemberDto convertMemberDto(ActivityMember member);

    /**
     * convert activity relations dto to relations.
     * 
     * @param relationsDto
     * @return
     */
    ActivityRelation convertActivityRelations(ActivityRelationDto relationsDto);

    /**
     * convert from activity members dto to activity relations.
     * 
     * @param memberDto
     * @return
     */
    ActivityRelation convertFromActivityMembersDtoToActivityRelations(
            ActivityMemberDto memberDto);
}
