/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.dxo.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;
import org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityRelation;

/**
 * Activity Dxo Implement.
 * 
 * @author yousuke
 * 
 */
public class ActivityDxoImpl implements ActivityDxo {

    public ActivityDxoImpl() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivity(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto)
     */
    public Activity convertActivity(ActivityDto activityDto) {
        if (activityDto == null) return null;
        try {
            Activity activity = new Activity();
            BeanUtils.copyProperties(activity, activityDto);
            return activity;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivities(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public Activity[] convertActivities(ActivityDto[] activityDtos) {
        if (activityDtos == null || activityDtos.length == 0) return null;
        
        try {
            List<Activity> list = new ArrayList<Activity>();
            for (ActivityDto dto : activityDtos) {
                Activity activity = new Activity();
                BeanUtils.copyProperties(activity, dto);

                dto.setRegisteredFormated(
                    this.getRegisteredFormated(activity.getRegistered(), Locale.US));

                list.add(activity);
            }
            return list.toArray(new Activity[list.size()]);
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivityDtos(
     *      org.seasar.tugboat.gtd.contents.activity.entity.Activity[])
     */
    public ActivityDto[] convertActivityDtos(Activity[] activities) {
        if (activities == null) return null;
        
        try {
            List<ActivityDto> list = new ArrayList<ActivityDto>();
            for (Activity activity : activities) {
                ActivityDto dto = new ActivityDto();
                BeanUtils.copyProperties(dto, activity);

                dto.setRegisteredFormated(
                    this.getRegisteredFormated(activity.getRegistered(), Locale.US));

                list.add(dto);
            }
            return list.toArray(new ActivityDto[list.size()]);
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.dxo.RelationDxo#convertActivityDtos(
     *      org.seasar.tugboat.gtd.contents.relation.entity.Activity[],
     *      org.seasar.tugboat.gtd.contents.relation.entity.ActivityMembers[])
     */
    public ActivityDto[] convertActivityDtos(Activity[] activities, ActivityMember[] members) {
        if (activities == null || activities.length == 0) return null;
        
        try {
            List<ActivityDto> listActivity = new ArrayList<ActivityDto>();
            for (Activity activity : activities) {
                ActivityDto activityDto = new ActivityDto();
                BeanUtils.copyProperties(activityDto, activity);

                for (ActivityMember member : members) {
                    if (activity.getId() == member.getActivityId()) {
                        ActivityMemberDto memberDto = new ActivityMemberDto();
                        BeanUtils.copyProperties(memberDto, member);

                        activityDto.addMember(memberDto);
                    }
                }
                listActivity.add(activityDto);
            }
            return listActivity.toArray(new ActivityDto[listActivity.size()]);
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.dxo.RelationDxo#convertActivityDto(
     *      org.seasar.tugboat.gtd.contents.relation.entity.Activity,
     *      org.seasar.tugboat.gtd.contents.relation.entity.ActivityMembers[])
     */
    public ActivityDto convertActivityDto(Activity activity, ActivityMember[] members) {
        if (activity == null || members == null || members.length == 0) return null;
        
        try {
            ActivityDto dto = new ActivityDto();
            BeanUtils.copyProperties(dto, activity);

            for (ActivityMember member : members) {
                ActivityMemberDto memberDto = new ActivityMemberDto();
                BeanUtils.copyProperties(memberDto, member);

                dto.setRegisteredFormated(
                    this.getRegisteredFormated(activity.getRegistered(), Locale.US));

                dto.addMember(memberDto);
            }
            return dto;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivityMembers(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto[])
     */
    public ActivityMember[] convertActivityMembers(ActivityMemberDto[] memberDtos) {
        if (memberDtos == null || memberDtos.length == 0) return null;
        
        try {
            List<ActivityMember> list = new ArrayList<ActivityMember>();
            for (ActivityMemberDto dto : memberDtos) {
                ActivityMember member = new ActivityMember();
                BeanUtils.copyProperties(member, dto);

                list.add(member);
            }
            return list.toArray(new ActivityMember[list.size()]);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivityMember(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public ActivityMember convertActivityMember(ActivityMemberDto memberDto) {
        if (memberDto == null) return null;
        
        try {
            ActivityMember member = new ActivityMember();
            BeanUtils.copyProperties(member, memberDto);

            return member;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.dxo.RelationDxo#convertMemberDto(
     *      org.seasar.tugboat.gtd.contents.relation.entity.ActivityMembers[])
     */
    public ActivityMemberDto[] convertMemberDtos(ActivityMember[] members) {
        if (members == null || members.length == 0) return null;
        
        try {
            List<ActivityMemberDto> list = new ArrayList<ActivityMemberDto>();
            for (ActivityMember member : members) {
                ActivityMemberDto dto = new ActivityMemberDto();
                BeanUtils.copyProperties(dto, member);

                dto.setRegisteredFormated(
                    this.getRegisteredFormated(member.getRegistered(), Locale.US));

                list.add(dto);
            }
            return list.toArray(new ActivityMemberDto[list.size()]);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.dxo.RelationDxo#convertMemberDto(
     *      org.seasar.tugboat.gtd.contents.relation.entity.ActivityMembers)
     */
    public ActivityMemberDto convertMemberDto(ActivityMember member) {
        if (member == null) return null;
        
        try {
            ActivityMemberDto dto = new ActivityMemberDto();
            BeanUtils.copyProperties(dto, member);

            dto.setRegisteredFormated(
                this.getRegisteredFormated(member.getRegistered(), Locale.US));

            return dto;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertActivityRelations(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationsDto)
     */
    public ActivityRelation convertActivityRelations(ActivityRelationDto relationsDto) {
        if (relationsDto == null) return null;
        
        try {
            ActivityRelation relations = new ActivityRelation();
            BeanUtils.copyProperties(relations, relationsDto);

            return relations;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /**
     * get registered date formated.
     * 
     * @param registered
     * @param locale
     * @return
     */
    String getRegisteredFormated(String registered, Locale locale) {
        String result = null;
        DateFormat dfShort = new SimpleDateFormat("yyyy-MM-dd", locale);
        if (registered == null || "".equals(registered.trim())
                || registered.length() < 14) {
        } else {
            try {
                result = dfShort.format(new SimpleDateFormat("yyyyMMddHHmmss")
                        .parse(registered));
            } catch (Exception e) {
                throw new SystemUnexpectedRuntimeException(e);
            }
        }
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo#convertFromActivityMembersDtoToActivityRelations(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public ActivityRelation convertFromActivityMembersDtoToActivityRelations(ActivityMemberDto memberDto) {
        ActivityRelation relations = new ActivityRelation();
        relations.setActivityId(memberDto.getActivityId());
        relations.setUserId(memberDto.getUserId());

        return relations;
    }
}
