/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.page.impl;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityRelationDto;
import org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper;
import org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * Activity Ajax
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(
    formId = "activityAjax", description = "activityAjax", 
    isAuthorizationCheck = true, isDirectAccessCheck = true, isAdminTool= false, errorPage = "")
public class ActivityAjaxImpl implements ActivityAjax {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(ActivityAjaxImpl.class);

    public ActivityAjaxImpl() {
        super();
    }

    /* Activity Service */
    ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }

    /* Activity Member Service */
    ActivityMembersService activityMemberService_;

    public void setActivityMemberService(
            ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* Activity Helper */
    ActivityHelper activityHelper_;

    public void setActivityHelper(ActivityHelper activityHelper) {
        this.activityHelper_ = activityHelper;
    }

    /* keyword */
    String keyword_;

    public void setKeyword(String keyword) {
        this.keyword_ = keyword;
    }

    /* id */
    int userId_;

    public void setUserId(int userId) {
        this.userId_ = userId;
    }

    /* activity id */
    int activityId_;

    public void setActivityId(int activityId) {
        this.activityId_ = activityId;
    }

    /* activity name */
    String activityName_;

    public void setActivityName(String activityName) {
        this.activityName_ = activityName;
    }

    /* description */
    String description_;

    public void setDescription(String description) {
        this.description_ = description;
    }

    /* status id */
    int statusId_;

    public void setStatusId(int statusId) {
        this.statusId_ = statusId;
    }

    /* color id */
    int colorId_;

    public void setColorId(int colorId) {
        this.colorId_ = colorId;
    }

    /* offset */
    int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    int limit_;

    public int getLimit() {
        return limit_;
    }

    public void setLimit(int limit) {
        this.limit_ = limit;
    }

    /* current page number */
    int currentPageNumber_;

    public int getCurrentPageNumber() {
        return currentPageNumber_;
    }

    public void setCurrentPageNumber(int currentPageNumber) {
        this.currentPageNumber_ = currentPageNumber;
    }
    
    /* is send mail */
    boolean isSendMail_;
    
    public boolean getIsSendMail() {
        return isSendMail_;
    }
    
    public void setIsSendMail(boolean isSendMail) {
        this.isSendMail_ = isSendMail;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#getOwnActivities()
     */
    public String getOwnActivities() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        ActivityDto[] activities = 
            this.activityService_.getOwnActivities(sessionLoginUser.getUserId());

        return this.activityHelper_.createOwnActivitiesJSON(activities);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#getOwnActivitiesForSelectMenu()
     */
    public String getOwnActivitiesForSelectMenu() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        ActivityDto[] activities = 
            this.activityService_.getOwnActivities(sessionLoginUser.getUserId());

        return this.activityHelper_
                .createOwnActivitiesForSelectMenuJSON(activities);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#getMembersByActivityId()
     */
    public String getMembersByActivityId() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        ActivityMemberDto[] members = 
            this.activityMemberService_.getActivityMembersList(
                    this.activityId_, this.offset_, this.limit_);

        int rows = this.activityService_.getActivityCountByUserId(this.activityId_);

        return this.activityHelper_.createMembersJSON(members, rows,
                this.currentPageNumber_, this.limit_, sessionLoginUser.getUserId());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#addActivity()
     */
    public String add() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        ActivityDto dto = new ActivityDto();
        dto.setName(this.activityName_);
        dto.setDescription(this.description_);
        dto.setModifiedUserId(sessionLoginUser.getUserId());

        ActivityRelationDto relationsDto = new ActivityRelationDto();
        relationsDto.setColorId(this.colorId_);
        relationsDto.setUserId(sessionLoginUser.getUserId());

        ActivityMemberDto member = new ActivityMemberDto();
        member.setUserId(sessionLoginUser.getUserId());
        member.setRoleId(1);
        member.setModifiedUserId(sessionLoginUser.getUserId());

        this.activityService_.addWithMember(dto, relationsDto, member);
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#updateActivity()
     */
    public String update() {
        System.out.println(this.toString());
        
        
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        ActivityDto dto = new ActivityDto();
        dto.setId(this.activityId_);
        dto.setName(this.activityName_);
        dto.setDescription(this.description_);
        dto.setModifiedUserId(sessionLoginUser.getUserId());
        dto.setIsSendMail(this.isSendMail_ ? "1" : "0");

        this.activityService_.update(dto);
        return null;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#updateActivityColor()
     */
    public String updateActivityColor() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        ActivityRelationDto dto = new ActivityRelationDto();
        dto.setActivityId(this.activityId_);
        dto.setColorId(this.colorId_);
        dto.setUserId(sessionLoginUser.getUserId());
        this.activityService_.updateActivityColor(dto);
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityAjax#remove()
     */
    public String remove() {
        ActivityDto dto = new ActivityDto();
        dto.setId(this.activityId_);
        int result = this.activityService_.remove(dto);
        
        return result > 0 ? null :  "[ERROR]";
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
