/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.page.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.helper.ActivityHelper;
import org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * activity member ajax impl.
 * 
 * @author yosuke
 * 
 */
@FormAnnotation(
    formId = "activityMemberAjax", description = "activityMemberAjax", 
    isAuthorizationCheck = true, isDirectAccessCheck = true, isAdminTool= false, errorPage = "")
public class ActivityMemberAjaxImpl implements ActivityMemberAjax {

    /* activity id */
    int activityId_;

    public int getActivityId() {
        return activityId_;
    }

    public void setActivityId(int activityId) {
        this.activityId_ = activityId;
    }
    
    /* activity ids */
    String activityIds_;
    
    public String getActivityIds() {
        return activityIds_;
    }
    public void setActivityIds(String activityIds) {
        this.activityIds_ = activityIds;
    }

    /* user id */
    int userId_;

    public int getUserId() {
        return userId_;
    }

    public void setUserId(int userId) {
        this.userId_ = userId;
    }

    /* role id */
    int roleId_;

    public int getRoleId() {
        return roleId_;
    }

    public void setRoleId(int roleId) {
        this.roleId_ = roleId;
    }

    /* select member char for auto completer */
    String selectMemberChar_;

    public void setSelectMemberChar(String selectMemberChar) {
        this.selectMemberChar_ = selectMemberChar;
    }

    /* Activity Member Service */
    ActivityMembersService activityMemberService_;

    public void setActivityMemberService(
            ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* Activity Helper */
    ActivityHelper activityHelper_;

    public void setActivityHelper(ActivityHelper activityHelper) {
        this.activityHelper_ = activityHelper;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#add()
     */
    public String add() {
        try {
            ActivityMemberDto dto = new ActivityMemberDto();
            BeanUtils.copyProperties(dto, this);

            UserDto sessionLoginUser = SessionUtil.getLoginUser();
            dto.setModifiedUserId(sessionLoginUser.getUserId());

            this.activityMemberService_.add(dto);
            return "";
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#update()
     */
    public String update() {
        try {
            ActivityMemberDto dto = new ActivityMemberDto();
            BeanUtils.copyProperties(dto, this);

            UserDto sessionLoginUser = SessionUtil.getLoginUser();
            dto.setModifiedUserId(sessionLoginUser.getUserId());

            this.activityMemberService_.update(dto);
            return "";
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#getActivityMembers()
     */
    public String getActivityMembers() {
        UserDto[] users = 
            this.activityMemberService_.getActivityMembers(this.selectMemberChar_);

        return this.activityHelper_.createUsersForAutoCompleterHTML(users);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#getActivityMembersByActivities()
     */
    public String getActivityMembersByActivities() {
        if (this.activityIds_ == null || "".equals(this.activityIds_)) {
            return this.getActivityMembersJSON();
        }
        
        StringTokenizer token = new StringTokenizer(this.activityIds_, ",");
        List<Integer> list = new ArrayList<Integer>();
        while (token.hasMoreTokens()) {
            Integer i = new Integer(token.nextToken());
            list.add(i);
        }
        ActivityMemberDto[] members = 
            this.activityMemberService_.getActivityMembersByActivities(list.toArray(new Integer[list.size()]));
        
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        
        return this.activityHelper_.createPersonInChargeJSON(sessionLoginUser.getUserId(), members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#getActivityMembersByActivityIdAndKeyword()
     */
    public String getActivityMembersByActivityIdAndKeyword() {
        UserDto[] users = 
            this.activityMemberService_.getActibityMembersByActivityIdAndKeyword(
                this.activityId_, this.selectMemberChar_);

        return this.activityHelper_.createUsersForAutoCompleterHTML(users);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#checkAlreadyExists()
     */
    public String checkAlreadyExists() {
        ActivityMemberDto member = 
            this.activityMemberService_.getMemberByActivityIdAndUserId(this.activityId_, this.userId_);
        return this.activityHelper_.createMemberJson(member);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#getActivityMembersJSON()
     */
    public String getActivityMembersJSON() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        ActivityMemberDto[] membersDtos = 
            this.activityMemberService_.getMembersListRelatedToUser(sessionLoginUser.getUserId());

        return this.activityHelper_.createPersonInChargeJSON(
                sessionLoginUser.getUserId(), membersDtos);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.page.ActivityMemberAjax#remove()
     */
    public String remove() {
        ActivityMemberDto member = new ActivityMemberDto();
        member.setActivityId(this.activityId_);
        member.setUserId(this.userId_);
        
        int result = this.activityMemberService_.remove(member);
        
        return result > 0 ? null : "[ERROR]";
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
