/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.service;

import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * activity members service.
 * 
 * @author saito
 * 
 */
public interface ActivityMembersService {
    /**
     * Get activity members
     * 
     * @param keyword
     * @return
     */
    UserDto[] getActivityMembers(String keyword);

    /**
     * get activity members by activity id.
     * 
     * @param activityId
     * @param keyword
     * @return
     */
    UserDto[] getActibityMembersByActivityIdAndKeyword(int activityId, String keyword);

    /**
     * get activity member
     * 
     * @param id
     * @return
     */
    UserDto getActivityMemberById(int id);

    /**
     * get activity members list.
     * 
     * @param activityId
     */
    ActivityMemberDto[] getActivityMembersList(int activityId);

    /**
     * get activity members list.
     * 
     * @param activityId
     * @param limit
     * @param offset
     * @return
     */
    ActivityMemberDto[] getActivityMembersList(int activityId, int offset, int limit);
    
    /**
     * get activity members by activities.
     * @param activityIds
     * @return
     */
    ActivityMemberDto[] getActivityMembersByActivities(Integer[] activityIds);

    /**
     * get members list related to user.
     * 
     * @param userId
     */
    ActivityMemberDto[] getMembersListRelatedToUser(int userId);

    /**
     * get activity members by id and keyword.
     * 
     * @param id
     * @param keyword
     * @return
     */
    ActivityMemberDto[] getMembersByIdAndKeyword(int id, String keyword);

    /**
     * get members by activity id and user id.
     * 
     * @param activityId
     * @param userId
     * @return
     */
    ActivityMemberDto getMemberByActivityIdAndUserId(int activityId, int userId);

    /**
     * add member.
     * 
     * @param memberDto
     * @return
     */
    void add(ActivityMemberDto memberDto);

    /**
     * update member.
     * 
     * @param memberDto
     * @return
     */
    void update(ActivityMemberDto memberDto);
    
    /**
     * remove member.
     * @param memberDto
     * @return
     */
    int remove(ActivityMemberDto memberDto);
}
