/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.activity.service.impl;

import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityRelationsDao;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityPageCondition;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityRelation;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.user.dao.UsersDao;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.dxo.UserDxo;
import org.seasar.tugboat.gtd.contents.user.entity.User;
import org.seasar.tugboat.gtd.webservices.service.FeedTokenService;

/**
 * activity members service impl.
 * 
 * @author yosuke
 * 
 */
public class ActivityMembersServiceImpl implements ActivityMembersService {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(ActivityMembersServiceImpl.class);

    /* activity members dao */
    ActivityMembersDao activityMembersDao_;

    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }

    /* Activity DXO */
    ActivityDxo activityDxo_;

    public void setActivityDxo(ActivityDxo activityDxo) {
        this.activityDxo_ = activityDxo;
    }

    /* user dao */
    UsersDao usersDao_;

    public void setUsersDao(UsersDao usersDao) {
        this.usersDao_ = usersDao;
    }

    /* user dxo */
    UserDxo userDxo_;

    public void setUserDxo(UserDxo userDxo) {
        this.userDxo_ = userDxo;
    }

    /* activity relations dao */
    ActivityRelationsDao activityRelationsDao_;

    public void setActivityRelationsDao(
            ActivityRelationsDao activityRelationsDao) {
        this.activityRelationsDao_ = activityRelationsDao;
    }
    
    /* task relations dao */
    TaskRelationsDao taskRelationsDao_;
    
    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }
    
    /* meeting basics dao */
    MeetingBasicsDao meetingBasicDao_;
    
    public void setMeetingBasicsDao(MeetingBasicsDao meetingBasicDao) {
        this.meetingBasicDao_ = meetingBasicDao;
    }

    /* feed token service */
    FeedTokenService feedTokenService_;
    
    public void setFeedTokenService(FeedTokenService feedTokenService) {
        this.feedTokenService_ = feedTokenService;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityService#getActivityMember(java.lang.String)
     */
    public UserDto[] getActivityMembers(String keyword) {
    	if(keyword!=null) keyword = keyword.toLowerCase();    	
    	
        User[] users = this.usersDao_.findUsersByKeyword(keyword + "%");

        return this.userDxo_.convertUserDtos(users);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#getActibityMembersByActivityIdAndKeyword(
     *      int, java.lang.String)
     */
    public UserDto[] getActibityMembersByActivityIdAndKeyword(int activityId, String keyword) {
    	if(keyword!=null) keyword = keyword.toLowerCase();
    	
        User[] users = 
            this.usersDao_.findUsersByActivityIdAndKeyword(activityId, "%" + keyword + "%");
        return this.userDxo_.convertUserDtos(users);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityService#getActivityMemberById(int)
     */
    public UserDto getActivityMemberById(int id) {
        User user = this.usersDao_.findUserById(id);
        return this.userDxo_.convertUserDto(user);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityMembersService#createActivityMembersList(int)
     */
    public ActivityMemberDto[] getActivityMembersList(int activityId) {
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityId(activityId);
        return this.activityDxo_.convertMemberDtos(members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#getActivityMembersList(int,
     *      int, int)
     */
    public ActivityMemberDto[] getActivityMembersList(int activityId, int offset, int limit) {
        ActivityPageCondition condition = new ActivityPageCondition();
        condition.setActivityId(activityId);
        condition.setOffset(offset);
        condition.setLimit(limit);
        
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityIdPerPage(condition);
        return this.activityDxo_.convertMemberDtos(members);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#getActivityMembersByActivities(java.lang.Integer[])
     */
    public ActivityMemberDto[] getActivityMembersByActivities(Integer[] activityIds) {
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityIds(activityIds);
        
        return this.activityDxo_.convertMemberDtos(members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityMembersService#createMembersListRelatedToUser(int)
     */
    public ActivityMemberDto[] getMembersListRelatedToUser(int userId) {
        ActivityMember[] members = 
            this.activityMembersDao_.findByUserId(userId);
        return this.activityDxo_.convertMemberDtos(members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.relation.logic.ActivityService#getMembersByIdAndKeyword(
     *      int, java.lang.String)
     */
    public ActivityMemberDto[] getMembersByIdAndKeyword(int id, String keyword) {
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByIdAndKeyword(id, "%" + keyword + "%");

        return this.activityDxo_.convertMemberDtos(members);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#getMemberByActivityIdAndUserId(int,
     *      int)
     */
    public ActivityMemberDto getMemberByActivityIdAndUserId(int activityId, int userId) {
        ActivityMember member = 
            this.activityMembersDao_.findByActivityIdAndUserId(activityId, userId);

        return this.activityDxo_.convertMemberDto(member);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#add(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public void add(ActivityMemberDto memberDto) {
        ActivityMember member = 
            this.activityDxo_.convertActivityMember(memberDto);
        this.activityMembersDao_.insert(member);

        ActivityRelation notUsedColor = 
            this.activityRelationsDao_.findNotUsedActivityColorByMemberId(memberDto.getUserId());
        ActivityRelation relations = 
            this.activityDxo_.convertFromActivityMembersDtoToActivityRelations(memberDto);
        relations.setColorId(
                relations == null || relations.getColorId() == 0 
                ? 1
                : notUsedColor.getColorId());

        this.activityRelationsDao_.insert(relations);
        
        // create feed token.
        this.feedTokenService_.create(memberDto.getActivityId(), memberDto.getUserId());
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#update(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityMembersDto)
     */
    public void update(ActivityMemberDto memberDto) {
        ActivityMember member = 
            this.activityDxo_.convertActivityMember(memberDto);
        this.activityMembersDao_.update(member);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService#remove(
     *     org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto)
     */
    public int remove(ActivityMemberDto memberDto) {
        int tasks = 
            this.taskRelationsDao_.getCountByActivityIdAndPersonId(
                    memberDto.getActivityId(), memberDto.getUserId());
        
        int meetings = 
            this.meetingBasicDao_.getCountByActivityIdAndPostedUserId(
                    memberDto.getActivityId(), memberDto.getUserId());
        
        int result = 0;
        if (tasks == 0 && meetings == 0) {
            result =
                this.activityMembersDao_.updateDeleteFlagByActivityIdAndUserId(
                    memberDto.getActivityId(), memberDto.getUserId());
            
            // delete feed token.
            this.feedTokenService_.deleteByActivityIdAndUserId(memberDto.getActivityId(), memberDto.getUserId());
        }
        return result;
    }
}
