/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.common.page.impl;

import java.util.Map;

import net.sf.json.JSONObject;
import net.sf.textile4j.Textile;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.common.page.ApplicationCommonAjax;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.container.factory.SingletonS2ContainerFactory;

/**
 * applicatio common ajax implement.
 * 
 * @author yousuke.hara
 * 
 */
@FormAnnotation(formId = "applicationCommonAjax", description = "applicationCommonAjax", isAuthorizationCheck = true, isDirectAccessCheck = true, isAdminTool = false, errorPage = "")
public class ApplicationCommonAjaxImpl implements ApplicationCommonAjax {

    /* source */
    String source_;

    public String getSource() {
        return source_;
    }

    public void setSource(String source) {
        this.source_ = source;
    }
    
    /* activity id */
    int activityId;
    public void setActivityId(int activityId) {
        this.activityId = activityId;
    }

    /* activity service */
    ActivityService activityService_;
    
    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.common.ajax.ApplicationCommonAjax#convertTextile()
     */
    public String convertTextile() {
        Textile textile = new Textile();
        return textile.process(this.source_);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.common.ajax.ApplicationCommonAjax#getLoginUserInfo()
     */
    public String getLoginUserInfo() {
        S2Container container = SingletonS2ContainerFactory.getContainer();
        UserDto sessionLoginUser = (UserDto) container
                .getComponent(UserDto.LOGIN_USER_SESSION_ATTRIBUTE_NAME);

        try {
            Map<?,?> map = BeanUtils.describe(sessionLoginUser);
            JSONObject json = JSONObject.fromObject(map);

            return json.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.common.page.ApplicationCommonAjax#getSendMailEnabled()
     */
    public String getSendMailEnabled() {
        String smtp = SystemConfiguration.getValue("tugboat.gtd.mail.smtp");
        String from = SystemConfiguration.getValue("tugboat.gtd.mail.from");
        String returnPath = SystemConfiguration
                .getValue("tugboat.gtd.mail.return_path");
        boolean bSmtp = smtp != null && !"".equals(smtp);
        boolean bFrom = from!= null && !"".equals(from);
        boolean bReturnPath = returnPath!= null && !"".equals(returnPath);
        boolean bSendMail = activityService_.getActivityById(activityId).getIsSendMail().equals("1");
        return "(" + (bSmtp && bFrom && bReturnPath && bSendMail) +")";
        
    }

    
}
