/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dao;

import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;

/**
 * meeting agenda discussions dao.
 * 
 * @author naoya.saito
 * 
 */
@S2Dao(bean = MeetingAgendaDiscussion.class)
public interface MeetingAgendaDiscussionsDao {

    /**
     * insert meeting agenda discussion.
     * 
     * @param discussion
     * @return
     */
    @Arguments("discussion")
    int insert(MeetingAgendaDiscussion discussion);

    /**
     * update meeting agenda discussion.
     * 
     * @param discussion
     * @return
     */
    @Arguments("discussion")
    int update(MeetingAgendaDiscussion discussion);

    /**
     * update meeting agenda discussion delete flag.
     * 
     * @param discussion
     * @return
     */
    @Arguments("discussion")
    int updateDelFlag(MeetingAgendaDiscussion discussion);

    /**
     * find meeting agenda discussion id.
     * 
     * @return
     */
    MeetingAgendaDiscussion findNewId();

    /**
     * find by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    MeetingAgendaDiscussion[] findByUserIdForTimeline(int userId);

    /**
     * find meeting agenda discussion relation by id.
     * 
     * @param id
     * @return
     */
    @Arguments("id")
    MeetingAgendaDiscussion findDiscussionRelationById(int id);
}
