/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;

/**
 * meeting agendas DAO.
 * 
 * @author naoya.saito
 * 
 */
@S2Dao(bean = MeetingAgenda.class)
public interface MeetingAgendasDao {

    /**
     * insert meeting agenda.
     * 
     * @param agenda
     * @return
     */
    @Arguments("agenda")
    int insert(MeetingAgenda agenda);

    /**
     * update meeting agenda.
     * 
     * @param agenda
     * @return
     */
    @Arguments("agenda")
    int update(MeetingAgenda agenda);

    /**
     * update meeting agenda delete flag.
     * 
     * @param agenda
     * @return
     */
    @Arguments("agenda")
    int updateDelFlag(MeetingAgenda agenda);

    /**
     * find meeting agenda id.
     * 
     * @return
     */
    MeetingAgenda findNewId();

    /**
     * find by meeting id.
     * 
     * @param meetingId
     * @return
     */
    @Arguments("meetingId")
    MeetingAgenda[] findByMeetingId(int meetingId);
    
    /**
     * find with discussion by meeting id and agenda id.
     * 
     * @param meetingId
     * @param agendaId
     * @return
     */
    @Arguments({"meetingId", "agendaId"})
    MeetingAgenda[] findWithDiscussionByMeetingIdAndAgendaId(int meetingId, int agendaId);
    
    /**
     * find with meeting Agendas with discussion by basic id.
     * 
     * @param meetingId
     * @return
     */
    @Arguments("meetingId")
    MeetingAgenda[] findWithDiscussionByMeetingId(int meetingId);
    
    /**
     * find by user id for timeline.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    MeetingAgenda[] findByUserIdForTimeline(int userId);

    /**
     * find meeting agenda relation by id.
     * 
     * @param id
     * @return
     */
    @Arguments("id")
    MeetingAgenda findAgendaRelationById(int id);
    
    /**
     * get cooment rows.
     * @param ids
     * @return
     */
    @Arguments("ids")
    MeetingAgenda[] getCommentRows(Integer[] ids);
    
    /**
     * get task rows.
     * @param ids
     * @return
     */
    @Arguments("ids")
    MeetingAgenda[] getTaskRows(Integer[] ids);
}
