/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dao;

import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBrowsePageCondition;
import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;

/**
 * meeting basics DAO.
 * 
 * @author naoya.saito
 * 
 */
@S2Dao(bean = MeetingBasic.class)
public interface MeetingBasicsDao {

    /**
     * insert meeting basic.
     * 
     * @param basic
     * @return
     */
    @Arguments("basic")
    int insert(MeetingBasic basic);

    /**
     * update meeting basic.
     * 
     * @param basic
     * @return
     */
    @Arguments("basic")
    int update(MeetingBasic basic);
    
    /**
     * update meeting basic status.
     * 
     * @param basic
     * @return
     */
    @Arguments("basic")
    int updateStatus(MeetingBasic basic);

    /**
     * find meeting basic by id.
     * 
     * @param userId
     * @param basicId
     * @return
     */
    @Arguments( { "userId", "basicId" })
    MeetingBasic findById(int userId, int basicId);

    /**
     * find meeting basic id.
     * 
     * @return
     */
    MeetingBasic findNewId();

    /**
     * find meeting basics by user id.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    MeetingBasic[] findByUserId(MeetingBrowsePageCondition condition);

    /**
     * find meeting count by query params.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    MeetingBasic findCountByQueryParams(MeetingBrowsePageCondition condition);

    /**
     * find meeting count by folder.
     * 
     * @param condition
     * @return
     */
    @Arguments("condition")
    MeetingBasic findDefaultCountByFolder(MeetingBrowsePageCondition condition);

    /**
     * find meeting basics with task by id.
     * 
     * @param id
     * @param userId
     * @return
     */
    @Arguments( { "id", "userId" })
    MeetingBasic[] findWithTaskById(int id, int userId);

    /**
     * find by user id for timeline.
     * 
     * @param userId
     * @return
     */
    @Arguments("basic")
    MeetingBasic[] findByUserIdForTimeline(int userId);
    
    /**
     * get count by activity id.
     * @param activityId
     * @return
     */
    @Arguments("activityId")
    int getCountByActivityId(int activityId);
    
    /**
     * get count by activity id and posted user id.
     * @param activityId
     * @param postedUserId
     * @return
     */
    @Arguments( { "activityId", "postedUserId" })
    int getCountByActivityIdAndPostedUserId(int activityId, int postedUserId);
}
