/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;

/**
 * meeting to tasks DAO.
 * 
 * @author naoya.saito
 * 
 */
@S2Dao(bean = MeetingToTask.class)
public interface MeetingToTasksDao {

    /**
     * find meeting tasks by basic id.
     * 
     * @param basicId
     * @return
     */
    @Arguments("condition")
    MeetingToTask[] findTasksByBasicId(MeetingBrowsePageCondition condition);

    /**
     * find meeting tasks by agenda id.
     * 
     * @param agenda
     * @return
     */
    @Arguments("agendaId")
    MeetingToTask[] findTasksByAgendaId(int  agendaId);
    
    /**
     * find task count by basic id.
     * 
     * @param condition
     * @return
     */
    @Arguments("basicId")
    MeetingToTask findTaskCountByBasicId(int basicId);

    /**
     * insert meeting relations.
     * 
     * @param toTask
     * @return
     */
    @Arguments("toTask")
    int insert(MeetingToTask toTask);

    /**
     * find by task id.
     * 
     * @param taskId
     * @return
     */
    @Arguments("taskId")
    MeetingToTask[] findByTaskId(int taskId);
}
