/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.helper;

import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;

/**
 * meeting helper
 * 
 * @author naoya
 * 
 */
public interface MeetingHelper {

    /**
     * create activities json for task.
     * 
     * @param activities
     * @return
     */
    String createActivitiesJsonForTask(ActivityDto[] activities);

    /**
     * create meeting list.
     * 
     * @param basicDtoList
     * @param totalRecords
     * @param currentPageNumber
     * @param recordPerPage
     * @return
     */
    String createMeetingList(MeetingBasicDto[] basicDtoList, int totalRecords,
            int currentPageNumber, int recordPerPage);

    /**
     * create agenda list.
     * 
     * @param basicDtoList
     * @param agendaDto
     * @param user
     * @param hasDiscussion
     * @return
     */
    String createAgendasJson(MeetingBasicDto basicDtoList, MeetingAgendaDto[] agendaDto, UserDto user, boolean hasDiscussion);

    /**
     * create basic json.
     * 
     * @param basicDto
     * @return
     */
    String createBasicJson(MeetingBasicDto basicDto);

    /**
     * create agenda json.
     * 
     * @param agendaDto
     * @return
     */
    String createAgendaJson(MeetingAgendaDto agendaDto);

    /**
     * create agenda discussion json with meeting basic.
     * 
     * @param basicDto
     * @param discussionDto
     * @param relationsDto 
     * @param taskDto 
     * @return
     */
    String createAgendaDiscussionJsonWithMeetingBasic(MeetingBasicDto basicDto,
            MeetingAgendaDiscussionDto discussionDto, TaskDto taskDto, TaskRelationDto relationsDto);

    /**
     * create sidebar members.
     * 
     * @param activityMembersDto
     * @return
     */
    String createSidebarMembers(ActivityMemberDto[] activityMembersDto);

    /**
     * create sidebar tasks.
     * 
     * @param toTaskDto
     * @param totalRecords
     * @param currentPageNumber
     * @param recordPerPage
     * @return
     */
    String createSidebarTasks(MeetingToTaskDto[] toTaskDto, int totalRecords,
            int currentPageNumber, int recordPerPage);

    /**
     * create browser register json.
     * 
     * @param activityMembersDto
     * @return
     */
    String createBrowserRegisterJson(ActivityMemberDto[] activityMembersDto);

    /**
     * create meeting ballon json.
     * 
     * @param basicDto
     * @return
     */
    String createMeetingBallonJson(MeetingBasicDto basicDto);

    /**
     * create folder meeting count json.
     * 
     * @param basicDto
     * @return
     */
    String createFolderMeetingCountJson(MeetingBasicDto basicDto);
}
