/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.page.impl;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityMemberDto;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityMembersService;
import org.seasar.tugboat.gtd.contents.activity.service.ActivityService;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.helper.MeetingHelper;
import org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.mailservices.service.MailService;

/**
 * meeting detail ajax impl.
 * 
 */
@FormAnnotation(formId = "meetingDetailAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class MeetingDetailAjaxImpl implements MeetingDetailAjax {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(MeetingDetailAjaxImpl.class);

    /* basic id */
    int basicId_;

    public int getBasicId() {
        return basicId_;
    }

    public void setBasicId(int eventId) {
        this.basicId_ = eventId;
    }

    /* status id */
    int statusId_;

    public int getStatusId() {
        return statusId_;
    }

    public void setStatusId(int statusId) {
        this.statusId_ = statusId;
    }

    /* activity id */
    int activityId_;

    public int getActivityId() {
        return activityId_;
    }

    public void setActivityId(int activityId) {
        this.activityId_ = activityId;
    }

    /* agenda id */
    int agendaId_;

    public void setAgendaId(int topicId) {
        this.agendaId_ = topicId;
    }

    public int getAgendaId() {
        return this.agendaId_;
    }

    /* agenda priority */
    String priority_ = "0";

    public void setPriority(String priority) {
        this.priority_ = priority;
    }

    public String getPriority() {
        return this.priority_;
    }

    /* agenda or discussion posted user id */
    int postedUserId_ = -1;

    public void setPostedUserId(int postedUserId) {
        this.postedUserId_ = postedUserId;
    }

    public int getPostedUserId() {
        return this.postedUserId_;
    }

    /* discussion id */
    int discussionId_;

    public void setDiscussionId(int discussionId) {
        this.discussionId_ = discussionId;
    }

    public int getDiscussionId() {
        return this.discussionId_;
    }

    /* discussion posted type */
    String postedType_ = null;

    public void setPostedType(String postedType) {
        this.postedType_ = postedType;
    }

    public String getPostedType() {
        return this.postedType_;
    }

    /* bodyTextile */
    String bodyTextile_;

    public String getBodyTextile() {
        return bodyTextile_;
    }

    public void setBodyTextile(String bodyTextile) {
        this.bodyTextile_ = bodyTextile;
    }

    /* title */
    String title_;

    public String getTitle() {
        return title_;
    }

    public void setTitle(String title) {
        this.title_ = title;
    }

    /* descriptionTextile */
    String descriptionTextile_;

    public String getDescriptionTextile() {
        return descriptionTextile_;
    }

    public void setDescriptionTextile(String descriptionTextile) {
        this.descriptionTextile_ = descriptionTextile;
    }

    /* personInChargeId */
    String personInChargeId_;

    public String getPersonInChargeId() {
        return personInChargeId_;
    }

    public void setPersonInChargeId(String personInChargeId) {
        this.personInChargeId_ = personInChargeId;
    }

    /* current page number */
    int currentPageNumber_;

    public int getCurrentPageNumber() {
        return currentPageNumber_;
    }

    public void setCurrentPageNumber(int currentPageNumber) {
        this.currentPageNumber_ = currentPageNumber;
    }

    /* offset */
    int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    int limit_;

    public int getLimit() {
        return limit_;
    }

    public void setLimit(int limit) {
        this.limit_ = limit;
    }
    
    /* task title */
    String taskTitle_;
    public String getTaskTitle() {
        return taskTitle_;
    }
    
    public void setTaskTitle(String taskTitle) {
        this.taskTitle_ = taskTitle;
    }
    
    /* task description */
    String taskDescription_;
    
    public String getTaskDescription() {
        return taskDescription_;
    }
    
    public void setTaskDescription(String taskDescription) {
        this.taskDescription_ = taskDescription;
    }
    
    /* task dead line */
    String taskDeadLine_;
    
    public String getTaskDeadLine(){
        return taskDeadLine_;
    }
    
    public void setTaskDeadLine(String taskDeadLine) {
        this.taskDeadLine_ = taskDeadLine;
    }
    
    /* task person id */
    int taskPersonId_;
    
    public int getTaskPersonId() {
        return taskPersonId_;
    }
    
    public void setTaskPersonId(int taskPersonId) {
        this.taskPersonId_ = taskPersonId;
    }
    
    /* task classification id */
    int taskClassificationId_;
    
    public int getTaskClassificationId(){
        return taskClassificationId_;
    }
    
    public void setTaskClassificationId(int taskClassificationId) {
        this.taskClassificationId_ = taskClassificationId;
    }
    
    int taskStatusId_;
    
    public int getTaskStatusId() {
        return taskStatusId_;
    }
    
    public void setTaskStatusId(int taskStatusId) {
        this.taskStatusId_ = taskStatusId;
    }
    
    /* meeting Helper */
    MeetingHelper meetingHelper_;

    public void setMeetingHelper(MeetingHelper meetingHelper) {
        this.meetingHelper_ = meetingHelper;
    }

    /* meeting service */
    MeetingService meetingService_;

    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }

    /* activity members service */
    ActivityMembersService activityMemberService_;

    public void setActivityMemberService(
            ActivityMembersService activityMemberService) {
        this.activityMemberService_ = activityMemberService;
    }

    /* mail service */
    MailService mailService_;

    public void setMailService(MailService mailService) {
        this.mailService_ = mailService;
    }

    /* activity service */
    ActivityService activityService_;

    public void setActivityService(ActivityService activityService) {
        this.activityService_ = activityService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getAgendaListOnLoad()
     */
    public String getAgendaListOnLoad() {
        UserDto userInfo = SessionUtil.getLoginUser();

        MeetingBasicDto dto = new MeetingBasicDto();
        dto.setId(this.basicId_);
        dto.setUserId(userInfo.getUserId());

        MeetingBasicDto meetingInfo = 
            this.meetingService_.getMeetingBasic(dto);

        MeetingAgendaDto[] agendas = 
            this.meetingService_.getAgendaList(this.basicId_);

        return this.meetingHelper_.createAgendasJson(meetingInfo, agendas, userInfo, false);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateMeeting()
     */
    public String updateMeeting() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingBasicDto basicDto = new MeetingBasicDto();
        basicDto.setId(this.basicId_);
        basicDto.setStatusId(this.statusId_);
        basicDto.setActivityId(this.activityId_);
        basicDto.setTitle(this.title_);
        basicDto.setBody(this.descriptionTextile_);
        basicDto.setPostedUserId(sessionLoginUser.getUserId());
        basicDto.setUserId(sessionLoginUser.getUserId());
        basicDto = this.meetingService_.updateMeeting(basicDto);

        return this.meetingHelper_.createBasicJson(basicDto);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#updateStatus()
     */
    public String updateStatus() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        MeetingBasicDto basicDto =
            this.meetingService_.updateStatus(
                this.basicId_, this.statusId_, sessionLoginUser.getUserId());
        
        return this.meetingHelper_.createBasicJson(basicDto);
    }
    
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#addAgenda()
     */
    public String addAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        try {
            MeetingAgendaDto agendaDto = new MeetingAgendaDto();
            agendaDto.setPriority(this.priority_);
            agendaDto.setBody(this.bodyTextile_);
            agendaDto.setPostedUserId(this.postedUserId_);
            agendaDto.setModifiedUserId(sessionLoginUser.getUserId());
            agendaDto = this.meetingService_.addAgenda(agendaDto, this.basicId_);

            return this.meetingHelper_.createAgendaJson(agendaDto);
        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#getAgendaDiscussion()
     */
    public String getAgendaDiscussion() {
        UserDto userInfo = SessionUtil.getLoginUser();
        MeetingAgendaDto[] agendas = 
            this.meetingService_.getAgendasWithDiscussionByIdAndAgendaId(this.basicId_, this.agendaId_);
        
        return this.meetingHelper_.createAgendasJson(null, agendas, userInfo, true);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#addAgendaDiscussion()
     */
    public String addAgendaDiscussion() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        try {
            MeetingBasicDto basicDto = new MeetingBasicDto();
            basicDto.setId(this.basicId_);
            basicDto.setStatusId(MeetingBasicDto.STATUS_OPEN);
            basicDto.setPostedUserId(sessionLoginUser.getUserId());
            basicDto.setUserId(sessionLoginUser.getUserId());
            
            MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
            discussionDto.setAgendaId(this.agendaId_);
            discussionDto.setType(this.postedType_);
            discussionDto.setBody(this.bodyTextile_);
            discussionDto.setPostedUserId(this.postedUserId_);
            discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

            MeetingToTaskDto toTaskDto = new MeetingToTaskDto();
            BeanUtils.copyProperties(toTaskDto, this);

            TaskDto taskDto = new TaskDto();
            taskDto.setTitle(this.taskTitle_);
            taskDto.setDescription(this.taskDescription_);
            taskDto.setDescriptionTextile(this.taskDescription_);
            taskDto.setDeadline(this.taskDeadLine_);
            taskDto.setClassificationId(this.taskClassificationId_);
            taskDto.setStatusId(this.taskStatusId_);
            taskDto.setUserId(sessionLoginUser.getUserId());
            taskDto.setActivityId(this.activityId_);

            TaskRelationDto relationsDto = new TaskRelationDto();
            relationsDto.setActivityId(this.activityId_);
            relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
            relationsDto.setClassificationId(Classification.INBOX);

            relationsDto.setPersonInChargeId(taskPersonId_);
            relationsDto.setUserId(sessionLoginUser.getUserId());

            discussionDto = 
                this.meetingService_.addAgendaDiscussion(
                    discussionDto, taskDto, relationsDto, toTaskDto);
            
            MeetingBasicDto liveMeetingBasicDto = 
                this.meetingService_.getMeetingBasic(basicDto);

            return this.meetingHelper_.createAgendaDiscussionJsonWithMeetingBasic(
                liveMeetingBasicDto, discussionDto, taskDto, relationsDto);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateAgenda()
     */
    public String updateAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDto agendaDto = new MeetingAgendaDto();
        agendaDto.setId(this.agendaId_);
        agendaDto.setPostedUserId(this.postedUserId_);
        agendaDto.setBody(this.bodyTextile_);
        agendaDto.setModifiedUserId(sessionLoginUser.getUserId());
        agendaDto = this.meetingService_.updateAgenda(agendaDto);

        return this.meetingHelper_.createAgendaJson(agendaDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#deleteAgenda()
     */
    public void deleteAgenda() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDto agendaDto = new MeetingAgendaDto();
        agendaDto.setId(this.agendaId_);
        agendaDto.setModifiedUserId(sessionLoginUser.getUserId());

        this.meetingService_.deleteAgenda(agendaDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#updateAgendaDiscussion()
     */
    public String updateAgendaDiscussion() {
        try {
            UserDto sessionLoginUser = SessionUtil.getLoginUser();

            MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
            discussionDto.setId(this.discussionId_);
            discussionDto.setAgendaId(this.agendaId_);
            discussionDto.setType(this.postedType_);
            discussionDto.setPostedUserId(this.postedUserId_);
            discussionDto.setBody(this.bodyTextile_);
            discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

            MeetingToTaskDto toTaskDto = new MeetingToTaskDto();
            BeanUtils.copyProperties(toTaskDto, this);

            TaskDto taskDto = new TaskDto();
            taskDto.setTitle(this.taskTitle_);
            taskDto.setDescription(this.taskDescription_);
            taskDto.setDescriptionTextile(this.taskDescription_);
            taskDto.setDeadline(this.taskDeadLine_);
            taskDto.setUserId(sessionLoginUser.getUserId());

            TaskRelationDto relationsDto = new TaskRelationDto();
            relationsDto.setActivityId(this.activityId_);
            relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
            relationsDto.setClassificationId(Classification.INBOX);

            relationsDto.setPersonInChargeId(taskPersonId_);
            relationsDto.setUserId(sessionLoginUser.getUserId());

            discussionDto = 
                this.meetingService_.updateAgendaDiscussion(
                    discussionDto, toTaskDto, taskDto, relationsDto);
            return this.meetingHelper_.createAgendaDiscussionJsonWithMeetingBasic(
                null, discussionDto,taskDto,null);

        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#deleteAgendaDiscussion()
     */
    public void deleteAgendaDiscussion() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        MeetingAgendaDiscussionDto discussionDto = new MeetingAgendaDiscussionDto();
        discussionDto.setId(this.discussionId_);
        discussionDto.setAgendaId(this.agendaId_);
        discussionDto.setType(this.postedType_);
        discussionDto.setPostedUserId(this.postedUserId_);
        discussionDto.setActivityId(this.activityId_);
        discussionDto.setMeetingBasicId(this.basicId_);
        discussionDto.setModifiedUserId(sessionLoginUser.getUserId());

        this.meetingService_.deleteAgendaDiscussion(discussionDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getSidebarMembersContents()
     */
    public String getSidebarMembersContents() {
        ActivityMemberDto[] activityMembersDto = 
            this.activityMemberService_.getActivityMembersList(this.activityId_);

        return this.meetingHelper_.createSidebarMembers(activityMembersDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingAjax#getSidebarTasksContents()
     */
    public String getSidebarTasksContents() {
        MeetingToTaskDto conditionDto = new MeetingToTaskDto();
        conditionDto.setBasicId(this.basicId_);
        conditionDto.setOffset(this.offset_);
        conditionDto.setLimit(this.limit_);

        MeetingToTaskDto[] liveMeetingToTaskDto = 
            this.meetingService_.getTasksByBasicId(conditionDto);
        MeetingToTaskDto toTaskDto = 
            this.meetingService_.getTaskCountByBasicId(conditionDto);
        String result = 
            this.meetingHelper_.createSidebarTasks(
                liveMeetingToTaskDto, toTaskDto.getRowCount(), this.currentPageNumber_, this.limit_);
        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this, ToStringStyle.MULTI_LINE_STYLE);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.page.MeetingDetailAjax#sendMail()
     */
    public String sendMail() {
        int userId = SessionUtil.getLoginUser().getUserId();
        MeetingBasicDto dto = new MeetingBasicDto();
        dto.setId(basicId_);
        dto.setUserId(userId);
        MeetingBasicDto eventHeaderDto = 
            this.meetingService_.getMeetingBasic(dto);
        MeetingAgendaDto[] topicDtoList = 
            this.meetingService_.getAgendasWithDiscuss(basicId_);

        ActivityDto activityDto = 
            this.activityService_.getActivityById(eventHeaderDto.getActivityId());
        
        MeetingToTaskDto meetingToTaskDto = new MeetingToTaskDto();
        meetingToTaskDto.setBasicId(basicId_);
        meetingToTaskDto = meetingService_.getTaskCountByBasicId(meetingToTaskDto);
        meetingToTaskDto.setBasicId(basicId_);
        meetingToTaskDto.setOffset(0);
        meetingToTaskDto.setLimit(meetingToTaskDto.getRowCount());
        MeetingToTaskDto[] meetingToTaskDtos = meetingService_.getTasksByBasicId(meetingToTaskDto);
        
        this.mailService_.send(eventHeaderDto, topicDtoList, activityDto, meetingToTaskDtos, userId);
        return "";
    }

}
