/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.meeting.service.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendaDiscussionsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingAgendasDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicBodiesDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingBasicsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingRelationsDao;
import org.seasar.tugboat.gtd.contents.meeting.dao.MeetingToTasksDao;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.dxo.MeetingDxo;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussionBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasicBody;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingRelation;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingToTask;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.mailservices.service.MailService;

/**
 * meeting service impl.
 * 
 */
public class MeetingServiceImpl implements MeetingService {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(MeetingServiceImpl.class);

    /* meeting dxo */
    MeetingDxo meetingDxo_;

    public void setMeetingDxo(MeetingDxo meetingDxo) {
        this.meetingDxo_ = meetingDxo;
    }

    /* meeting basic dao */
    MeetingBasicsDao meetingBasicsDao_;

    public void setMeetingBasicsDao(MeetingBasicsDao meetingBasicsDao) {
        this.meetingBasicsDao_ = meetingBasicsDao;
    }

    /* meeting agenda Dao */
    MeetingAgendasDao meetingAgendasDao_;

    public void setMeetingAgendasDao(MeetingAgendasDao meetingAgendasDao) {
        this.meetingAgendasDao_ = meetingAgendasDao;
    }

    /* meeting agenda comment Dao */
    MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao_;

    public void setMeetingAgendaDiscussionsDao(
            MeetingAgendaDiscussionsDao meetingAgendaDiscussionsDao) {
        this.meetingAgendaDiscussionsDao_ = meetingAgendaDiscussionsDao;
    }

    /* meeting relations Dao */
    MeetingRelationsDao meetingRelationsDao_;

    public void setMeetingRelationsDao(MeetingRelationsDao meetingRelationsDao) {
        this.meetingRelationsDao_ = meetingRelationsDao;
    }

    /* meeting to task Dao */
    MeetingToTasksDao meetingToTasksDao_;

    public void setMeetingToTasksDao(MeetingToTasksDao meetingToTasksDao) {
        this.meetingToTasksDao_ = meetingToTasksDao;
    }

    /* meeting basic bodies Dao */
    MeetingBasicBodiesDao meetingBasicBodiesDao_;

    public void setMeetingBasicBodiesDao(
            MeetingBasicBodiesDao meetingBasicBodiesDao) {
        this.meetingBasicBodiesDao_ = meetingBasicBodiesDao;
    }

    /* meeting agenda bodies Dao */
    MeetingAgendaBodiesDao meetingAgendaBodiesDao_;

    public void setMeetingAgendaBodiesDao(
            MeetingAgendaBodiesDao meetingAgendaBodiesDao) {
        this.meetingAgendaBodiesDao_ = meetingAgendaBodiesDao;
    }

    /* meeting agenda discussion bodies Dao */
    MeetingAgendaDiscussionBodiesDao meetingAgendaDiscussionBodiesDao_;

    public void setMeetingAgendaDiscussionBodiesDao(
            MeetingAgendaDiscussionBodiesDao meetingAgendaDiscussionBodiesDao) {
        this.meetingAgendaDiscussionBodiesDao_ = meetingAgendaDiscussionBodiesDao;
    }

    /* task service */
    TaskService taskService_;

    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }

    /* task dxo */
    TaskDxo taskDxo_;

    public void setTaskDxo(TaskDxo taskDxo) {
        this.taskDxo_ = taskDxo;
    }

    /* taskRelationsDao Dao */
    TaskRelationsDao taskRelationsDao_;

    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }

    /* search service */
    SearchService searchService_;

    public void setSearchService(SearchService searchService) {
        this.searchService_ = searchService;
    }
    
    /* mail service */
    MailService mailService_;
    
    public void setMailService(MailService mailService) {
        this.mailService_ = mailService;
    }
    
    /* activity members dao */
    ActivityMembersDao activityMembersDao_;
    
    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }
    
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createMeetingBasicList(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto[] getMeetingBasicList(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());
        condition.setQueryStatusIds(meetingBasicDto.getQueryStatusIds());
        condition.setQueryActivityIdsIds(meetingBasicDto.getQueryActivityIds());
        condition.setQueryRegistrationIds(meetingBasicDto.getQueryRegisterIds());
        condition.setOffset(meetingBasicDto.getOffset());
        condition.setLimit(meetingBasicDto.getLimit());

        MeetingBasic[] basicList = 
            this.meetingBasicsDao_.findByUserId(condition);

        return this.meetingDxo_.convertMeetingBasicDtoList(basicList);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getDefaultCountByFolder(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getDefaultCountByFolder(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());

        MeetingBasic basic = 
            this.meetingBasicsDao_.findDefaultCountByFolder(condition);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getMeetingCountByQueryParams(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getMeetingCountByQueryParams(MeetingBasicDto meetingBasicDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(meetingBasicDto.getUserId());
        condition.setQueryStatusIds(meetingBasicDto.getQueryStatusIds());
        condition.setQueryActivityIdsIds(meetingBasicDto.getQueryActivityIds());
        condition.setQueryRegistrationIds(meetingBasicDto.getQueryRegisterIds());

        MeetingBasic basic = 
            this.meetingBasicsDao_.findCountByQueryParams(condition);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createMeetingBasic(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto getMeetingBasic(MeetingBasicDto dto) {
        MeetingBasic basic = 
            this.meetingBasicsDao_.findById(dto.getUserId(),dto.getId());

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createMeetingBasicWithTask(
     *      int, int)
     */
    public MeetingBasicDto getMeetingBasicWithTask(int id, int userId) {
        MeetingBasic[] basicList = 
            this.meetingBasicsDao_.findWithTaskById(id, userId);

        return this.meetingDxo_.convertMeetingBasicDtoWithTask(basicList);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createAgendaList(int)
     */
    public MeetingAgendaDto[] getAgendaList(int id) {
        MeetingAgenda[] agendas = 
            this.meetingAgendasDao_.findByMeetingId(id);
        
        List<Integer> list = new ArrayList<Integer>();
        for (MeetingAgenda a : agendas) {
            list.add(a.getId());
        }
        Integer[] ids = list.toArray(new Integer[list.size()]);
        
        // get discussions rows.
        if (ids != null && ids.length > 0) {
            MeetingAgenda[] commentRows = this.meetingAgendasDao_.getCommentRows(ids);
            MeetingAgenda[] taskRows = this.meetingAgendasDao_.getTaskRows(ids);
            
            for (MeetingAgenda a : agendas) {
                for (MeetingAgenda c : commentRows) {
                    if (a.getId() == c.getId()) {
                        a.setCommentRows(c.getCommentRows());
                        break;
                    }
                }
                for (MeetingAgenda t : taskRows) {
                    if (a.getId() == t.getId()) {
                        a.setTaskRows(t.getTaskRows());
                        break;
                    }
                }
            }
        }
        return this.meetingDxo_.convertAgendaDtos(agendas);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getAgendasWithDiscussionByIdAndAgendaId(int, int)
     */
    public MeetingAgendaDto[] getAgendasWithDiscussionByIdAndAgendaId(int id, int agendaId) {
        MeetingAgenda[] agendas = 
            this.meetingAgendasDao_.findWithDiscussionByMeetingIdAndAgendaId(id, agendaId);
        
        return this.meetingDxo_.convertMeetingAgendaDtoList(agendas);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#createAgendasWithDiscuss(int)
     */
    public MeetingAgendaDto[] getAgendasWithDiscuss(int id) {
        MeetingAgenda[] agendas = 
            this.meetingAgendasDao_.findWithDiscussionByMeetingId(id);

        return this.meetingDxo_.convertMeetingAgendaDtoList(agendas);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addMeeting(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto addMeeting(MeetingBasicDto dto) {
        MeetingBasic nextId = this.meetingBasicsDao_.findNewId();
        dto.setId(nextId.getId());

        MeetingBasic basic = ((MeetingBasic)dto);

        this.meetingBasicsDao_.insert(basic);
        MeetingBasicBody basicBody = new MeetingBasicBody();
        basicBody.setBody(basic.getBody());
        basicBody.setBasicId(nextId.getId());
        
        this.meetingBasicBodiesDao_.insert(basicBody);
        basic = this.meetingBasicsDao_.findById(dto.getUserId(), basic.getId());

        this.searchService_.generateMeetingIndex(basic);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateMeeting(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto)
     */
    public MeetingBasicDto updateMeeting(MeetingBasicDto dto) {
        MeetingBasic basic = ((MeetingBasic) dto);
        this.meetingBasicsDao_.update(basic);

        MeetingBasicBody basicBody = new MeetingBasicBody();
        basicBody.setBody(basic.getBody());
        basicBody.setBasicId(basic.getId());
        this.meetingBasicBodiesDao_.update(basicBody);

        basic = this.meetingBasicsDao_.findById(dto.getUserId(), basic.getId());
        searchService_.generateMeetingIndex(basic);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateStatus(int, int, int)
     */
    public MeetingBasicDto updateStatus(int id, int statusId, int userId) {
        MeetingBasic entity = new MeetingBasic();
        entity.setId(id);
        entity.setStatusId(statusId);
        entity.setPostedUserId(userId);
        
        this.meetingBasicsDao_.updateStatus(entity);
        
        MeetingBasic basic = 
            this.meetingBasicsDao_.findById(userId, id);

        return this.meetingDxo_.convertMeetingBasicDto(basic);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto, int)
     */
    public MeetingAgendaDto addAgenda(MeetingAgendaDto dto, int id) {
        MeetingAgenda nextId = this.meetingAgendasDao_.findNewId();
        dto.setId(nextId.getId());

        MeetingAgenda agenda = ((MeetingAgenda) dto);

        MeetingRelation relations = new MeetingRelation();
        relations.setBasicId(id);
        relations.setAgendaId(nextId.getId());

        this.meetingAgendasDao_.insert(agenda);
        this.meetingRelationsDao_.insert(relations);

        MeetingAgendaBody agendaBody = new MeetingAgendaBody();
        agendaBody.setAgendaId(nextId.getId());
        agendaBody.setBody(agenda.getBody());
        this.meetingAgendaBodiesDao_.insert(agendaBody);

        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        searchService_.generateMeetingAgendaIndex(agenda);

        return this.meetingDxo_.convertMeetingAgendaDto(agenda);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public MeetingAgendaDto updateAgenda(MeetingAgendaDto dto) {
        MeetingAgenda agenda = ((MeetingAgenda) dto);
        this.meetingAgendasDao_.update(agenda);

        MeetingAgendaBody agendaBody = new MeetingAgendaBody();
        agendaBody.setAgendaId(agenda.getId());
        agendaBody.setBody(agenda.getBody());
        this.meetingAgendaBodiesDao_.update(agendaBody);

        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        searchService_.generateMeetingAgendaIndex(agenda);

        return this.meetingDxo_.convertMeetingAgendaDto(agenda);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#deleteAgenda(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public void deleteAgenda(MeetingAgendaDto dto) {
        MeetingAgenda agenda = ((MeetingAgenda) dto);
        this.meetingAgendasDao_.updateDelFlag(agenda);

        MeetingToTask[] toTasks = this.meetingToTasksDao_.findTasksByAgendaId(agenda.getId());
    	int[] taskIds = new int[toTasks.length];
        for (int i=0; i<toTasks.length; i++) {
        	taskIds[i]=toTasks[i].getTaskId(); 
        }
        //When task is trashed, status is todo and Classification is TRASH.
    	this.taskService_.updateClassificationId(
            Classification.TRASH, TaskRelation.STATUS_TODO, taskIds, agenda.getModifiedUserId());
        
        agenda = this.meetingAgendasDao_.findAgendaRelationById(agenda.getId());
        agenda.setBody("");
        this.searchService_.generateMeetingAgendaIndex(agenda);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#addAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto)
     */
    public MeetingAgendaDiscussionDto addAgendaDiscussion(
            MeetingAgendaDiscussionDto dto,
            TaskDto taskDto,
            TaskRelationDto taskRelationsDto,
            MeetingToTaskDto toTaskDto) {
        
        MeetingAgendaDiscussion agendaId = 
            this.meetingAgendaDiscussionsDao_.findNewId();
        dto.setId(agendaId.getId());

        MeetingAgendaDiscussion discussion = 
            this.meetingDxo_.convertMeetingAgendaDiscussion(dto);
        this.meetingAgendaDiscussionsDao_.insert(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            toTaskDto.setDiscussionId(discussion.getId());
            this.addTask(taskDto, taskRelationsDto, toTaskDto);
            
        } else {
            MeetingAgendaDiscussionBody discussionBody = new MeetingAgendaDiscussionBody();
            discussionBody.setDiscussionId(discussion.getId());
            discussionBody.setAgendaId(discussion.getAgendaId());
            discussionBody.setBody(discussion.getBody());
            this.meetingAgendaDiscussionBodiesDao_.insert(discussionBody);
        }

        discussion = 
            this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion.getId());
        MeetingAgendaDiscussionDto discussionDto = 
            this.meetingDxo_.convertMeetingAgendaDiscussionDto(discussion);
        
        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussionDto.setBody(taskDto.getTitle());
            discussionDto.setBodyConvertedTextile(taskDto.getTitle());
        } else {
            searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
        return discussionDto;
    }

    /**
     * add task.
     * 
     * @param taskDto
     * @param taskRelationsDto
     * @param toTaskDto
     */
    void addTask(TaskDto taskDto, TaskRelationDto taskRelationsDto, MeetingToTaskDto toTaskDto) {
        TaskBodyDto bodyDto = new TaskBodyDto();
        bodyDto.setBody(taskDto.getDescriptionTextile());

        int taskId = 
            this.taskService_.addTask(taskDto, bodyDto, taskRelationsDto);

        toTaskDto.setTaskId(taskId);
        MeetingToTask toTask = 
            this.meetingDxo_.convertMeetingToTask(toTaskDto);
        this.meetingToTasksDao_.insert(toTask);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#updateAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDiscussionDto,
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto )
     */
    public MeetingAgendaDiscussionDto updateAgendaDiscussion(
            MeetingAgendaDiscussionDto discussionDto,
            MeetingToTaskDto toTaskDto, TaskDto taskDto, TaskRelationDto taskRelationDto) {
        
        MeetingAgendaDiscussion discussion = 
            this.meetingDxo_.convertMeetingAgendaDiscussion(discussionDto);
        this.meetingAgendaDiscussionsDao_.update(discussion);

        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            MeetingToTask toTask = 
                this.meetingDxo_.convertMeetingToTask(toTaskDto);
            TaskRelation taskRelations = 
                this.taskRelationsDao_.findTaskByDiscussionId(toTask.getDiscussionId());

            TaskRelationDto taskRelationsDto = 
                this.taskDxo_.convertTaskDtoWithoutDescription(taskRelations);
            taskRelationsDto.setTitle(taskDto.getTitle());
            taskRelationsDto.setDescription(taskDto.getDescription());
            taskRelationsDto.setDescriptionTextile(taskDto.getDescriptionTextile());
            taskRelationsDto.setDeadline(taskDto.getDeadline());
            taskRelationsDto.setUserId(taskDto.getUserId());
            taskRelationsDto.setPersonInChargeId(taskRelationDto.getPersonInChargeId());

            TaskBodyDto taskBodyDto = new TaskBodyDto();
            try {
                BeanUtils.copyProperties(taskDto, taskRelationsDto);
                taskDto.setId(taskRelationsDto.getTaskId());

                BeanUtils.copyProperties(taskBodyDto, taskRelationsDto);
                taskBodyDto.setBody(taskRelationsDto.getDescription());

                taskRelations = this.taskService_.updateTask(taskDto, taskBodyDto, taskRelationsDto, null);

            } catch (Exception e) {
                e.printStackTrace();
                throw new SystemUnexpectedRuntimeException(e);
            }
        } else {
            MeetingAgendaDiscussionBody discussionBody = new MeetingAgendaDiscussionBody();
            discussionBody.setDiscussionId(discussion.getId());
            discussionBody.setAgendaId(discussion.getAgendaId());
            discussionBody.setBody(discussion.getBody());
            this.meetingAgendaDiscussionBodiesDao_.update(discussionBody);
        }
        discussion = 
            this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion.getId());
        discussionDto = 
            this.meetingDxo_.convertMeetingAgendaDiscussionDto(discussion);

        if (String.valueOf(
                MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussionDto.setBody(taskDto.getTitle());
            discussionDto.setBodyConvertedTextile(taskDto.getTitle());
        } else {
            searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
        return discussionDto;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#deleteAgendaDiscussion(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingAgendaDto)
     */
    public void deleteAgendaDiscussion(MeetingAgendaDiscussionDto discussionDto) {
        MeetingAgendaDiscussion discussion = 
            this.meetingDxo_.convertMeetingAgendaDiscussion(discussionDto);
        this.meetingAgendaDiscussionsDao_.updateDelFlag(discussion);
        
        if (String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {

            TaskRelation taskRelation = 
                this.taskRelationsDao_.findTaskByDiscussionId(discussionDto.getId());
            
            //When task is trashed, status is todo and Classification is TRASH.
        	this.taskService_.updateClassificationId(
                Classification.TRASH,
                TaskRelation.STATUS_TODO, 
                new int[]{taskRelation.getTaskId()},
                discussionDto.getModifiedUserId() );
        }
        discussion = 
            this.meetingAgendaDiscussionsDao_.findDiscussionRelationById(discussion.getId());

        if (!String.valueOf(MeetingAgendaDiscussion.TYPE_TODO).equals(discussion.getType())) {
            discussion.setBody("");
            this.searchService_.generateMeetingAgendaDiscussionIndex(discussion);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getTasksByBasicId(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto)
     */
    public MeetingToTaskDto[] getTasksByBasicId(MeetingToTaskDto meetingToTaskDto) {
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setBasicId(meetingToTaskDto.getBasicId());
        condition.setOffset(meetingToTaskDto.getOffset());
        condition.setLimit(meetingToTaskDto.getLimit());
        
        MeetingToTask[] toTasks = 
            this.meetingToTasksDao_.findTasksByBasicId(condition);

        return this.meetingDxo_.convertMeetingToTaskDtoList(toTasks);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getTaskCountByBasicId(
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingBasicDto.MeetingToTaskDto)
     */
    public MeetingToTaskDto getTaskCountByBasicId(MeetingToTaskDto liveMeetingToTaskDto) {
        MeetingToTask toTask = 
            this.meetingToTasksDao_.findTaskCountByBasicId(
                liveMeetingToTaskDto.getBasicId());
        
        return this.meetingDxo_.convertMeetingToTaskDto(toTask);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#getMeetingToTasksByTaskId(int)
     */
    public MeetingToTaskDto[] getMeetingToTasksByTaskId(int taskId) {
        MeetingToTask[] meetingToTasks = 
            this.meetingToTasksDao_.findByTaskId(taskId);
        
        return this.meetingDxo_.convertMeetingToTaskDtoList(meetingToTasks);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.meeting.service.MeetingService#checkBelongActivityMeeting(java.lang.String, int)
     */
    public boolean checkBelongActivityMeeting(int id, int userId) {
        MeetingBasic basic = meetingBasicsDao_.findById(userId, id);
        return basic != null;
    }
}
