/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dao;

import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;

/**
 * task comment dao.
 * 
 * @author yousuke.hara
 */
@S2Dao(bean = TaskComment.class)
public interface TaskCommentsDao {

    /**
     * select new comment id.
     * 
     * @return
     */
    int selectNewCommentId();

    /**
     * add comment.
     * 
     * @return
     */
    @Arguments("comment")
    int insert(TaskComment comment);

    /**
     * find by id.
     * 
     * @param id
     * @return
     */
    @Arguments("id")
    TaskComment findById(int id);

    /**
     * find by task id.
     * 
     * @param taskId
     * @return
     */
    @Arguments("taskId")
    TaskComment[] findByTaskId(int taskId);

    /**
     * find by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    TaskComment[] findByUserId(int userId);
}
