/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.dxo;

import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseActivityDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseClassificationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowsePersonDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseStatusDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskSelectViewDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;
import org.seasar.tugboat.gtd.contents.task.entity.Task;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseActivity;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseClassification;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePerson;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowseStatus;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskCommentBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskHistory;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskSelectView;
import org.seasar.tugboat.gtd.contents.task.entity.TaskTag;
import org.seasar.tugboat.gtd.contents.task.entity.TaskToTask;

/**
 * task dxo.
 * 
 * @author yosuke
 * 
 */
public interface TaskDxo {
    /**
     * Convert taskDto to task.
     * 
     * @param task
     * @return
     */
    Task convertTask(TaskDto taskDto);

    /**
     * convert calendar tasks to calendar task dtos.
     * 
     * @param tasks
     * @return
     */
    TaskDto[] convertTaskDtos(Task[] tasks);

    /**
     * convert calendar task to calendar task dto.
     * 
     * @param task
     * @return
     */
    TaskDto convertTaskDto(Task task);

    /**
     * convert task relations dto to task relations.
     * 
     * @param taskRelationsDto
     * @return
     */
    TaskRelation convertTaskRelations(TaskRelationDto taskRelationsDto);

    /**
     * convert task to task dto.
     * 
     * @param task
     * @return
     */
    TaskRelationDto convertTaskDtoWithoutDescription(TaskRelation task);

    /**
     * convert tasks to task dtos.
     * 
     * @param tasks
     * @return
     */
    TaskRelationDto[] convertTaskDtosWithoutDescription(TaskRelation[] tasks);

    /**
     * convert task relations to task relations dtos.
     * 
     * @param taskRelations
     * @return
     */
    TaskRelationDto[] convertTaskRelationsDtos(TaskRelation[] taskRelations);

    /**
     * convert task relation to task relations dto.
     * 
     * @param taskRelations
     * @return
     */
    TaskRelationDto convertTaskRelationsDto(TaskRelation taskRelations, TaskTag[] taskTags);

    /**
     * convert clandar task to calendr task relation dtos.
     * 
     * @param tasks
     * @return
     */
    TaskRelationDto[] convertTaskDtosIncludeTags(TaskRelation[] tasks);

    /**
     * convert task body dto to task body.
     * 
     * @param taskBodyDto
     * @return
     */
    TaskBody convertTaskBody(TaskBodyDto taskBodyDto);

    /**
     * Convert taskDto to task history.
     * 
     * @param taskRelationsDto
     * @return
     */
    TaskHistory convertTaskHistory(TaskRelationDto taskRelationsDto);

    /**
     * convert calendar task comment to calendar task comment dto.
     * 
     * @return
     */
    TaskCommentDto convertTaskCommentDto(TaskComment entity);

    /**
     * convert calendar task comments to calendar task comment dtos.
     * 
     * @param entities
     * @return
     */
    TaskCommentDto[] convertTaskCommentDtos(TaskComment[] entities);

    /**
     * convert calendar task comment dto to calendar task comment.
     * 
     * @param dto
     * @return
     */
    TaskComment convertTaskComment(TaskCommentDto dto);

    /**
     * convert task comment body dto to task comment body.
     * 
     * @param dto
     * @return
     */
    TaskCommentBody convertTaskCommentBody(TaskCommentBodyDto dto);

    /**
     * convert task to task dto to task to task.
     * 
     * @param taskToTasksDto
     * @return
     */
    TaskToTask convertTaskToTasks(TaskToTaskDto taskToTasksDto);

    /**
     * convert task to task dtos to task to tasks.
     * 
     * @param taskToTasks
     * @return
     */
    TaskToTaskDto[] convertTasktoTaskDtos(TaskToTask[] taskToTasks);

    /**
     * convert from 'task to tasks' to task.
     * 
     * @param taskToTask
     * @return
     */
    Task convertFromTaskToTaskToTask(TaskToTask taskToTask);

    /**
     * convert from 'task to tasks' to task body.
     * 
     * @param taskToTask
     * @return
     */
    TaskBody convertFromTaskToTaskToTaskBody(TaskToTask taskToTask);

    /**
     * convert from 'task to tasks' to task relation.
     * 
     * @param taskToTask
     * @return
     */
    TaskRelation convertFromTaskToTaskToTaskRelations(TaskToTask taskToTask);

    /**
     * convert from 'task to tasks' to task history.
     * 
     * @param taskToTask
     * @return
     */
    TaskHistory convertFromTaskToTaskToTaskHistory(TaskToTask taskToTask);

    /**
     * convert browse statuses to browse status dtos.
     * 
     * @param browseStatuses
     * @return
     */
    TaskBrowseStatusDto[] convertTaskBrowseStatusDtos(
            TaskBrowseStatus[] browseStatuses);

    /**
     * convert browse classifications to browse classification dtos.
     * 
     * @param browseClassifications
     * @return
     */
    TaskBrowseClassificationDto[] convertTaskBrowseClassficationDtos(
            TaskBrowseClassification[] browseClassifications);

    /**
     * convert browse activities to browse activity dtos.
     * 
     * @param browseActivities
     * @return
     */
    TaskBrowseActivityDto[] convertTaskBrowseActivityDtos(
            TaskBrowseActivity[] browseActivities);

    /**
     * convert browse persons to browse person dtos.
     * 
     * @param browsePersons
     * @return
     */
    TaskBrowsePersonDto[] convertTaskBrowsePersonDtos(
            TaskBrowsePerson[] browsePersons);

    /**
     * convert task select view to task select view dto.
     * 
     * @param taskSelectView
     * @return
     */
    TaskSelectViewDto convertTaskSelectViewDto(TaskSelectView taskSelectView);

    /**
     * convert tags to tag dtos.
     * 
     * @param tags
     * @return
     */
    TagDto[] convertTagDtos(Tag[] tags);
    
    /**
     * convert task tags to task tag dtos.
     * 
     * @param taskTags
     * @return
     */
    TaskTagDto[] convertTaskTagDtos(TaskTag[] taskTags);
}
