/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.helper;

import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;

/**
 * task helper.
 * 
 * @author yosuke
 * 
 */
public interface TaskHelper {
    /**
     * create activities json for task.
     * 
     * @param calendars
     * @return
     */
    String createActivitiesJsonForTask(ActivityDto[] activities);

    /**
     * create inbox task list.
     * 
     * @param tasks
     * @param totalRecords
     * @param currentPageNumber
     * @param recordPerPage
     * @param userId
     * @return
     * 
     */
    String createTaskListJSON(
        TaskRelationDto[] tasks,
        int totalRecords, int currentPageNumber, int recordPerPage,
        int userId);

    /**
     * create default folder tasks.
     * 
     * @param folderTasks
     * @return
     */
    String createDefaultFolderTasksJSON(TaskRelationDto folderTasks);

    /**
     * create task detail json.
     * 
     * @param taskDto
     * @param tagDto
     * @return
     */
    String createTaskDetailJSON(TaskRelationDto taskDto, TaskTagDto[] tagDto);

    /**
     * create task comments json.
     * 
     * @param userId
     * @param comments
     * @return
     */
    String createTaskCommentsJSON(int userId, TaskCommentDto[] comments);

    /**
     * create task for balloon json.
     * 
     * @param taskDto
     * @return
     */
    String createTaskForBalloonJSON(TaskRelationDto taskDto);

    /**
     * create task to tasks and trackbacks json.
     * 
     * @param taskToTaskDtos
     * @param meetingToTaskDtos
     * @return
     */
    String createTaskToTasksAndTrackbacksJSON(
        TaskToTaskDto[] taskToTaskDtos, MeetingToTaskDto[] meetingToTaskDtos);

    /**
     * create before view statuses json.
     * 
     * @param viewStatuses
     * @return
     */
    String createBeforeViewStatusesJSON(TaskViewStateRelationDto viewStatuses, int userId);

    /**
     * create tags list html.
     * 
     * @param tagDto
     * @return
     */
    String createTagsListHTML(TagDto[] tagDto);

    /**
     * create task tags json.
     * 
     * @param taskTags
     * @return
     */
    String createTaskTagsJSON(TaskTagDto[] taskTags);

    /**
     * create own tags jons.
     * 
     * @param taskTags
     * @return
     */
    String createOwnTagsJSON(TaskTagDto[] taskTags);
    
    /**
     * create due list json.
     * @param taskRelations
     * @return
     */
    String createDueListJSON(TaskRelationDto[] taskRelations);
}
