/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.helper.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;

/**
 * task helper impl.
 * 
 * @author yosuke
 * 
 */
public class TaskHelperImpl implements TaskHelper {

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createActivitiesJsonForTask(
     *      org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto[])
     */
    public String createActivitiesJsonForTask(ActivityDto[] activities) {
        if (activities == null || activities.length == 0) return null;

        List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
        for (ActivityDto c : activities) {
            Map<String, String> map = new HashMap<String, String>();
            map.put("id", String.valueOf(c.getId()));
            map.put("value", c.getName());
            map.put("color", c.getColorValue());
            listJson.add(map);
        }
        JSONArray jsonArray = 
            JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
        return jsonArray.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskListJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationsByActivityDto[],
     *      int, int, int, int)
     */
    public String createTaskListJSON(
        TaskRelationDto[] tasks,
        int totalRecords, int currentPageNumber, int recordPerPage,
        int userId) {
        
        if (tasks == null || tasks.length == 0) return "";

        //
        // set record info.
        //
        List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
        for (TaskRelationDto t : tasks) {
            Map<String, Object> map = new HashMap<String, Object>();
            map.put("id", String.valueOf(t.getTaskId()));
            map.put("activityId", String.valueOf(t.getActivityId()));
            map.put("activityName", String.valueOf(t.getActivityValue()));
            map.put("activityColor", t.getColorValue());
            map.put("title", t.getTitle());
            map.put("statusId", String.valueOf(t.getStatusId()));
            map.put("classId", String.valueOf(t.getClassificationId()));
            map.put("postedBy", t.getUserLastName() + "," + t.getUserFirstName());
            map.put("postedAt", t.getRegisteredFormated());
            map.put("personInChargeLastName", t.getPersonInChargeLastName());
            map.put("personInChargeFirstName", t.getPersonInChargeFirstName());
            
            map.put("due", t.getDeadlineFormated());
            map.put("isMe", t.getPersonInChargeId() == userId);
            
            if (t.getTaskTagDtos().length > 0) {
                List<Map<?,?>> tagList = new ArrayList<Map<?,?>>();
                for (TaskTagDto tag : t.getTaskTagDtos()) {
                    Map<String, Object> jsonTag = new HashMap<String, Object>();
                    jsonTag.put("tagId", tag.getTagId());
                    jsonTag.put("tagValue", tag.getTagValue());
                    tagList.add(jsonTag);
                }
                JSONArray jsonTagArray = 
                    JSONArray.fromObject(tagList.toArray(new Map[tagList.size()]));
                map.put("tags", jsonTagArray);
            }
            listJson.add(map);
        }
        JSONArray jsonArray = 
            JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

        //
        // set page info.
        //
        Map<String, Integer> page = new HashMap<String, Integer>();
        page.put("results", new Integer(totalRecords));
        page.put("totalPages", 
            new Integer(totalRecords / recordPerPage + (totalRecords % recordPerPage == 0 ? 0 : 1)));
        page.put("currentPageNumber", new Integer(currentPageNumber));

        JSONObject json = JSONObject.fromObject(page);
        
        return "({tasks:" + jsonArray.toString() 
              + ",page:"  + json.toString() 
              + "})";
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createDefaultFolderTasksJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto)
     */
    public String createDefaultFolderTasksJSON(TaskRelationDto folderTasks) {
        if (folderTasks == null) return null;

        Map<String, String> map = new HashMap<String, String>();
        map.put("inboxTasks",      String.valueOf(folderTasks.getInboxTasks()));
        map.put("todayTasks",      String.valueOf(folderTasks.getTodayTasks()));
        map.put("weekTasks",       String.valueOf(folderTasks.getWeekTasks()));
        map.put("monthTasks",      String.valueOf(folderTasks.getMonthTasks()));
        map.put("holdTasks",       String.valueOf(folderTasks.getHoldTasks()));
        map.put("somedayTasks",    String.valueOf(folderTasks.getSomedayTasks()));
        map.put("todayToDoTasks",  String.valueOf(folderTasks.getTodayTodoTasks()));
        map.put("todayDoingTasks", String.valueOf(folderTasks.getTodayDoingTasks()));
        map.put("todayDoneTasks",  String.valueOf(folderTasks.getTodayDoneTasks()));
        map.put("archiveTasks",    String.valueOf(folderTasks.getArchiveTasks()));
        map.put("trashBoxTasks",   String.valueOf(folderTasks.getTrashBoxTasks()));
        map.put("allTasks", 
          String.valueOf(
              folderTasks.getTodayTasks()
            + folderTasks.getWeekTasks()
            + folderTasks.getMonthTasks()
            + folderTasks.getHoldTasks()
            + folderTasks.getSomedayTasks()));

        JSONObject json = JSONObject.fromObject(map);
        return json.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskCommentsJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto[])
     */
    public String createTaskCommentsJSON(int userId, TaskCommentDto[] comments) {
        if (comments == null || comments.length == 0) return null;

        List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
        try {
            for (TaskCommentDto comment : comments) {
                comment.setIsMyComment(userId == comment.getUserId());
                Map<String, Object> map = BeanUtils.describe(comment);
                map.put("isMyComment", comment.getIsMyComment());
                listJson.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

            return jsonArray.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskDetailJSON(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto,
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createTaskDetailJSON(TaskRelationDto taskDto, TaskTagDto[] tagDto) {
        if (taskDto == null) return null;
        try {
            // task detail.
            Map<?,?> map = BeanUtils.describe(taskDto);
            JSONObject json = JSONObject.fromObject(map);
            
            // task tags.
            List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
            if (tagDto != null && tagDto.length > 0) {
                for (TaskTagDto t : tagDto) {
                    listJson.add(BeanUtils.describe(t));
                }
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));

            return "{task:" + json.toString() + ",tags:" + jsonArray.toString() + "}";
            
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskForBalloonJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto)
     */
    public String createTaskForBalloonJSON(TaskRelationDto taskDto) {
        if (taskDto == null) return null;
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("id", String.valueOf(taskDto.getTaskId()));
        map.put("title", taskDto.getTitle());
        map.put("activityId", taskDto.getActivityId());
        map.put("activityColor", taskDto.getColorValue());
        map.put("activityName", taskDto.getActivityValue());
        map.put("personName", 
                  taskDto.getPersonInChargeLastName() 
                + ","
                + taskDto.getPersonInChargeFirstName());
        
        map.put("personId", taskDto.getPersonInChargeId());
        
        map.put("dueFormated", taskDto.getDeadlineFormated());
        map.put("dueValue", taskDto.getDeadline());
        
        map.put("postedBy", taskDto.getUserLastName() 
                + ","
                + taskDto.getUserFirstName());
        map.put("postedAt", taskDto.getRegisteredFormated());
        map.put("statusId", String.valueOf(taskDto.getStatusId()));
        map.put("classId", String.valueOf(taskDto.getClassificationId()));
        map.put("description", String.valueOf(taskDto.getDescription()));
        
        JSONObject json = JSONObject.fromObject(map);
        
        // set task tags.
        TaskTagDto[] taskTags = taskDto.getTaskTagDtos();
        List<Map<?,?>> listJson = new ArrayList<Map<?,?>>();
        try {
            for (TaskTagDto t : taskTags) {
                Map<?,?> mapTaskTag = BeanUtils.describe(t);
                listJson.add(mapTaskTag);
            }
            
        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
        JSONArray jsonArray = 
            JSONArray.fromObject(listJson.toArray(new Map[listJson.size()]));
        
        return "{task:" + json.toString() + ",tags:" + jsonArray.toString() + "}";
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskToTasksAndTrackbacksJSON(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto[],
     *      org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTasksDto[])
     */
    public String createTaskToTasksAndTrackbacksJSON(
        TaskToTaskDto[] taskToTaskDtos, MeetingToTaskDto[] meetingToTaskDtos) {

        if (
            (taskToTaskDtos == null    || taskToTaskDtos.length == 0) 
         && (meetingToTaskDtos == null || meetingToTaskDtos.length == 0)) return null;

        List<Map<?,?>> taskToTasksJson = new ArrayList<Map<?,?>>();
        List<Map<?,?>> meetingToTasksJson = new ArrayList<Map<?,?>>();
        
        try {
            if (taskToTaskDtos != null && taskToTaskDtos.length > 0) {
                for (TaskToTaskDto dto : taskToTaskDtos) {
                    Map<?,?> map = BeanUtils.describe(dto);
                    taskToTasksJson.add(map);
                }
            }
            if (meetingToTaskDtos != null && meetingToTaskDtos.length > 0) {
                for (MeetingToTaskDto dto : meetingToTaskDtos) {
                    Map<?,?> map = BeanUtils.describe(dto);
                    meetingToTasksJson.add(map);
                }
            }
            JSONArray jsonArrayTaskToTasks = 
                JSONArray.fromObject(taskToTasksJson.toArray(new Map[taskToTasksJson.size()]));
            
            JSONArray jsonArrayMeetingToTasks = 
                JSONArray.fromObject(meetingToTasksJson.toArray(new Map[meetingToTasksJson.size()]));
            
            return "({taskToTasks:" + jsonArrayTaskToTasks.toString() 
                + ", meetingToTasks:" + jsonArrayMeetingToTasks.toString() + "})";
            
        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createBeforeViewStatusesJSON(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto)
     */
    public String createBeforeViewStatusesJSON(TaskViewStateRelationDto viewStatuses, int userId) {
        // set view statuses.
        StringBuilder bufViewStatuses = new StringBuilder();
        bufViewStatuses.append("{");
        
        if (viewStatuses.getTaskSelectViewDto().getIsVisibleBrowser()  == 0) {
            bufViewStatuses.append("statuses:[null],")
                           .append("classes:[null],")
                           .append("activities:[null],")
                           .append("people:[" + userId +  "],");
        } else {
            JSONArray jsonStatuses = 
                JSONArray.fromObject(viewStatuses.getTaskBrowseStatuses());
            bufViewStatuses.append("statuses:").append(jsonStatuses.toString()).append(",");
            
            JSONArray jsonClasses = 
                JSONArray.fromObject(viewStatuses.getTaskBrowseClassifications());
            bufViewStatuses.append("classes:").append(jsonClasses.toString()).append(",");
            
            JSONArray jsonActivity =
                JSONArray.fromObject(viewStatuses.getTaskBrowseActivities());
            bufViewStatuses.append("activities:").append(jsonActivity.toString()).append(",");
            
            JSONArray jsonPerson =
                JSONArray.fromObject(viewStatuses.getTaskBrowsePersons());
            bufViewStatuses.append("people:").append(jsonPerson.toString()).append(",");
        }
        bufViewStatuses.append("select_class:")
                       .append(viewStatuses.getTaskSelectViewDto().getClassId())
                       .append(",");
        
        bufViewStatuses.append("select_mode:")
                       .append(viewStatuses.getTaskSelectViewDto().getModeId())
                       .append(",");
        
        bufViewStatuses.append("select_tab:")
                       .append(viewStatuses.getTaskSelectViewDto().getSelectedTabId())
                       .append(",");

        bufViewStatuses.append("is_visible_browser:")
                       .append(viewStatuses.getTaskSelectViewDto().getIsVisibleBrowser());

        bufViewStatuses.append("}");
        return bufViewStatuses.toString();
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTagsListHTML(
     *   org.seasar.tugboat.gtd.contents.task.dto.TagDto[])
     */
    public String createTagsListHTML(TagDto[] tagDto) {
        StringBuilder buf = new StringBuilder();
        buf.append(
          "<ul class=\"list_for_auto_completer\" xmlns:tgb=\"http://tugboat.seasar.org/ns/compatibility/annotations/1.0\">");        

        if (tagDto == null || tagDto.length == 0) {
        }
        else {
            for (TagDto t : tagDto) {
                buf.append("<li ")
                   .append("tgb:value=\"")
                   .append(t.getId())
                   .append("\">")
                   .append(t.getValue())
                   .append("</li>");
            }
        }
        buf.append("</ul>");
        return buf.toString();
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createTaskTagsJSON(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createTaskTagsJSON(TaskTagDto[] taskTags) {
        if (taskTags == null) return null;
        
        List<Map<?,?>> list = new ArrayList<Map<?,?>>();
        try {
            for (TaskTagDto t : taskTags) {
                Map<?,?> map = BeanUtils.describe(t);
                list.add(map);
            }
            JSONArray jsonArray = 
                JSONArray.fromObject(list.toArray(new Map[list.size()]));
            return jsonArray.toString();
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createOwnTagsJSON(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto[])
     */
    public String createOwnTagsJSON(TaskTagDto[] taskTags) {
        return this.createTaskTagsJSON(taskTags);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.helper.TaskHelper#createDueListJSON(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto[])
     */
    public String createDueListJSON(TaskRelationDto[] taskRelations) {
    	if (taskRelations== null || taskRelations.length == 0) return "";

    	try {
	    	List<Map> list = new ArrayList<Map>();
	    	
	    	DateFormat dfShort = new SimpleDateFormat("yyyyMMdd");
	    	Calendar tempCal = new GregorianCalendar();
	    	int todayNum = Integer.parseInt(dfShort.format(tempCal.getTime()));
	    	
	    	tempCal.add(Calendar.DAY_OF_MONTH, 3);
	    	int afterThreeDaysNum = Integer.parseInt(dfShort.format(tempCal.getTime()));
	    	
	    	Calendar cal = null;
	    	
	    	for (TaskRelationDto dto : taskRelations) {
	    		Map<String,Object> map = new HashMap<String, Object>();
	    		map.put("due", dto.getDeadline());
	    		map.put("due_formatted", dto.getDeadlineFormated());
	    		map.put("rows",dto.getRows());
	    		
	    		cal = new GregorianCalendar();
	    		cal.setTime(
	    		    new SimpleDateFormat("yyyyMMdd").parse(dto.getDeadline()));
	    		int dueNum = 
	    			Integer.parseInt(dfShort.format(cal.getTime()));
	    		
	    		map.put("color",
	    			todayNum          >= dueNum ? "red"    :
	    			afterThreeDaysNum >= dueNum ? "yellow"
	    					                    : "blue"
	    		);
	    		list.add(map);    		
	    	}
	    	JSONArray jsonArray = 
	            JSONArray.fromObject(list.toArray(new Map[list.size()]));
	    	return jsonArray.toString();
	    	
    	} catch (Exception e) {
    		throw new SystemUnexpectedRuntimeException(e);
    	}
    }
}
