/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.page.impl;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.meeting.dto.MeetingToTaskDto;
import org.seasar.tugboat.gtd.contents.meeting.service.MeetingService;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskToTaskDto;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;
import org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax;
import org.seasar.tugboat.gtd.contents.task.service.TaskDetailService;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.contents.task.service.TaskTagService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * task detail ajax impl.
 * 
 * @author yosuke
 * 
 */
@FormAnnotation(formId = "taskDetailAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class TaskDetailAjaxImpl implements TaskDetailAjax {
    /* task id */
    int id_;

    public int getId() {
        return id_;
    }
    public void setId(int id) {
        this.id_ = id;
    }

    /* staus id */
    int statusId_;

    public int getStatusId() {
        return statusId_;
    }
    public void setStatusId(int statusId) {
        this.statusId_ = statusId;
    }
    
    /* activity id */
    int activityId_;

    public int getActivityId() {
        return activityId_;
    }
    public void setActivityId(int activityId) {
        this.activityId_ = activityId;
    }
    
    /* classification id */
    int classificationId_;

    public int getClassificationId() {
        return classificationId_;
    }
    public void setClassificationId(int classificationId) {
        this.classificationId_ = classificationId;
    }

    /* person id */
    int personId_;

    public int getPersonId() {
        return personId_;
    }
    public void setPersonId(int personId) {
        this.personId_ = personId;
    }
    
    /* title */
    String title_;

    public String getTitle() {
        return title_;
    }
    public void setTitle(String title) {
        this.title_ = title;
    }

    /* description */
    String description_;

    public String getDescription() {
        return description_;
    }
    public void setDescription(String description) {
        this.description_ = description;
    }
    
    /* deadline */
    String deadline_;

    public String getDeadline() {
        return deadline_;
    }
    public void setDeadline(String deadline) {
        this.deadline_ = deadline;
    }

    /* comment id */
    int commentId_;

    public int getCommentId() {
        return commentId_;
    }
    public void setCommentId(int commentId) {
        this.commentId_ = commentId;
    }

    /* comment */
    String comment_;

    public String getComment() {
        return comment_;
    }
    public void setComment(String comment) {
        this.comment_ = comment;
    }
    
    /* tags */
    String tags_;
    public String getTags() {
        return tags_;
    }
    public void setTags(String tags) {
        this.tags_ = tags;
    }
    

    /* task service. */
    TaskService taskService_;
    
    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }
    
    /* task detail service. */
    TaskDetailService taskDetailService_;

    public void setTaskDetailService(TaskDetailService taskDetailService) {
        this.taskDetailService_ = taskDetailService;
    }

    /* task helper */
    TaskHelper taskHelper_;

    public void setTaskHelper(TaskHelper taskHelper) {
        this.taskHelper_ = taskHelper;
    }
    
    /* meeting service */
    MeetingService meetingService_;
    
    public void setMeetingService(MeetingService meetingService) {
        this.meetingService_ = meetingService;
    }
    
    /* task tag service */
    TaskTagService taskTagService_;
    public void setTaskTagService(TaskTagService taskTagService) {
        this.taskTagService_ = taskTagService;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#update()
     */
    public String update() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        try {
            TaskDto task = new TaskDto();
            BeanUtils.copyProperties(task, this);
            task.setUserId(sessionLoginUser.getUserId());

            TaskBodyDto body = new TaskBodyDto();
            body.setTaskId(this.id_);
            body.setBody(this.description_);

            TaskRelationDto relationsDto = new TaskRelationDto();
            relationsDto.setTaskId(this.id_);
            relationsDto.setActivityId(this.activityId_);
            relationsDto.setClassificationId(this.classificationId_);
            relationsDto.setStatusId(this.statusId_);
            relationsDto.setPersonInChargeId(this.personId_);
            relationsDto.setUserId(sessionLoginUser.getUserId());

            String[] tags = null;
            if (this.tags_ != null && !"".equals(this.tags_.trim())) {
                StringTokenizer token = new StringTokenizer(this.tags_, ",");
                Set<String> tagList = new HashSet<String>();
                while (token.hasMoreTokens()) {
                    String tag = token.nextToken();
                    if (tag == null || "".equals(tag.trim())) continue;
                    
                    tagList.add(tag.trim());
                }
                
                tags = new String[tagList.size()];
                if (! tagList.isEmpty()) {
                    Iterator<String> it = tagList.iterator();
                    int i = 0;
                    while (it.hasNext()) {
                        tags[i] = it.next();
                        i++;
                    }
                }
            }
            TaskRelationDto relations = 
                this.taskService_.updateTask(task, body, relationsDto, tags);
            
            TaskTagDto[] tagDto = 
                this.taskTagService_.findByTaskId(this.id_, sessionLoginUser.getUserId());
            
            return this.taskHelper_.createTaskDetailJSON(relations, tagDto);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#getTaskDetailJSON()
     */
    public String getTaskDetailJSON() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        TaskRelationDto taskDto = 
            this.taskService_.getTaskById(this.id_, sessionLoginUser.getUserId());

        TaskTagDto[] tagDto = 
            this.taskTagService_.findByTaskId(this.id_, sessionLoginUser.getUserId());
        
        return this.taskHelper_.createTaskDetailJSON(taskDto, tagDto);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#addComment()
     */
    public String addComment() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        try {
            TaskCommentDto commentDto = new TaskCommentDto();
            commentDto.setTaskId(this.id_);
            commentDto.setUserId(sessionLoginUser.getUserId());

            TaskCommentBodyDto bodyDto = new TaskCommentBodyDto();
            bodyDto.setTaskId(this.id_);
            bodyDto.setBody(this.comment_);

            TaskCommentDto[] comments = 
                this.taskDetailService_.addTaskComment(commentDto, bodyDto);

            return this.taskHelper_.createTaskCommentsJSON(sessionLoginUser.getUserId(), comments);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#getAllCommentsByTaskId()
     */
    public String getAllCommentsByTaskId() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        TaskCommentDto[] comments = 
            this.taskDetailService_.getTaskCommentsByTaskId(this.id_);

        return this.taskHelper_.createTaskCommentsJSON(
                sessionLoginUser.getUserId(), comments);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#getCommentByCommentId()
     */
    public String getCommentByCommentId() {
        TaskCommentDto taskComment = 
            this.taskDetailService_.getTaskCommentByCommentId(this.commentId_);

        return taskComment.toString();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#updateComment()
     */
    public String updateComment() {
        TaskCommentBodyDto body = new TaskCommentBodyDto();
        body.setTaskId(this.id_);
        body.setCommentId(this.commentId_);
        body.setBody(this.comment_);
        this.taskDetailService_.updateTaskComment(body);
        return "";
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#deleteComment()
     */
    public String deleteComment() {
        TaskCommentBodyDto body = new TaskCommentBodyDto();
        body.setTaskId(this.id_);
        body.setCommentId(this.commentId_);
        this.taskDetailService_.deleteTaskComment(body);

        return this.getAllCommentsByTaskId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#addTaskToTasks()
     */
    public String addTaskToTasks() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();

        TaskToTaskDto dto = new TaskToTaskDto();
        dto.setSourceTaskId(this.id_);
        dto.setDestinationActivityId(this.activityId_);
        dto.setDestinationTitle(this.title_);
        dto.setDestinationDescription(this.description_);
        dto.setDestinationPersonInChargeId(this.personId_);
        dto.setDestinationDue(
          this.deadline_ == null || "".equals(this.deadline_.trim()) 
          ? null : this.deadline_
        );
        
        String[] tags = null;
        if (this.tags_ != null && !"".equals(this.tags_.trim())) {
            StringTokenizer token = new StringTokenizer(this.tags_, ",");
            Set<String> tagList = new HashSet<String>();
            while (token.hasMoreTokens()) {
                String tag = token.nextToken();
                if (tag == null || "".equals(tag.trim())) continue;
                
                tagList.add(tag.trim());
            }
            
            tags = new String[tagList.size()];
            if (! tagList.isEmpty()) {
                Iterator<String> it = tagList.iterator();
                int i = 0;
                while (it.hasNext()) {
                    tags[i] = it.next();
                    i++;
                }
            }
        }
        dto.setDestinationTags(tags);
        dto.setDestinationPostedUserId(sessionLoginUser.getUserId());
        
        this.taskDetailService_.addTaskToTasks(dto);

        return this.getTaskToTasksAndTrackbacksJSON();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskDetailAjax#getTaskToTasksAndTrackbacksJSON()
     */
    public String getTaskToTasksAndTrackbacksJSON() {
        TaskToTaskDto[] taskToTaskDtos = 
            this.taskDetailService_.getTaskToTasksByTaskId(this.id_);
        
        MeetingToTaskDto[] meetingToTaskDtos = 
            this.meetingService_.getMeetingToTasksByTaskId(this.id_);

        return this.taskHelper_.createTaskToTasksAndTrackbacksJSON(
            taskToTaskDtos, meetingToTaskDtos);
    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
    
}
