/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.page.impl;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.StringTokenizer;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.helper.TaskHelper;
import org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax;
import org.seasar.tugboat.gtd.contents.task.service.TaskTagService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * task tag ajax impl.
 * 
 * @author yosuke
 * 
 */
@FormAnnotation(formId = "taskTagAjax", isAuthorizationCheck = true, isDirectAccessCheck = true)
public class TaskTagAjaxImpl implements TaskTagAjax {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(TaskTagAjaxImpl.class);
    
    /* id (task id) */
    int id_;
    
    public int getId() {
        return id_;
    }
    public void setId(int id) {
        this.id_ = id;
    }
    
    /* tags */
    String tags_;
    
    public String getTags() {
        return tags_;
    }
    public void setTags(String tags) {
        this.tags_ = tags;
    }
    
    /* keyword */
    String keyword_;
    
    public void setKeyword(String keyword) {
        this.keyword_ = keyword;
    }
    
    /* offset */
    int offset_;

    public int getOffset() {
        return offset_;
    }

    public void setOffset(int offset) {
        this.offset_ = offset;
    }

    /* limit */
    int limit_;

    public int getLimit() {
        return limit_;
    }
    public void setLimit(int limit) {
        this.limit_ = limit;
    }
    
    /* current page number */
    int currentPageNumber_ = 1;

    public int getCurrentPageNumber() {
        return currentPageNumber_;
    }
    public void setCurrentPageNumber(int currentPageNumber) {
        this.currentPageNumber_ = currentPageNumber;
    }
    
    /* params */
    String params_;
    
    public String getParams() {
        return params_;
    }
    public void setParams(String params) {
        this.params_ = params;
    }
    
    /* task tag service */
    TaskTagService taskTagService_;
    
    public void setTaskTagService(TaskTagService taskTagService) {
        this.taskTagService_ = taskTagService;
    }
    
    /* task helper */
    TaskHelper taskHelper_;
    
    public void setTaskHelper(TaskHelper taskHelper) {
        this.taskHelper_ = taskHelper;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#findByOwnTags()
     */
    public String findByOwnTags() {
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        TaskTagDto[] tags = 
            this.taskTagService_.findByUserId(sessionLoginUser.getUserId());
        
        return this.taskHelper_.createOwnTagsJSON(tags);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#findByTaskId()
     */
    public String findByTaskId() {
        return null;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#update()
     */
    public String update() {
        StringTokenizer token = new StringTokenizer(this.tags_, ",");
        Set<String> tagList = new HashSet<String>();
        while (token.hasMoreTokens()) {
            String tag = token.nextToken();
            if (tag == null || "".equals(tag.trim())) continue;
            
            tagList.add(tag.trim());
        }
        
        String[] tags = new String[tagList.size()];
        if (! tagList.isEmpty()) {
            Iterator<String> it = tagList.iterator();
            int i = 0;
            while (it.hasNext()) {
                tags[i] = it.next();
                logger_.debug("@[" + tags[i] + "]");
                i++;
            }
        }
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        TaskTagDto[] taskTags = 
            this.taskTagService_.update(this.id_, tags, sessionLoginUser.getUserId());
        
        return this.taskHelper_.createTaskTagsJSON(taskTags);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#findTags()
     */
    public String findTags() {
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#findTagsByKeyword()
     */
    public String findTagsByKeyword() {
        logger_.debug(this.toString());
        
        TagDto[] dtos = 
            this.taskTagService_.findTagsByKeyword(this.keyword_);
        
        return this.taskHelper_.createTagsListHTML(dtos);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.page.TaskTagAjax#updateByDragnDrop()
     */
    public String updateByDragnDrop() {
        if (this.params_ == null || "".equals(this.params_.trim())) return null;
        
        JSONObject jsonObject = JSONObject.fromObject(this.params_);
        int tagId = new Integer((String)jsonObject.get("tagId"));
        
        JSONArray paramsJson = jsonObject.getJSONArray("params");
        
        int[] taskIds = new int[paramsJson.size()];
        for (int i = 0; i < paramsJson.size(); i++) {
            taskIds[i] = 
                Integer.parseInt((String) paramsJson.getJSONObject(i).get("id"));
        }
        UserDto sessionLoginUser = SessionUtil.getLoginUser();
        TaskTagDto[] taskTags = 
            this.taskTagService_.update(taskIds, tagId, sessionLoginUser.getUserId());
        
        return this.taskHelper_.createTaskTagsJSON(taskTags);
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
}
