/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;

/**
 * task service.
 * 
 * @author yousuke.hara
 * 
 */
public interface TaskService {

    /**
     * add task.
     * 
     * @param task
     * @param body
     * @param relations
     * @return
     */
    int addTask(TaskDto task, TaskBodyDto body, TaskRelationDto relations);
    

    /**
     * add task with tags.
     * 
     * @param task
     * @param body
     * @param relations
     * @param tags
     * @return
     */
    int addTask(TaskDto task, TaskBodyDto body, TaskRelationDto relations, String[] tags);

    /**
     * update task.
     * 
     * @param taskDto
     * @param bodyDto
     * @param relationsDto
     * @param tags
     * @return
     */
    TaskRelationDto updateTask(TaskDto taskDto, TaskBodyDto bodyDto, TaskRelationDto relationsDto, String[] tags);

    /**
     * get default folder tasks.
     * 
     * @param userId
     * @return
     */
    TaskRelationDto getDefaultFoloderTasks(int userId);

    /**
     * get task by id.
     * 
     * @param taskId
     * @param userId
     * @return
     */
    TaskRelationDto getTaskById(int taskId, int userId);

    /**
     * get tasks by query params.
     * 
     * @param taskDto
     * @param select folder id
     * @return
     */
    TaskRelationDto[] getTasksByQueryParams(TaskDto taskDto, int selectFolderId, boolean isNotUpdateViewStatus);

    /**
     * get task count by query params.
     * 
     * @param taskDto
     * @return
     */
    int getTaskCountByQueryParams(TaskDto taskDto);

    /**
     * update classification.
     * 
     * @param classId
     * @param statusId
     * @param taskIds
     * @param user id
     * @return
     */
    int updateClassificationId(int classId, int statusId, int[] taskIds, int userId);

    /**
     * get week start date and week end date.
     * 
     * @param userId
     * @return [0:week start date],[1:week end date]
     */
    String[] getWeekStartDateAndWeekEndDate(int userId);

    /**
     * empty a trash box.
     * 
     * @param userId
     * @return
     */
    int emptyTrashBox(int userId);

    /**
     * update selected mode.
     * 
     * @param mode
     * @param userId
     * @return
     */
    void updateSelectedMode(int mode, int userId);

    /**
     * update is visible browser.
     * 
     * @param isVisible
     * @param userId
     */
    void updateIsVisibleBrowser(int isVisible, int userId);
    
    /**
     * update selected tab.
     * 
     * @param tabId
     * @param userId
     */
    void updateSelectedTab(int tabId, int userId);

    /**
     * get task view state relations.
     * 
     * @param userId
     * @return
     */
    TaskViewStateRelationDto getViewStateRelations(int userId);
    
    /**
     * check belong activity task.
     * 
     * @param id
     * @param userId
     * @return
     */
    boolean checkBelongActivityTask(int id, int userId);
    
    /**
     * get tasks by activity and user id.
     * 
     * @param activity
     * @param userId
     * @param limit
     * @return
     */
    TaskRelationDto[] getTasksByActivityAndUserId(int activity, int userId, int limit);
    
    /**
     * get due list by user id.
     * 
     * @param userId
     * @return
     */
    TaskRelationDto[] getDueListByUserId(int userId);
    
    /**
     * get tasks by due.
     * 
     * @param due
     * @param userId
     * @return
     */
    TaskRelationDto[] getTasksByDue(String due, int userId);
}
