package org.seasar.tugboat.gtd.contents.task.service;

import org.seasar.tugboat.gtd.contents.task.dto.TagDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;

/**
 * task tag service.
 * 
 * @author yosuke
 * 
 */
public interface TaskTagService {

    /**
     * update tags.
     * 
     * @param taskId
     * @param tags
     * @param userId
     * @return
     */
    TaskTagDto[] update(int taskId, String[] tags, int userId);

    /**
     * update task's tags.
     * 
     * @param taskId
     * @param tagId
     * @param userId
     * @return
     */
    TaskTagDto[] update(int[] taskId, int tagId, int userId);

    /**
     * find by user id.
     * 
     * @param userId
     * @return
     */
    TaskTagDto[] findByUserId(int userId);

    /**
     * find by task id.
     * 
     * @param taskId
     * @return
     */
    TaskTagDto[] findByTaskId(int taskId, int userId);

    /**
     * find tags by keyword.
     * 
     * @param keyword
     * @return
     */
    TagDto[] findTagsByKeyword(String keyword);

    /**
     * find tasks by has tasks.
     * 
     * @param userId
     * @param offset
     * @param limit
     * @return
     */
    TaskRelationDto[] findTasksByHasTasks(int userId, int offset, int limit);

    /**
     * find tasks by has tasks.
     * 
     * @param tagIds
     * @param userId
     * @param offset
     * @param limit
     * @return
     */
    TaskRelationDto[] findTasksByHasTasks(
        Integer[] tagIds, int userId, int offset, int limit);

    /**
     * get rows by has tasks.
     * 
     * @param userId
     * @return
     */
    int getRowsByHasTasks(int userId);

    /**
     * get rows by has tasks.
     * 
     * @param tagIds
     * @param userId
     * @return
     */
    int getRowsByHasTasks(Integer[] tagIds, int userId);
}
