/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Locale;

import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.service.SearchService;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivitiesDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.activity.dto.ActivityDto;
import org.seasar.tugboat.gtd.contents.activity.dxo.ActivityDxo;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dao.TagsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBodiesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBrowserActivitiesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBrowserClassificationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBrowserPersonsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskBrowserStatusesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskCommentsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskHistoriesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskSelectTagsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskSelectViewsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskTagsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TasksDao;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseActivityDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseClassificationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowsePersonDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBrowseStatusDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskCommentDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskSelectViewDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskViewStateRelationDto;
import org.seasar.tugboat.gtd.contents.task.dxo.TaskDxo;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;
import org.seasar.tugboat.gtd.contents.task.entity.Task;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBody;
import org.seasar.tugboat.gtd.contents.task.entity.TaskBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskHistory;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.task.entity.TaskSelectView;
import org.seasar.tugboat.gtd.contents.task.entity.TaskTag;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.mailservices.service.MailService;

/**
 * task service impl.
 * 
 * @author yousuke.hara
 * 
 */
public class TaskServiceImpl implements TaskService {
    /* LOGGER */
    Logger logger_ = Logger.getLogger(TaskServiceImpl.class);

    /* task dxo */
    TaskDxo taskDxo_;

    public void setTaskDxo(TaskDxo taskDxo) {
        this.taskDxo_ = taskDxo;
    }

    /* tasks Dao */
    TasksDao tasksDao_;

    public void setTasksDao(TasksDao tasksDao) {
        this.tasksDao_ = tasksDao;
    }

    /* task relations dao */
    TaskRelationsDao taskRelationsDao_;

    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }

    /* task body dao */
    TaskBodiesDao taskBodiesDao_;

    public void setTaskBodiesDao(TaskBodiesDao taskBodiesDao) {
        this.taskBodiesDao_ = taskBodiesDao;
    }

    /* task history Dao */
    TaskHistoriesDao taskHistoriesDao_;

    public void setTaskHistoriesDao(TaskHistoriesDao taskHistoriesDao) {
        this.taskHistoriesDao_ = taskHistoriesDao;
    }

    /* activity dao */
    ActivitiesDao activitiesDao_;

    public void setActivitiesDao(ActivitiesDao activitiesDao) {
        this.activitiesDao_ = activitiesDao;
    }

    /* search service */
    SearchService searchService_;

    public void setSearchService(SearchService searchService) {
        this.searchService_ = searchService;
    }

    /* taskSelectViewsDao */
    TaskSelectViewsDao taskSelectViewsDao_;

    public void setTaskSelectViewsDao(TaskSelectViewsDao taskSelectViewsDao) {
        this.taskSelectViewsDao_ = taskSelectViewsDao;
    }

    /* task select tags dao */
    TaskSelectTagsDao taskSelectTagsDao_;

    public void setTaskSelectTagsDao(TaskSelectTagsDao taskSelectTagsDao) {
        this.taskSelectTagsDao_ = taskSelectTagsDao;
    }

    /* task browser activities dao */
    TaskBrowserActivitiesDao taskBrowserActivitiesDao_;

    public void setTaskBrowserActivitiesDao(
            TaskBrowserActivitiesDao taskBrowserActivitiesDao) {
        this.taskBrowserActivitiesDao_ = taskBrowserActivitiesDao;
    }

    /* task browser classifications dao */
    TaskBrowserClassificationsDao taskBrowserClassificationsDao_;

    public void setTaskBrowserClassificationsDao(
            TaskBrowserClassificationsDao taskBrowserClassificationsDao) {
        this.taskBrowserClassificationsDao_ = taskBrowserClassificationsDao;
    }

    /* task browser persons dao */
    TaskBrowserPersonsDao taskBrowserPersonsDao_;

    public void setTaskBrowserPersonsDao(
            TaskBrowserPersonsDao taskBrowserPersonsDao) {
        this.taskBrowserPersonsDao_ = taskBrowserPersonsDao;
    }

    /* task browser statuses dao */
    TaskBrowserStatusesDao taskBrowserStatusesDao_;

    public void setTaskBrowserStatusesDao(
            TaskBrowserStatusesDao taskBrowserStatusesDao) {
        this.taskBrowserStatusesDao_ = taskBrowserStatusesDao;
    }

    /* task tags dao */
    TaskTagsDao taskTagsDao_;

    public void setTaskTagsDao(TaskTagsDao taskTagsDao) {
        this.taskTagsDao_ = taskTagsDao;
    }

    /* tags dao */
    TagsDao tagsDao_;

    public void setTagsDao(TagsDao tagsDao) {
        this.tagsDao_ = tagsDao;
    }

    /* activity members dao */
    ActivityMembersDao activityMembersDao_;

    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }

    /* mail service */
    MailService mailService_;

    public void setMailService(MailService mailService) {
        this.mailService_ = mailService;
    }

    /* activity dxo */
    ActivityDxo activityDxo_;

    public void setActivityDxo(ActivityDxo activityDxo) {
        this.activityDxo_ = activityDxo;
    }

    /* task comment dao */
    TaskCommentsDao taskCommentsDao_;

    public void setTaskCommentsDao(TaskCommentsDao taskCommentsDao) {
        this.taskCommentsDao_ = taskCommentsDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#addTask(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationsDto)
     */
    public int addTask(TaskDto task, TaskBodyDto body, TaskRelationDto relations) {
        return this.addTask(task, body, relations, null);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#addTask(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto,
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto,
     *      java.lang.String[])
     */
    public int addTask(TaskDto task, TaskBodyDto body, TaskRelationDto relations, String[] tags) {
        int taskId = this.tasksDao_.getNewTaskId().getId();
        task.setId(taskId);
        body.setTaskId(taskId);
        relations.setTaskId(taskId);

        // data convert.
        Task taskEntity = this.taskDxo_.convertTask(task);
        TaskBody bodyEntity = this.taskDxo_.convertTaskBody(body);
        TaskRelation relationsEntity = 
            this.taskDxo_.convertTaskRelations(relations);
        TaskHistory historyEntity = this.taskDxo_.convertTaskHistory(relations);

        // store data.
        this.tasksDao_.insert(taskEntity);
        this.taskRelationsDao_.insert(relationsEntity);
        this.taskBodiesDao_.insert(bodyEntity);
        this.taskHistoriesDao_.insert(historyEntity);

        if (tags != null && tags.length > 0) {
            this.addTaskTags(task, tags, taskId);
        }

        // add search indexes.
        TaskRelation relationsEntityForSearch = 
            this.taskRelationsDao_.findByTaskId(task.getId(), task.getUserId());

        this.searchService_.generateTaskIndex(relationsEntityForSearch);

        // send mail
        TaskRelationDto taskDto = 
            this.taskDxo_.convertTaskDtoWithoutDescription(relationsEntityForSearch);
        Activity activity = 
            this.activitiesDao_.selectActivityById(task.getActivityId());
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityId(task.getActivityId());
        ActivityDto activityDto = 
            this.activityDxo_.convertActivityDto(activity, members);

        this.mailService_.sendMail(taskDto, null, activityDto, relations.getUserId());

        return taskId;
    }

    /**
     * add task tags.
     * 
     * @param task
     * @param tags
     * @param taskNewId
     */
    void addTaskTags(TaskDto task, String[] tags, int taskNewId) {
        // add task tags.
        List<Integer> tagIds = new ArrayList<Integer>();

        for (int i = 0; i < tags.length; i++) {
            Tag tag = this.tagsDao_.findByValue(tags[i]);
            if (tag == null) {
                this.tagsDao_.insert(tags[i]);
                tag = this.tagsDao_.findByValue(tags[i]);
            }
            TaskTag taskTag = new TaskTag();
            taskTag.setTaskId(taskNewId);
            taskTag.setUserId(task.getUserId());
            taskTag.setTagId(tag.getId());

            tagIds.add(tag.getId());

            this.taskTagsDao_.insert(taskTag);
        }

        // share tags.
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityId(task.getActivityId());

        for (ActivityMember m : members) {
            if (m.getUserId() == task.getUserId()) continue;

            for (Integer id : tagIds) {
                TaskTag taskTag = new TaskTag();
                taskTag.setTaskId(taskNewId);
                taskTag.setUserId(m.getUserId());
                taskTag.setTagId(id);
                this.taskTagsDao_.insert(taskTag);
            }
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#updateTask(
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskDto,
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto,
     *   org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto,
     *   java.lang.String[])
     */
    public TaskRelationDto updateTask(TaskDto taskDto, TaskBodyDto bodyDto, TaskRelationDto relationsDto, String[] tags) {
        Task task = this.taskDxo_.convertTask(taskDto);
        TaskBody body = this.taskDxo_.convertTaskBody(bodyDto);
        TaskRelation relations = this.taskDxo_.convertTaskRelations(relationsDto);
        TaskHistory history = this.taskDxo_.convertTaskHistory(relationsDto);

        this.tasksDao_.update(task);
        this.taskRelationsDao_.update(relations);
        this.taskBodiesDao_.update(body);
        this.taskHistoriesDao_.insert(history);

        TaskRelation relationsEntityForSearch = 
            this.taskRelationsDao_.findByTaskId(task.getId(), task.getUserId());
        this.searchService_.generateTaskIndex(relationsEntityForSearch);

        // update tags.
        this.updateTaskTags(task.getId(), tags, task.getUserId());
        
        // send mail.
        TaskRelationDto taskRelationDto = 
            this.taskDxo_.convertTaskDtoWithoutDescription(relationsEntityForSearch);
        Activity activity = 
            this.activitiesDao_.selectActivityById(task.getActivityId());
        ActivityMember[] members = 
            this.activityMembersDao_.findActivityMembersByActivityId(task.getActivityId());

        ActivityDto activityDto = 
            this.activityDxo_.convertActivityDto(activity, members);
        TaskCommentDto[] taskCommentDtos;
        taskCommentDtos = getTaskCommentsByTaskId(task.getId());

        this.mailService_.sendMail(
            taskRelationDto, taskCommentDtos, activityDto, relationsDto.getUserId());

        return this.getTaskById(task.getId(), taskDto.getUserId());
    }
    
    /**
     * update task tags.
     * 
     * @param taskId
     * @param tags
     * @param userId
     */
    void updateTaskTags(int taskId, String[] tags, int userId) {
        // delete tags.
        this.taskTagsDao_.delete(taskId);
        if (tags == null || tags.length == 0) {
            return;
        }
        
        // add task tags.
        List<Integer> tagIds = new ArrayList<Integer>();
        
        for (int i = 0; i < tags.length; i++) {
            Tag tag = this.tagsDao_.findByValue(tags[i]);
            if (tag == null) {
                this.tagsDao_.insert(tags[i]);
                tag = this.tagsDao_.findByValue(tags[i]);
            }
            TaskTag taskTag = new TaskTag();
            taskTag.setTaskId(taskId);
            taskTag.setUserId(userId);
            taskTag.setTagId(tag.getId());
            
            tagIds.add(tag.getId());
            
            this.taskTagsDao_.insert(taskTag);
        }
        
        // share tags.
        TaskRelation task = 
            this.taskRelationsDao_.findByTaskId(taskId, userId);
        
        ActivityMember[] members = 
            this.activityMembersDao_.selectByActivityId(task.getActivityId());
        
        for (ActivityMember m : members) {
            if (m.getUserId() == userId) continue;
            
            for (Integer id : tagIds) {
                TaskTag taskTag = new TaskTag();
                taskTag.setTaskId(taskId);
                taskTag.setUserId(m.getUserId());
                taskTag.setTagId(id);
                this.taskTagsDao_.insert(taskTag);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getTaskById(int,
     *      int)
     */
    public TaskRelationDto getTaskById(int taskId, int userId) {
        TaskRelation task = this.taskRelationsDao_.findByTaskId(taskId, userId);
        TaskTag[] taskTags = 
            this.taskTagsDao_.findByTaskIdAndUserId(taskId, userId);

        return this.taskDxo_.convertTaskRelationsDto(task, taskTags);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getDefaultFoloderTasks(int)
     */
    public TaskRelationDto getDefaultFoloderTasks(int userId) {
        Activity[] activities = this.activitiesDao_.selectOwnActivities(userId);

        int[] activityIds = new int[activities.length];
        for (int i = 0; i < activities.length; i++) {
            activityIds[i] = activities[i].getId();
        }
        TaskRelation task = 
            this.taskRelationsDao_.getDefaultFolderTasks(userId, activityIds);

        return this.taskDxo_.convertTaskDtoWithoutDescription(task);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getTasksByQueryParams(
     *      org.seasar.tugboat.gtd.contents.task.dto.TaskDto, java.lang.String,
     *      boolean)
     */
    public TaskRelationDto[] getTasksByQueryParams(TaskDto taskDto, int selectFolderId, boolean isNotUpdateViewStatus) {
        // update view statuses.
        if (isNotUpdateViewStatus) {
            // update task view's select folder.
            TaskSelectView taskSelectView = new TaskSelectView();
            taskSelectView.setClassId(selectFolderId);
            taskSelectView.setModeId(-1);
            taskSelectView.setIsVisibleBrowser(-1);
            taskSelectView.setSelectedTabId(-1);
            taskSelectView.setUserId(taskDto.getUserId());

            this.updateSelectedView(taskSelectView);

            // update task view's browser info.
            this.updateUserSelectedBrowserInfo(
                taskDto.getUserId(), 
                taskDto.getQueryStatusIds(),
                taskDto.getQueryClassificationIds(),
                taskDto.getQueryActivityIds(),
                taskDto.getQueryPersonIds());
        }
        
        // find tasks.
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setUserId(taskDto.getUserId());
        condition.setQueryStatusIds(taskDto.getQueryStatusIds());
        condition.setQueryClassificationIds(taskDto.getQueryClassificationIds());
        condition.setQueryActivityIds(taskDto.getQueryActivityIds());
        condition.setQueryPersonIds(taskDto.getQueryPersonIds());
        condition.setQueryPersonIsNull(taskDto.getQueryPersonIsNull());
        condition.setOffset(taskDto.getOffset());
        condition.setLimit(taskDto.getLimit());

        TaskRelation[] targetTasks = 
            this.taskRelationsDao_.findTaskIdsByQueryParams(condition);
        
        List<Integer> taskIdList = new ArrayList<Integer>();
        if (targetTasks != null && targetTasks.length > 0) {
            for (TaskRelation r : targetTasks) {
                taskIdList.add(r.getTaskId());
            }
        }
        Integer[] targetTaskIds = 
            taskIdList.toArray(new Integer[taskIdList.size()]);
        
        TaskRelation[] tasks = null;
        if (targetTaskIds == null || targetTaskIds.length == 0) {
        } else {
            tasks = 
                this.taskRelationsDao_.findByQueryParams(targetTaskIds, taskDto.getUserId());
        }
        return this.taskDxo_.convertTaskDtosIncludeTags(tasks);
    }

    /**
     * update user selected classification.
     * 
     * @param selectClass
     */
    void updateSelectedView(TaskSelectView selectView) {
        if (selectView.getClassId() == -1) return;

        TaskSelectView result = 
            this.taskSelectViewsDao_.findByUserId(selectView.getUserId());

        if (result == null) this.taskSelectViewsDao_.insert(selectView);
        else                this.taskSelectViewsDao_.update(selectView);
    }

    /**
     * update user selected browser info.
     * 
     * @param userId
     * @param statusIds
     * @param classIds
     * @param activityIds
     * @param PersonIds
     */
    void updateUserSelectedBrowserInfo(int userId, Integer[] statusIds,
            Integer[] classIds, Integer[] activityIds, Integer[] personIds) {

        // update task browser status.
        this.taskBrowserStatusesDao_.delete(userId);
        if (statusIds != null && statusIds.length > 0) {
            for (Integer id : statusIds) {
                this.taskBrowserStatusesDao_.insert(id.intValue(), userId);
            }
        }

        // update task browser class.
        this.taskBrowserClassificationsDao_.delete(userId);
        if (classIds != null && classIds.length > 0) {
            for (Integer id : classIds) {
                this.taskBrowserClassificationsDao_.insert(id.intValue(),
                        userId);
            }
        }

        // update task browser activity.
        this.taskBrowserActivitiesDao_.delete(userId);
        if (activityIds != null && activityIds.length > 0) {
            for (Integer id : activityIds) {
                this.taskBrowserActivitiesDao_.insert(id.intValue(), userId);
            }
        }

        // update task browser person.
        this.taskBrowserPersonsDao_.delete(userId);
        if (personIds != null && personIds.length > 0) {
            for (Integer id : personIds) {
                this.taskBrowserPersonsDao_.insert(id.intValue(), userId);
            }
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.calendar.logic.CalendarService#getTaskCountByQueryParams(
     *      org.seasar.tugboat.gtd.contents.calendar.dto.CalendarTaskDto)
     */
    public int getTaskCountByQueryParams(TaskDto taskDto) {
        TaskBrowsePageCondition condition = new TaskBrowsePageCondition();
        condition.setUserId(taskDto.getUserId());
        condition.setQueryStatusIds(taskDto.getQueryStatusIds());
        condition.setQueryClassificationIds(taskDto.getQueryClassificationIds());
        condition.setQueryActivityIds(taskDto.getQueryActivityIds());
        condition.setQueryPersonIds(taskDto.getQueryPersonIds());
        condition.setQueryPersonIsNull(taskDto.getQueryPersonIsNull());

        return this.taskRelationsDao_.getRowsByQueryParams(condition).getRows();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#updateClassificationId(int,
     *      int, int[], int)
     */
    public int updateClassificationId(int classId, int statusId, int[] taskIds, int userId) {
        for (int taskId : taskIds) {
            this.taskRelationsDao_.updateClassification(taskId, classId, statusId);

            TaskHistory history = new TaskHistory();
            history.setTaskId(taskId);
            history.setStatusId(statusId < 1 ? 1 : statusId);
            history.setUserId(userId);
            this.taskHistoriesDao_.insert(history);

            // send mail

            TaskRelation relationsEntityForSearch = 
                this.taskRelationsDao_.findByTaskId(taskId, userId);
            this.searchService_.generateTaskIndex(relationsEntityForSearch);

            TaskRelationDto taskRelationDto = 
                this.taskDxo_.convertTaskDtoWithoutDescription(relationsEntityForSearch);
            Activity activity = 
                this.activitiesDao_.selectActivityById(taskRelationDto.getActivityId());

            ActivityMember[] members = 
                this.activityMembersDao_.findActivityMembersByActivityId(taskRelationDto.getActivityId());

            ActivityDto activityDto = 
                this.activityDxo_.convertActivityDto(activity, members);
            TaskCommentDto[] taskCommentDtos;
            taskCommentDtos = getTaskCommentsByTaskId(taskId);

            this.mailService_.sendMail(taskRelationDto, taskCommentDtos,
                    activityDto, userId);

        }
        return taskIds.length;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getWeekStartDateAndWeekEndDate(int)
     */
    public String[] getWeekStartDateAndWeekEndDate(int userId) {
        int startWeek = Calendar.SUNDAY; // @TASK - default.
        String[] result = new String[2];
        Calendar calendar = new GregorianCalendar();
        calendar.setFirstDayOfWeek(startWeek);
        calendar.set(Calendar.DAY_OF_WEEK, startWeek);
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd", Locale.US);
        result[0] = df.format(calendar.getTime());

        calendar.setFirstDayOfWeek(startWeek);
        calendar.set(Calendar.DAY_OF_WEEK, startWeek + 6 <= 7 ? startWeek + 6
                : startWeek + 6 - 7);
        result[1] = df.format(calendar.getTime());

        return result;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#emptyTrashBox(int)
     */
    public int emptyTrashBox(int userId) {
        TaskRelation[] tasks = this.taskRelationsDao_.findByClassId(Classification.TRASH, userId);

        for (TaskRelation t : tasks) {
            t.setTitle("");
            t.setDescription("");
            this.searchService_.generateTaskIndex(t);
        }
        return this.tasksDao_.updateDeleteFlagByUserId(userId);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#updateSelectedMode(int,
     *      int)
     */
    public void updateSelectedMode(int modeId, int userId) {
        TaskSelectView result = this.taskSelectViewsDao_.findByUserId(userId);

        TaskSelectView selectView = new TaskSelectView();
        selectView.setClassId(-1);
        selectView.setModeId(modeId);
        selectView.setIsVisibleBrowser(-1);
        selectView.setSelectedTabId(-1);
        selectView.setUserId(userId);

        if (result == null)
            this.taskSelectViewsDao_.insert(selectView);
        else
            this.taskSelectViewsDao_.update(selectView);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#updateIsVisibleBrowser(int,
     *      int)
     */
    public void updateIsVisibleBrowser(int isVisible, int userId) {
        TaskSelectView result = this.taskSelectViewsDao_.findByUserId(userId);

        TaskSelectView selectView = new TaskSelectView();
        selectView.setClassId(-1);
        selectView.setModeId(-1);
        selectView.setIsVisibleBrowser(isVisible);
        selectView.setSelectedTabId(-1);
        selectView.setUserId(userId);

        if (result == null)
            this.taskSelectViewsDao_.insert(selectView);
        else
            this.taskSelectViewsDao_.update(selectView);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#updateSelectedTab(int,
     *      int)
     */
    public void updateSelectedTab(int tabId, int userId) {
        TaskSelectView result = this.taskSelectViewsDao_.findByUserId(userId);

        TaskSelectView selectView = new TaskSelectView();
        selectView.setClassId(-1);
        selectView.setModeId(-1);
        selectView.setIsVisibleBrowser(-1);
        selectView.setSelectedTabId(tabId);
        selectView.setUserId(userId);

        if (result == null) this.taskSelectViewsDao_.insert(selectView);
        else                this.taskSelectViewsDao_.update(selectView);
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getViewStateRelations(int)
     */
    public TaskViewStateRelationDto getViewStateRelations(int userId) {
        TaskBrowseStatusDto[] statusDtos = 
            this.taskDxo_.convertTaskBrowseStatusDtos(
                this.taskBrowserStatusesDao_.findByUserId(userId));
        TaskBrowseClassificationDto[] classDtos = 
            this.taskDxo_.convertTaskBrowseClassficationDtos(
                this.taskBrowserClassificationsDao_.findByUserId(userId));
        TaskBrowseActivityDto[] activityDtos = 
            this.taskDxo_.convertTaskBrowseActivityDtos(
                this.taskBrowserActivitiesDao_.findByUserId(userId));
        TaskBrowsePersonDto[] personDtos = 
            this.taskDxo_.convertTaskBrowsePersonDtos(
                this.taskBrowserPersonsDao_.findByUserId(userId));
        TaskSelectViewDto selectViewDto = 
            this.taskDxo_.convertTaskSelectViewDto(
                this.taskSelectViewsDao_.findByUserId(userId));

        TaskViewStateRelationDto viewStateRelations = new TaskViewStateRelationDto();
        viewStateRelations.setTaskBrowseStatusDtos(statusDtos);
        viewStateRelations.setTaskBrowseClassificationDtos(classDtos);
        viewStateRelations.setTaskBrowseActivityDtos(activityDtos);
        viewStateRelations.setTaskBrowsePersonDto(personDtos);
        viewStateRelations.setTaskSelectViewDto(selectViewDto);

        return viewStateRelations;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskDetailService#getTaskCommentsByTaskId(int)
     */
    public TaskCommentDto[] getTaskCommentsByTaskId(int taskId) {
        TaskComment[] comments = this.taskCommentsDao_.findByTaskId(taskId);
        return this.taskDxo_.convertTaskCommentDtos(comments);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#checkBelongActivityTask(int, int)
     */
    public boolean checkBelongActivityTask(int id, int userId) {
        TaskRelation relation = taskRelationsDao_.findByTaskId(id, userId);
        return relation != null;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getTasksByActivityAndUserId(int, int, int)
     */
    public TaskRelationDto[] getTasksByActivityAndUserId(int activity, int userId, int limit) {
        TaskDto taskDto = new TaskDto();
        taskDto.setUserId(userId);
        taskDto.setQueryActivityIds(new Integer[]{activity});
        taskDto.setOffset(0);
        taskDto.setLimit(limit);
        
        return this.getTasksByQueryParams(taskDto, Classification.TASKS, false);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getDueListByUserId(int)
     */
    public TaskRelationDto[] getDueListByUserId(int userId) {
    	TaskRelation[] taskRelations = this.taskRelationsDao_.getDueListByUserId(userId);
    	return this.taskDxo_.convertTaskRelationsDtos(taskRelations);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.service.TaskService#getTasksByDue(java.lang.String, int)
     */
    public TaskRelationDto[] getTasksByDue(String due, int userId) {
    	TaskRelation[] tasks = 
    		this.taskRelationsDao_.findByDeadline(due, userId);
    	
        return this.taskDxo_.convertTaskDtosIncludeTags(tasks);
        
    	//return this.taskDxo_.convertTaskRelationsDtos(taskRelations);
    }
}
