/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.task.service.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.container.service.impl.AbstractScheduler;
import org.seasar.tugboat.gtd.contents.task.dao.TaskHistoriesDao;
import org.seasar.tugboat.gtd.contents.task.dao.TaskRelationsDao;
import org.seasar.tugboat.gtd.contents.task.dao.TasksDao;
import org.seasar.tugboat.gtd.contents.task.entity.Task;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;

/**
 * transfer tasks scheduler impl.
 * 
 * @author yosuke
 * 
 */
public class TaskTransferSchedulerImpl extends AbstractScheduler {
    Logger logger_ = Logger.getLogger(TaskTransferSchedulerImpl.class);

    TimerTask task_;

    /* tasks dao */
    TasksDao tasksDao_;

    public void setTasksDao(TasksDao tasksDao) {
        this.tasksDao_ = tasksDao;
    }

    /* task relations dao. */
    TaskRelationsDao taskRelationsDao_;

    public void setTaskRelationsDao(TaskRelationsDao taskRelationsDao) {
        this.taskRelationsDao_ = taskRelationsDao;
    }

    /* task histories dao. */
    TaskHistoriesDao taskHistoriesDao_;

    public void setTaskHistoriesDao(TaskHistoriesDao taskHistoriesDao) {
        this.taskHistoriesDao_ = taskHistoriesDao;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.impl.AbstractScheduler#init()
     */
    public void init() {
        this.updateTaskClassification();
    }

    /**
     * update task classification.
     * 
     */
    void updateTaskClassification() {
        DateFormat dfShort = new SimpleDateFormat("yyyyMMdd");
        String date = dfShort.format(new Date());

        // a. now date == due date => today.folder.
        Task[] updateTargetIds = this.tasksDao_.getUpdateTargetByDate(date);
        if (updateTargetIds != null && updateTargetIds.length > 0) {
            List<Integer> updateList = new ArrayList<Integer>();
            for (Task t : updateTargetIds) {
                updateList.add(new Integer(t.getId()));
            }
            Integer[] ids = updateList.toArray(new Integer[updateList.size()]);
            this.taskRelationsDao_.updateClassificationsForTransferTasks(ids);
        }

        // b. today's done and hold's done => archives.folder.
        TaskRelation[] moveUpdateTargetIds = 
            this.taskRelationsDao_.getUpdateTargetByDateForMoveToArchive(date + "000000");
        
        if (moveUpdateTargetIds != null && moveUpdateTargetIds.length > 0) {
            List<Integer> updateList = new ArrayList<Integer>();
            for (TaskRelation t : moveUpdateTargetIds) {
                updateList.add(new Integer(t.getTaskId()));
            }
            Integer[] ids = updateList.toArray(new Integer[updateList.size()]);
            this.taskRelationsDao_.updateClassificationsMoveToArchive(ids);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.impl.AbstractScheduler#start()
     */
    public void start() {
        this.startDateTime_ = 
            new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date());
        this.stopDateTime_ = "";

        try {
            if (this.task_ == null) {
                this.task_ = new JobTimerTask();
            }
            Timer timer = new Timer();
            timer.schedule(this.task_, 0L, 1000L);

            this.isStarted_ = true;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.container.logic.impl.AbstractScheduler#stop()
     */
    public void stop() {
        this.startDateTime_ = "";
        this.stopDateTime_ = 
            new SimpleDateFormat("yyyy/MM/dd HH:mm:ss").format(new Date());

        if (this.task_ == null)
            return;

        try {
            this.task_.cancel();
            this.task_ = null;

            this.isStarted_ = false;

            logger_.debug("¥n***** TaskTransferSchedulerImpl.Stopped! *****¥n");
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /**
     * job timer task.
     * 
     * @author yosuke
     * 
     */
    class JobTimerTask extends TimerTask {
        long preRunDateTime = 0;

        public JobTimerTask() {
        }

        /*
         * (non-Javadoc)
         * 
         * @see java.util.TimerTask#run()
         */
        public void run() {
            long dateTime = Long.parseLong(new SimpleDateFormat("yyyyMMddHHmm")
                    .format(new Date()));
            int hour = Integer.parseInt(new SimpleDateFormat("HH")
                    .format(new Date()));
            int minute = Integer.parseInt(new SimpleDateFormat("mm")
                    .format(new Date()));

            /* used schedule. */
            ScheduleInfo scheduleInfo = null;
            if (startSchedule_ == null || startSchedule_.size() == 0)
                return;

            for (int index = 0; index < startSchedule_.size(); index++) {
                scheduleInfo = (ScheduleInfo) startSchedule_.get(index);

                if (scheduleInfo.getHour() == hour
                        && scheduleInfo.getMinute() == minute
                        && dateTime != preRunDateTime) {
                    preRunDateTime = dateTime;
                    logger_.debug("*** TaskTransferSchedulerImpl.JobTimerTask#run[ hour:["
                      + hour + "],minute:[" + minute + "] ]");

                    updateTaskClassification();
                    break;
                }
            }
        }
    }
}
