/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.timeline.dxo.impl;

import org.apache.commons.lang.StringUtils;
import org.seasar.tugboat.gtd.commons.util.CalendarUtil;
import org.seasar.tugboat.gtd.commons.util.SystemConfiguration;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.task.entity.TaskComment;
import org.seasar.tugboat.gtd.contents.task.entity.TaskRelation;
import org.seasar.tugboat.gtd.contents.timeline.dxo.TimelineDxo;
import org.seasar.tugboat.gtd.contents.timeline.entity.xml.Data;

/**
 * Timeline dxo implement.
 * 
 * @author yousuke.hara
 * 
 */
public class TimelineDxoImpl implements TimelineDxo {
    static final int FUNC_TYPE_TASK = 1;
    static final int FUNC_TYPE_MEETING = 2;
    static final int FUNC_TYPE_ACTIVITY = 3;
    static final int TYPE_TODO = 1;
    static final int TYPE_DOING = 2;
    static final int TYPE_DONE = 3;
    static final int TYPE_COMMENT = -1;
    static final int TYPE_MEETING_BASIC = 11;
    static final int TYPE_MEETING_AGENDA = 12;
    static final int TYPE_MEETING_COMMENT = 13;
    static final int TYPE_MEETING_TASK = 14;

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.timeline.dxo.TimelineDxo#convertTimelineDatas(
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskRelations[],
     *      org.seasar.tugboat.gtd.contents.task.entity.TaskComment[],
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic[],
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgenda[],
     *      org.seasar.tugboat.gtd.contents.meeting.entity.MeetingAgendaDiscussion[])
     */
    public Data convertTimelineDatas(TaskRelation[] tasks,
            TaskComment[] taskComments, MeetingBasic[] meetingBasics,
            MeetingAgenda[] agendas, MeetingAgendaDiscussion[] discussions) {

        Data timelineData = new Data();
        String timezone = "GMT+0900"; // TASK

        // task.
        if (tasks != null && tasks.length > 0) {
            this.addTasks(timelineData, timezone, tasks);
        }
        // task comment.
        if (taskComments != null && taskComments.length > 0) {
            this.addTaskComments(timelineData, timezone, taskComments);
        }
        // meeting basics.
        if (meetingBasics != null && meetingBasics.length > 0) {
            this.addMeetingBasics(timelineData, timezone, meetingBasics);
        }
        // meeting agendas.
        if (agendas != null && agendas.length > 0) {
            this.addMeetingAgenda(timelineData, timezone, agendas);
        }
        // meeting agenda discussions.
        if (discussions != null && discussions.length > 0) {
            this.addMeetingAgendaDiscussions(timelineData, timezone,
                    discussions);
        }
        return timelineData;
    }

    /**
     * add tasks.
     * 
     * @param timelineData
     * @param timezone
     */
    void addTasks(Data timelineData, String timezone, TaskRelation[] tasks) {
        for (TaskRelation t : tasks) {
            org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event timelineEvent = 
                new org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event();

            String taskIconPath = 
                SystemConfiguration.getValue("tugboat.gtd.timeline_icon_path")
                + this.getIconImageFileName(FUNC_TYPE_TASK, t.getStatusId());

            timelineEvent.setTitle(t.getTitle());
            timelineEvent
                    .setContent(
                              "<img src=\""
                            + SystemConfiguration.getValue("tugboat.gtd.activity_icon_path")
                            + t.getColorValue()
                            + ".gif\" border=\"0\" style=\"float:left; padding 3px;\" />"
                            + t.getDescription());
            timelineEvent.setStart(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setEnd(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setLink("gtdTaskDetail.html?id=" + t.getTaskId());
            timelineEvent.setIcon(taskIconPath);
            timelineEvent.setColor("#" + t.getColorValue());
            timelineEvent.setIsDuration(false);
            timelineData.addEvent(timelineEvent);
        }
    }

    /**
     * add task comments.
     * 
     * @param timelineData
     * @param timezone
     * @param taskComments
     */
    void addTaskComments(Data timelineData, String timezone,
            TaskComment[] taskComments) {
        for (TaskComment t : taskComments) {
            org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event timelineEvent = 
                new org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event();

            String taskIconPath = 
                SystemConfiguration.getValue("tugboat.gtd.timeline_icon_path")
                    + this.getIconImageFileName(FUNC_TYPE_TASK, -1);

            timelineEvent.setTitle(StringUtils.abbreviate(t.getComment(), 20));

            timelineEvent
                    .setContent(
                              "<img src=\""
                            + SystemConfiguration
                                    .getValue("tugboat.gtd.activity_icon_path")
                            + t.getColorValue()
                            + ".gif\" border=\"0\" style=\"float:left; padding 3px;\" />"
                            + t.getComment());

            timelineEvent.setStart(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setEnd(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setLink("gtdTaskDetail.html?id=" + t.getTaskId());
            timelineEvent.setIcon(taskIconPath);
            timelineEvent.setColor("#" + t.getColorValue());
            timelineEvent.setIsDuration(false);
            timelineData.addEvent(timelineEvent);
        }
    }

    /**
     * add meeting basics.
     * 
     * @param timelineData
     * @param timezone
     * @param meetingBasics
     */
    void addMeetingBasics(Data timelineData, String timezone,
            MeetingBasic[] meetingBasics) {
        for (MeetingBasic t : meetingBasics) {
            org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event timelineEvent = 
                new org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event();

            String taskIconPath = 
                SystemConfiguration.getValue("tugboat.gtd.timeline_icon_path")
                    + this.getIconImageFileName(FUNC_TYPE_MEETING,
                            TYPE_MEETING_BASIC);

            timelineEvent.setTitle(t.getTitle());
            timelineEvent
                    .setContent(
                              "<img src=\""
                            + SystemConfiguration.getValue("tugboat.gtd.activity_icon_path")
                            + t.getColor()
                            + ".gif\" border=\"0\" style=\"float:left; padding 3px;\" />"
                            + t.getBody());

            timelineEvent.setStart(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setEnd(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setLink("gtdMeetingDetail.html?id=" + t.getId());
            timelineEvent.setIcon(taskIconPath);
            timelineEvent.setColor("#" + t.getColor());
            timelineEvent.setIsDuration(false);
            timelineData.addEvent(timelineEvent);
        }
    }

    /**
     * add meeting agendas.
     * 
     * @param timelineData
     * @param timezone
     * @param meetingAgendas
     */
    void addMeetingAgenda(Data timelineData, String timezone,
            MeetingAgenda[] meetingAgendas) {
        for (MeetingAgenda t : meetingAgendas) {
            org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event timelineEvent = 
                new org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event();

            String taskIconPath = 
                SystemConfiguration.getValue("tugboat.gtd.timeline_icon_path")
                    + this.getIconImageFileName(FUNC_TYPE_MEETING, TYPE_MEETING_AGENDA);

            timelineEvent.setTitle(StringUtils.abbreviate(t.getBody(), 20));
            timelineEvent
                    .setContent(
                              "<img src=\""
                            + SystemConfiguration
                                    .getValue("tugboat.gtd.activity_icon_path")
                            + t.getColorValue()
                            + ".gif\" border=\"0\" style=\"float:left; padding 3px;\" />"
                            + t.getBody());

            timelineEvent.setStart(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setEnd(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setLink("gtdMeetingDetail.html?id=" + t.getMeetingBasicId());
            timelineEvent.setIcon(taskIconPath);
            timelineEvent.setColor("#" + t.getColorValue());
            timelineEvent.setIsDuration(false);
            timelineData.addEvent(timelineEvent);
        }
    }

    /**
     * add meeting agenda discussions.
     * 
     * @param timelineData
     * @param timezone
     * @param discussions
     */
    void addMeetingAgendaDiscussions(Data timelineData, String timezone,
            MeetingAgendaDiscussion[] discussions) {
        for (MeetingAgendaDiscussion t : discussions) {
            org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event timelineEvent = 
                new org.seasar.tugboat.gtd.contents.timeline.entity.xml.Event();

            String taskIconPath = SystemConfiguration
                    .getValue("tugboat.gtd.timeline_icon_path")
                    + this.getIconImageFileName(
                        FUNC_TYPE_MEETING,
                        (Integer.parseInt(t.getType()) == MeetingAgendaDiscussion.TYPE_COMMENT 
                        ? TYPE_MEETING_COMMENT
                        : TYPE_MEETING_TASK));

            timelineEvent.setTitle(t.getBody() == null 
                ? t.getTaskTitle()
                : StringUtils.abbreviate(t.getBody(), 20));

            timelineEvent
                    .setContent(
                              "<img src=\""
                            + SystemConfiguration.getValue("tugboat.gtd.activity_icon_path")
                            + t.getColorValue()
                            + ".gif\" border=\"0\" style=\"float:left; padding 3px;\" />"
                            + (t.getBody() == null ? t.getTaskDescription() : t.getBody()));

            timelineEvent.setStart(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setEnd(
                CalendarUtil.dateFormattedCalendar(t.getRegistered(), "HH:mm:ss", timezone, false));

            timelineEvent.setLink("gtdMeetingDetail.html?id=" + t.getMeetingBasicId());
            timelineEvent.setIcon(taskIconPath);
            timelineEvent.setColor("#" + t.getColorValue());
            timelineEvent.setIsDuration(false);

            timelineData.addEvent(timelineEvent);
        }
    }

    /**
     * get icon image file name.
     * 
     * @param functionType
     * @param classificationId
     * @return
     */
    String getIconImageFileName(int functionType, int type) {
        final String TASK_TODO = "task_todo.gif";
        final String TASK_DOING = "task_doing.gif";
        final String TASK_DONE = "task_done.gif";
        final String TASK_COMMENT = "task_comment.gif";

        final String MEETING_BASIC = "meeting_new.gif";
        final String MEETING_AGENDA = "agenda.gif";
        final String MEETING_COMMENT = "meeting_comment.gif";
        final String MEETING_TASK = "meeting_task.gif";

        String result = "";
        if      (functionType == FUNC_TYPE_TASK    && type == TYPE_TODO)            result = TASK_TODO;
        else if (functionType == FUNC_TYPE_TASK    && type == TYPE_DOING)           result = TASK_DOING;
        else if (functionType == FUNC_TYPE_TASK    && type == TYPE_DONE)            result = TASK_DONE;
        else if (functionType == FUNC_TYPE_TASK    && type == TYPE_COMMENT)         result = TASK_COMMENT;
        else if (functionType == FUNC_TYPE_MEETING && type == TYPE_MEETING_BASIC)   result = MEETING_BASIC;
        else if (functionType == FUNC_TYPE_MEETING && type == TYPE_MEETING_AGENDA)  result = MEETING_AGENDA;
        else if (functionType == FUNC_TYPE_MEETING && type == TYPE_MEETING_COMMENT) result = MEETING_COMMENT;
        else if (functionType == FUNC_TYPE_MEETING && type == TYPE_MEETING_TASK)    result = MEETING_TASK;
        return result;
    }
}
