/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package org.seasar.tugboat.gtd.contents.timeline.entity.xml;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;

/**
 * <b>Data</b> is generated from xmlTimeline.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="data">
 *       <oneOrMore>
 *         <ref name="event"/>
 *       </oneOrMore>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="data"&gt;
 *       &lt;oneOrMore&gt;
 *         &lt;ref name="event"/&gt;
 *       &lt;/oneOrMore&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version xmlTimeline.rng (Mon Dec 18 18:35:27 JST 2006)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
public class Data implements java.io.Serializable, Cloneable, IRContentClassHandler, IRNode {
	static final long serialVersionUID = 1L;
	
    // List<Event>
    private java.util.List event_ = new java.util.ArrayList();
    private StringBuffer saxBuffer_;
    private IRNode parentRNode_;

    /**
     * Creates a <code>Data</code>.
     *
     */
    public Data() {
    }

    /**
     * Creates a <code>Data</code>.
     *
     * @param source
     */
    public Data(Data source) {
        setup(source);
    }

    /**
     * Creates a <code>Data</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Data(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Data</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Data(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Data</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Data(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Data</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Data</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Data</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Data</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Data</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Data</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Data(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Data</code> by the Data <code>source</code>.
     *
     * @param source
     */
    public void setup(Data source) {
        int size;
        this.event_.clear();
        size = source.event_.size();
        for (int i = 0;i < size;i++) {
            addEvent((Event)source.getEvent(i).clone());
        }
    }

    /**
     * Initializes the <code>Data</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Data</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Data</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        event_.clear();
        while (true) {
            if (Event.isMatch(stack)) {
                addEvent(new Event(stack));
            } else {
                break;
            }
        }
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Data((Data)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("data");
        int size;
        size = this.event_.size();
        for (int i = 0;i < size;i++) {
            Event value = (Event)this.event_.get(i);
            value.makeElement(element);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Data</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Data</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Data</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Data</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Data</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Data</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the Event property <b>event</b>.
     *
     * @return Event[]
     */
    public Event[] getEvent() {
        Event[] array = new Event[event_.size()];
        return ((Event[])event_.toArray(array));
    }

    /**
     * Sets the Event property <b>event</b>.
     *
     * @param event
     */
    public void setEvent(Event[] event) {
        this.event_.clear();
        for (int i = 0;i < event.length;i++) {
            addEvent(event[i]);
        }
        for (int i = 0;i < event.length;i++) {
            event[i].rSetParentRNode(this);
        }
    }

    /**
     * Sets the Event property <b>event</b>.
     *
     * @param event
     */
    public void setEvent(Event event) {
        this.event_.clear();
        addEvent(event);
        if (event != null) {
            event.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Event property <b>event</b>.
     *
     * @param event
     */
    public void addEvent(Event event) {
        this.event_.add(event);
        if (event != null) {
            event.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Event property <b>event</b>.
     *
     * @param event
     */
    public void addEvent(Event[] event) {
        for (int i = 0;i < event.length;i++) {
            addEvent(event[i]);
        }
        for (int i = 0;i < event.length;i++) {
            event[i].rSetParentRNode(this);
        }
    }

    /**
     * Gets number of the Event property <b>event</b>.
     *
     * @return int
     */
    public int sizeEvent() {
        return (event_.size());
    }

    /**
     * Gets the Event property <b>event</b> by index.
     *
     * @param index
     * @return Event
     */
    public Event getEvent(int index) {
        return ((Event)event_.get(index));
    }

    /**
     * Sets the Event property <b>event</b> by index.
     *
     * @param index
     * @param event
     */
    public void setEvent(int index, Event event) {
        this.event_.set(index, event);
        if (event != null) {
            event.rSetParentRNode(this);
        }
    }

    /**
     * Adds the Event property <b>event</b> by index.
     *
     * @param index
     * @param event
     */
    public void addEvent(int index, Event event) {
        this.event_.add(index, event);
        if (event != null) {
            event.rSetParentRNode(this);
        }
    }

    /**
     * Remove the Event property <b>event</b> by index.
     *
     * @param index
     */
    public void removeEvent(int index) {
        this.event_.remove(index);
    }

    /**
     * Remove the Event property <b>event</b> by object.
     *
     * @param event
     */
    public void removeEvent(Event event) {
        this.event_.remove(event);
    }

    /**
     * Clear the Event property <b>event</b>.
     *
     */
    public void clearEvent() {
        this.event_.clear();
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<data");
        buffer.append(">");
        size = this.event_.size();
        for (int i = 0;i < size;i++) {
            Event value = (Event)this.event_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.append("</data>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<data");
        buffer.write(">");
        size = this.event_.size();
        for (int i = 0;i < size;i++) {
            Event value = (Event)this.event_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.write("</data>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<data");
        buffer.print(">");
        size = this.event_.size();
        for (int i = 0;i < size;i++) {
            Event value = (Event)this.event_.get(i);
            value.makeTextElement(buffer);
        }
        buffer.print("</data>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Generates SAX events for ContentHandler.
     *
     * @param handler
     * @exception SAXException
     */
    public void makeDocument(ContentHandler handler) throws SAXException {
        handler.startDocument();
        makeElement(handler);
        handler.endDocument();
    }

    /**
     * Generates SAX events for ContentHandler.
     *
     * @param handler
     * @exception SAXException
     */
    public void makeElement(ContentHandler handler) throws SAXException {
        int size;
        String string;
        String slotQName;
        AttributesImpl attrs = new AttributesImpl();
        handler.startElement("", "data", "data", attrs);
        size = this.event_.size();
        for (int i = 0;i < size;i++) {
            Event value = (Event)this.event_.get(i);
            value.makeElement(handler);
        }
        handler.endElement("", "data", "data");
    }

    /**
     * Generates SAX events for ContentHandler.
     *
     * @param attrs
     * @exception SAXException
     */
    public void makeAttribute(AttributesImpl attrs) throws SAXException {
    }

    /**
     * Event handler for InitElement during object construction.
     *
     * @param attrs
     */
    public void initElement(Attributes attrs) {
        String string;
    }

    /**
     * Event handler for declarePrefixMapping during object construction.
     *
     * @param prefix
     * @param uri
     */
    public void declarePrefixMapping(String prefix, String uri) {
    }

    /**
     * Event handler for startElement during object construction.
     *
     * @param namespaceURI
     * @param localName
     * @param qName
     * @param attrs
     * @return IRContentClassHandler
     */
    public IRContentClassHandler startElement(String namespaceURI, String localName, String qName, Attributes attrs) {
        if ("event".equals(qName)) {
            Event ref = new Event();
            ref.initElement(attrs);
            addEvent(ref);
            return (ref);
        }
        return (this);
    }

    /**
     * Event handler for endElement during object construction.
     *
     * @param namespaceURI
     * @param localName
     * @param qName
     * @return IRContentClassHandler
     */
    public IRContentClassHandler endElement(String namespaceURI, String localName, String qName) {
        String string;
        return ((IRContentClassHandler)rGetParentRNode());
    }

    /**
     * Event handler for characters during object construction.
     *
     * @param ch
     * @param start
     * @param length
     */
    public void characters(char[] ch, int start, int length) {
        if (saxBuffer_ != null) {
            saxBuffer_.append(ch, start, length);
        }
    }

    /**
     * Event handler for processngInstruction during object construction.
     *
     * @param target
     * @param value
     */
    public void processingInstruction(String target, String value) {
    }

    /**
     * Gets the IRNode property <b>parentRNode</b>.
     *
     * @return IRNode
     */
    public IRNode rGetParentRNode() {
        return (parentRNode_);
    }

    /**
     * Sets the IRNode property <b>parentRNode</b>.
     *
     * @param parentRNode
     */
    public void rSetParentRNode(IRNode parentRNode) {
        this.parentRNode_ = parentRNode;
    }

    /**
     * Gets child RNodes.
     *
     * @return IRNode[]
     */
    public IRNode[] rGetRNodes() {
        java.util.List classNodes = new java.util.ArrayList();
        classNodes.addAll(event_);
        IRNode[] nodes = new IRNode[classNodes.size()];
        return ((IRNode[])classNodes.toArray(nodes));
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Data</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "data")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!Event.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        while (true) {
            if (!Event.isMatchHungry(target)) {
                break;
            }
            $match$ = true;
        }
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Data</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Data</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
