/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.dao;

import org.seasar.dao.annotation.tiger.Arguments;
import org.seasar.dao.annotation.tiger.S2Dao;
import org.seasar.tugboat.gtd.contents.user.entity.User;
import org.seasar.tugboat.gtd.contents.user.entity.UserBrowsePageCondition;

/**
 * User Register Dao
 * 
 * @author yousuke.hara
 * 
 */
@S2Dao(bean = User.class)
public interface UsersDao {

    /**
     * Get User Id
     * 
     * @return
     */
    User getUserId();

    /**
     * Add User
     * 
     * @param dtp
     * @return
     */
    @Arguments("user")
    int insert(User user);

    /**
     * Update User
     * user obj need id
     * @param dto 
     * @return
     */
    @Arguments("user")
    int update(User user);

    /**
     * Get User
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    User findUserById(int userId);

    /**
     * find users by keyword.
     * 
     * @param keyword
     * @return
     */
    @Arguments("keyword")
    User[] findUsersByKeyword(String keyword);

    /**
     * find users by activity id and keyword.
     * 
     * @param activityId
     * @param keyword
     * @return
     */
    @Arguments( { "activityId", "keyword" })
    User[] findUsersByActivityIdAndKeyword(int activityId, String keyword);

    /**
     * find by login id.
     * deleted user is not target
     * 
     * @param loginId
     * @return
     */
    @Arguments("loginId")
    User findByLoginId(String loginId);
    
    /**
     * get all users.
     * 
     * @return
     */
    @Arguments("condition")
    User[] getAllUser(UserBrowsePageCondition condition);
    
    /**
     * update delete flag by user id.
     * 
     * @param userId
     * @return
     */
    @Arguments("userId")
    int updateDeleteFlagByUserId(int userId);
    
    /**
     * update delete flag by user.
     * 
     * @param user
     * @return
     */
    @Arguments("user")
    int updateDeleteFlagByUser(User user);
    
    /**
     * find user by login id and password.
     * 
     * @param loginId
     * @param password
     * @return
     */
    @Arguments({"loginId","password"})
    User findUserByLoginIdAndPassword(String loginId, String password);
    
    /**
     * find admin user by login id and password.
     * 
     * @param loginId
     * @param password
     * @return
     */
    @Arguments({"loginId","password"})
    User findAdminUserByLoginIdAndPassword(String loginId, String password);

    /**
     * count user
     * @return
     */
    int getCount();
    
}
