/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.dxo.impl;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BeanUtils;
import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.dxo.UserDxo;
import org.seasar.tugboat.gtd.contents.user.entity.User;
import org.seasar.tugboat.gtd.contents.user.entity.UserRole;

/**
 * user dxo impl.
 * 
 * @author yosuke
 * 
 */
public class UserDxoImpl implements UserDxo {

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.dxo.UserDxo#convertUser(
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public User convertUser(UserDto userDto) {
        if (userDto == null) return null;
        try {
            User user = new User();
            BeanUtils.copyProperties(user, userDto);
            return user;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.dxo.UserDxo#convertUserDto(
     *      org.seasar.tugboat.gtd.contents.user.entity.User)
     */
    public UserDto convertUserDto(User user) {
        if (user == null) return null;
        try {
            UserDto userDto = new UserDto();
            BeanUtils.copyProperties(userDto, user);
            return userDto;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.dxo.UserDxo#convertUserDtos(
     *      org.seasar.tugboat.gtd.contents.user.entity.User)
     */
    public UserDto[] convertUserDtos(User[] users) {
        if (users == null) return null;
        try {
            List<UserDto> list = new ArrayList<UserDto>();
            for (User user : users) {
                UserDto userDto = new UserDto();
                BeanUtils.copyProperties(userDto, user);
                list.add(userDto);
            }
            return list.toArray(new UserDto[list.size()]);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.dxo.UserDxo#convertUsers(
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto[])
     */
    public User[] convertUsers(UserDto[] userDtos) {
        if (userDtos == null) return null;
        try {
            List<User> list = new ArrayList<User>();
            for (UserDto dto : userDtos) {
                User user = new User();
                BeanUtils.copyProperties(user, dto);
                list.add(user);
            }
            return list.toArray(new User[list.size()]);
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.dxo.UserDxo#convertUserRole(
     *   org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public UserRole convertUserRole(UserDto userDto) {
        if (userDto == null) return null;
        try {
            UserRole role = new UserRole();
            role.setUserId(userDto.getUserId());
            role.setRoleId(userDto.getRoleId());
            return role;
        } catch (Exception e) {
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
}
