/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.page.impl;

import net.sf.json.JSONObject;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.apache.commons.validator.EmailValidator;
import org.seasar.tugboat.gtd.commons.util.SessionUtil;
import org.seasar.tugboat.gtd.container.annotation.FormAnnotation;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.page.UserAjax;
import org.seasar.tugboat.gtd.contents.user.service.UserService;

/**
 * user ajax implement.
 *
 */
@FormAnnotation(
        formId = "userAjax", description = "userAjax", 
        isAuthorizationCheck = true, isDirectAccessCheck = false, isAdminTool = false, errorPage = "error")
public class UserAjaxImpl implements UserAjax {
    /* user service */
    UserService userService_;

    public void setUserService(UserService userService) {
        this.userService_ = userService;
    }

    /* email */
    String email;

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    /* firstName */
    String firstName;

    public String getFirstName() {
        return firstName;
    }

    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    /* lastName */
    String lastName;

    public String getLastName() {
        return lastName;
    }

    public void setLastName(String lastName) {
        this.lastName = lastName;
    }

    /* password */
    String password;

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAjax#get()
     */
    public String get() {
        UserDto user = SessionUtil.getLoginUser();
        JSONObject jUser = JSONObject.fromObject(user);
        jUser.remove("password");
        jUser.remove("passwordDigest");
        return jUser.toString();
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.page.UserAjax#update()
     */
    public String update() {
        if(!EmailValidator.getInstance().isValid(email)) {
            return "email";
        }
        UserDto user = SessionUtil.getLoginUser();
        user.setEmail(email);
        user.setFirstName(firstName);
        user.setLastName(lastName);
        user.setPassword(password);
        userService_.updateUserInfo(user);
        return get();
    }
    
    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

}
