/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.service;

import java.io.IOException;
import java.io.InputStream;

import org.seasar.tugboat.gtd.contents.user.dto.UserAdminMessageDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;

/**
 * user service.
 *
 */
public interface UserService {
    /**
     * add from excel file.
     * 
     * @param userXls
     * @param modifiedUser
     * @return
     * @throws IOException
     */
    public UserAdminMessageDto[] addFromExcelFile(InputStream userXls, UserDto modifiedUser) throws IOException ;
    
    public UserDto[] getUsers(int offset, int limit);
    /**
     * add User
     * @param user UserDto(userId will be ignored)
     * @return added UserDto with userId.
     * When loginId already exists, return null
     */
    public UserDto addUser(UserDto user);
    
    /**
     * update User information
     * @param user
     * @return
     */
    public int update(UserDto user);
    
    /**
     * search User by UserID
     * @param userId
     * @return
     */
    public UserDto getUserById(int userId);
    
    /**
     * delete User
     * @param user
     * @return deleted User count
     */
    public int delete(UserDto user);
    /**
     * get new UserID
     * @return
     */
    public UserDto getUserId();
    /**
     * delete User by userId
     * @param userId
     * @return deleted User count
     */
    public int deleteByUserId(int userId);
    
    /**
     * delete User by LoginID
     * @param loginId
     * @return deleted User count
     */
    public int deleteByLoginId(String loginId);
    
    /**
     * update User.
     * role cannot update.
     * 
     * @param user
     * @return
     */
    public int updateUserInfo(UserDto user);
    
    /**
     * get user count
     */
    public int getCount();
}
