/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.contents.user.service.impl;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivitiesDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityMembersDao;
import org.seasar.tugboat.gtd.contents.activity.dao.ActivityRelationsDao;
import org.seasar.tugboat.gtd.contents.activity.entity.Activity;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityMember;
import org.seasar.tugboat.gtd.contents.activity.entity.ActivityRelation;
import org.seasar.tugboat.gtd.contents.common.entity.Role;
import org.seasar.tugboat.gtd.contents.user.dao.UserRolesDao;
import org.seasar.tugboat.gtd.contents.user.dao.UsersDao;
import org.seasar.tugboat.gtd.contents.user.dto.UserAdminMessageDto;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.contents.user.dxo.UserDxo;
import org.seasar.tugboat.gtd.contents.user.entity.User;
import org.seasar.tugboat.gtd.contents.user.entity.UserBrowsePageCondition;
import org.seasar.tugboat.gtd.contents.user.entity.UserRole;
import org.seasar.tugboat.gtd.contents.user.service.UserService;
import org.seasar.tugboat.gtd.webservices.service.FeedTokenService;

/**
 * user service implement.
 *
 */
public class UserServiceImpl implements UserService {
    public static final String ACTIVITY_NAME         = "Personal";
    public static final String ACTIVITY_DESCRIPTION  = "";
    public static final int ACTIVITY_RELATIONS_COLOR = 1; // RED
    public static final int ACTIVITY_MEMBERS_ROLEID  = 1; // Admin
    public static final int USER_ROLEID              = 2; // User

    /* users dao */
    UsersDao usersDao_;
    public void setUsersDao(UsersDao usersDao) {
        this.usersDao_ = usersDao;
    }
    
    /* activities dao */
    ActivitiesDao activitiesDao_;
    public void setActivitiesDao(ActivitiesDao activitiesDao) {
        this.activitiesDao_ = activitiesDao;
    }
    
    /* activity relations dao */
    ActivityRelationsDao activityRelationsDao_;
    public void setActivityRelationsDao(ActivityRelationsDao activityRelationsDao) {
        this.activityRelationsDao_ = activityRelationsDao;
    }
    
    /* activity members dao */
    ActivityMembersDao activityMembersDao_;
    public void setActivityMembersDao(ActivityMembersDao activityMembersDao) {
        this.activityMembersDao_ = activityMembersDao;
    }
    
    /* user role dao */
    UserRolesDao userRolesDao_;
    public void setUserRolesDao(UserRolesDao userRolesDao) {
        this.userRolesDao_ = userRolesDao;
    }
    
    /* user dxo */
    UserDxo userDxo_;
    public void setUserDxo(UserDxo userDxo) {
        this.userDxo_ = userDxo;
    }
    
    /* feed token service */
    FeedTokenService feedTokenService_;
    public void setFeedTokenService(FeedTokenService feedTokenService) {
        this.feedTokenService_ = feedTokenService;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#addXls(java.io.InputStream,
     *      org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public UserAdminMessageDto[] addFromExcelFile(InputStream userXls,
            UserDto modifiedUser) throws IOException {

        List<UserAdminMessageDto> lstMessage = new ArrayList<UserAdminMessageDto>();
        int success = 0;

        HSSFWorkbook workbook = new HSSFWorkbook(userXls);

        for (int i = 0; i < workbook.getNumberOfSheets(); ++i) {
            HSSFSheet sheet = workbook.getSheetAt(i);
            int rows = sheet.getLastRowNum();

            for (int row = 0; row < rows; row++) {
                HSSFRow roww = sheet.getRow(row + 1);
                User user = usersDao_.getUserId();
                int roleId;

                // create users table
                try {
                    user.setLoginId(this.getString(roww.getCell((short) 0))); // login_id(int)
                    user.setPassword(this.getString(roww.getCell((short) 1))); // password(String)
                    user.setEmail(this.getString(roww.getCell((short) 2))); // email(String)
                    user.setLastName(this.getString(roww.getCell((short) 3))); // last_name(String)
                    user.setFirstName(this.getString(roww.getCell((short) 4))); // first_name(String)
                    user.setModifiedUserId(modifiedUser.getUserId());
                    roleId = this.getInt(roww.getCell((short) 5));

                } catch (Exception e) {
                    UserAdminMessageDto message = new UserAdminMessageDto();
                    message.setLine(row + 2);
                    message.setCode(UserAdminMessageDto.FORMAT);
                    lstMessage.add(message);
                    
                    System.err.println("xls file syntax error. (row=" + (row + 2) + ")");
                    e.printStackTrace();
                    continue;
                }

                if (this.hasLoginId(user.getLoginId())) {
                    UserAdminMessageDto message = new UserAdminMessageDto();
                    message.setLine(row + 2);
                    message.setCode(UserAdminMessageDto.EXIST);
                    message.setUser(user.getLoginId());
                    lstMessage.add(message);
                    continue;
                }

                success++;
                this.usersDao_.insert(user);
                this.userRolesDao_.add(user, roleId);
                this.addPersonalActivity(user);
            }
        }
        return lstMessage.toArray(new UserAdminMessageDto[lstMessage.size()]);
    }

    /**
     * create "Personal" activity to target User<br>
     * - activity color = RED
     * - activity name = "Personal"
     * 
     * @param user
     */
    void addPersonalActivity(User user) {
        // create activities table
        Activity activity = activitiesDao_.selectNewId();
        activity.setName(ACTIVITY_NAME);
        activity.setDescription(ACTIVITY_DESCRIPTION);
        activity.setModifiedUserId(user.getModifiedUserId());
        activitiesDao_.insert(activity);

        // create activity_members table
        ActivityMember activityMember = new ActivityMember();
        activityMember.setActivityId(activity.getId());
        activityMember.setUserId(user.getUserId());
        activityMember.setRoleId(ACTIVITY_MEMBERS_ROLEID);
        activityMember.setModifiedUserId(user.getModifiedUserId());

        activityMembersDao_.insert(activityMember);

        // create activity_relations
        ActivityRelation activityRelation = new ActivityRelation();
        activityRelation.setActivityId(activity.getId());
        activityRelation.setColorId(ACTIVITY_RELATIONS_COLOR);
        activityRelation.setUserId(user.getUserId());

        activityRelationsDao_.insert(activityRelation);
        
        // create feed token.
        this.feedTokenService_.create(activity.getId(), user.getUserId());
    }

    /**
     * when loginId already exists, return true
     * 
     * @param loginId
     *            loginId
     * @return when loginId already exists, return true
     */
    boolean hasLoginId(String loginId) {
        return null != usersDao_.findByLoginId(loginId);
    }

    /**
     * 
     * @param cell
     * @return
     */
    int getInt(HSSFCell cell) {
        switch (cell.getCellType()) {
        case HSSFCell.CELL_TYPE_BLANK:
            return -1;
        case HSSFCell.CELL_TYPE_STRING:
            return Integer.parseInt(cell.getStringCellValue());
        case HSSFCell.CELL_TYPE_NUMERIC:
            return (int) cell.getNumericCellValue();
        default:
            return -1;
        }
    }

    /**
     * 
     * @param cell
     * @return
     */
    String getString(HSSFCell cell) {
        switch (cell.getCellType()) {
        case HSSFCell.CELL_TYPE_BLANK:
            return "";
        case HSSFCell.CELL_TYPE_STRING:
            return cell.getStringCellValue();
        case HSSFCell.CELL_TYPE_NUMERIC:
            return Double.toString(cell.getNumericCellValue());
        default:
            return null;
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#getUsers()
     */
    public UserDto[] getUsers(int offset, int limit) {
        UserBrowsePageCondition condition = new UserBrowsePageCondition();
        condition.setOffset(offset);
        condition.setLimit(limit);
        
        User[] users = this.usersDao_.getAllUser(condition);
        UserDto[] userDtos =
            this.userDxo_.convertUserDtos(users);
        return userDtos;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#getCount()
     */
    public int getCount() {
        return usersDao_.getCount();
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#addUser(org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public UserDto addUser(UserDto userDto) {
        if (usersDao_.findByLoginId(userDto.getLoginId()) != null) {
            // LoginID is already exist.
            return null;
        }

        int id = getUserId().getUserId();

        User user = userDxo_.convertUserDto(userDto);
        UserRole role = userDxo_.convertUserRole(userDto);
        user.setUserId(id);
        usersDao_.insert(user);
        userRolesDao_.add(user, role.getRoleId());
        UserDto reloadUserDto = 
            userDxo_.convertUserDto(usersDao_.findUserById(id));
        Role userRole = userRolesDao_.findUserRoleByUserId(id);
        reloadUserDto.setRoleId(userRole.getId());
        reloadUserDto.setRoleName(userRole.getName());

        addPersonalActivity(user);

        return reloadUserDto;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#delete(org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public int delete(UserDto userDto) {
        User user = userDxo_.convertUserDto(userDto);
        return usersDao_.updateDeleteFlagByUser(user);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#deleteByUserId(int)
     */
    public int deleteByUserId(int userId) {
        return usersDao_.updateDeleteFlagByUserId(userId);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#getUserById(int)
     */
    public UserDto getUserById(int userId) {
        User user = usersDao_.findUserById(userId);
        UserDto userDto = userDxo_.convertUserDto(user);
        Role role = userRolesDao_.findUserRoleByUserId(userId);
        if (user == null) {
            return null;
        }
        userDto.setRoleId(role.getId());
        userDto.setRoleName(role.getName());
        return userDto;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#update(org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public int update(UserDto userDto) {
        User user = userDxo_.convertUserDto(userDto);
        UserRole role = userDxo_.convertUserRole(userDto);
        userRolesDao_.update(user, role.getRoleId());
        return usersDao_.update(user);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#getUserId()
     */
    public UserDto getUserId() {
        User user = usersDao_.getUserId();
        return userDxo_.convertUserDto(user);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#deleteByLoginId(java.lang.String)
     */
    public int deleteByLoginId(String loginId) {
        User user = usersDao_.findByLoginId(loginId);
        if (user == null)
            return 0;
        return usersDao_.updateDeleteFlagByUser(user);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.user.service.UserService#updateUserInfo(org.seasar.tugboat.gtd.contents.user.dto.UserDto)
     */
    public int updateUserInfo(UserDto userDto) {
        User user = userDxo_.convertUserDto(userDto);
        return usersDao_.update(user);
    }

}
