/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.feeds.atom;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.Namespace;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;

import com.sun.syndication.feed.module.Module;
import com.sun.syndication.io.ModuleGenerator;

/**
 * task item module generator implement.
 * 
 * @author yosuke
 * 
 */
public class TaskDetailItemModuleGeneratorImpl implements ModuleGenerator {
    static final Namespace NS = 
        Namespace.getNamespace(TaskHeaderFeedModule.NS_TUGBOAT, TaskHeaderFeedModule.URI);
    static final Set<Namespace> NAMESPACES;
    static {
        Set<Namespace> nss = new HashSet<Namespace>();
        nss.add(NS);
        NAMESPACES = Collections.unmodifiableSet(nss);
    }
    
    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#generate(com.sun.syndication.feed.module.Module, org.jdom.Element)
     */
    public void generate(Module module, Element element) {
        TaskDetailItemModule m = (TaskDetailItemModule)module;
        
        Element detailElement = new Element(TaskDetailItemModule.ELEMENT_DETAIL, NS);
        
        // status element.
        Element statusElement = new Element(TaskDetailItemModule.ELEMENT_STATUS, NS);
        List<Attribute> statusAttributeList = new ArrayList<Attribute>();
        Attribute statusIdAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_ID, String.valueOf(m.getStatusId()));
        
        statusAttributeList.add(statusIdAttribute);
        statusElement.setAttributes(statusAttributeList);
        
        // activity element.
        Element activityElement = new Element(TaskDetailItemModule.ELEMENT_ACTIVITY, NS);
        List<Attribute> activityAttributeList = new ArrayList<Attribute>();
        Attribute activityIdAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_ID, String.valueOf(m.getActivityId()));
        activityAttributeList.add(activityIdAttribute);
        
        Attribute activityNameAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_NAME, String.valueOf(m.getActivityName()));
        activityAttributeList.add(activityNameAttribute);

        Attribute activityColorAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_COLOR, m.getActivityColor());
        activityAttributeList.add(activityColorAttribute);

        activityElement.setAttributes(activityAttributeList);
        
        // classification element.
        Element classificationElement = new Element(TaskDetailItemModule.ELEMENT_CLASSIFICATION, NS);
        List<Attribute> classificationAttributeList = new ArrayList<Attribute>();
        Attribute classIdAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_ID, String.valueOf(m.getClassificationId()));
        classificationAttributeList.add(classIdAttribute);
        
        classificationElement.setAttributes(classificationAttributeList);
        
        // person in charge element.
        Element personInChargeElement = new Element(TaskDetailItemModule.ELEMENT_PERSON_IN_CHARGE, NS);
        List<Attribute> personInChargeAttributeList = new ArrayList<Attribute>();
        
        Attribute personInChargeIdAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_ID, String.valueOf(m.getPersonInChargeId()));
        personInChargeAttributeList.add(personInChargeIdAttribute);
        
        Attribute personInChargeFirstNameAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_FIRST_NAME, m.getPersonInChargeFirstName());
        personInChargeAttributeList.add(personInChargeFirstNameAttribute);
            
        Attribute personInChargeLastNameAttribute = 
            new Attribute(TaskDetailItemModule.ATRB_LAST_NAME, m.getPersonInChargeLastName());
        personInChargeAttributeList.add(personInChargeLastNameAttribute);
        
        personInChargeElement.setAttributes(personInChargeAttributeList);
        
        //
        detailElement.addContent(statusElement);
        detailElement.addContent(activityElement);
        detailElement.addContent(classificationElement);
        detailElement.addContent(personInChargeElement);
        
        // tags.
        if (m.getTags() != null && m.getTags().length > 0) {
            for (TaskTagDto t : m.getTags()) {
                Element tag = new Element(TaskDetailItemModule.ELEMENT_TAG, NS);
                List<Attribute> tagAttributeList = new ArrayList<Attribute>();
                
                Attribute tagId = 
                    new Attribute(TaskDetailItemModule.ATRB_ID, String.valueOf(t.getTagId()));
                tagAttributeList.add(tagId);
                
                Attribute tagValue = 
                    new Attribute(TaskDetailItemModule.ATRB_NAME, t.getTagValue());
                tagAttributeList.add(tagValue);
                
                tag.setAttributes(tagAttributeList);
                detailElement.addContent(tag);
            }
        }
        element.addContent(detailElement);
    }

    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#getNamespaceUri()
     */
    public String getNamespaceUri() {
        return TaskDetailItemModule.URI;
    }

    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#getNamespaces()
     */
    public Set<Namespace> getNamespaces() {
        return NAMESPACES;
    }

}
