/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.feeds.atom;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.jdom.Attribute;
import org.jdom.Element;
import org.jdom.Namespace;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;

import com.sun.syndication.feed.module.Module;
import com.sun.syndication.io.ModuleGenerator;

/**
 * task feed module generator implment.
 * 
 * @author yosuke
 * 
 */
public class TaskHeaderFeedModuleGeneratorImpl implements ModuleGenerator {
    static final Namespace NS = 
        Namespace.getNamespace(TaskHeaderFeedModule.NS_TUGBOAT, TaskHeaderFeedModule.URI);
    static final Set<Namespace> NAMESPACES;
    static {
        Set<Namespace> nss = new HashSet<Namespace>();
        nss.add(NS);
        NAMESPACES = Collections.unmodifiableSet(nss);
    }
    
    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#generate(com.sun.syndication.feed.module.Module, org.jdom.Element)
     */
    public void generate(Module module, Element element) {
        if (!(module instanceof TaskHeaderFeedModule)) {
            return;
        }
        TaskHeaderFeedModule m = (TaskHeaderFeedModule)module;
        
        // user element.
        Element userElement = new Element(TaskHeaderFeedModule.ELEMENT_USER, NS);
        
        Element userIdElement = new Element(TaskHeaderFeedModule.ELEMENT_CREATED, NS);
        userIdElement.addContent(m.getCreated());
        userElement.addContent(userIdElement);
        
        Element nounceElement = new Element(TaskHeaderFeedModule.ELEMENT_NOUNCE, NS);
        nounceElement.addContent(m.getNonce());
        userElement.addContent(nounceElement);
        
        element.addContent(userElement);
        
        // header element.
        Element headerElement = new Element(TaskHeaderFeedModule.ELEMENT_HEADER, NS);
        
        if (m.getClassifications() != null && m.getClassifications().length > 0) {
            for (Classification c : m.getClassifications()) {
                Element folderElement = new Element(TaskHeaderFeedModule.ELEMENT_FOLDER, NS);
                List<Attribute> folderAtributeList = new ArrayList<Attribute>();
                
                Attribute folderId = 
                    new Attribute(TaskHeaderFeedModule.ATRB_ID, String.valueOf(c.getId()));
                folderAtributeList.add(folderId);
                
                Attribute folderName = 
                    new Attribute(
                        TaskHeaderFeedModule.ATRB_NAME, c.getValue());
                folderAtributeList.add(folderName);
                
                Attribute folderNumberOfCount =
                    new Attribute(
                        TaskHeaderFeedModule.ATRB_NUMBER_OF_COUNT, String.valueOf(c.getNumberOfCount()));
                folderAtributeList.add(folderNumberOfCount);
                
                folderElement.setAttributes(folderAtributeList);
                headerElement.addContent(folderElement);
            }    
        }
        if (m.getTags() != null && m.getTags().length > 0) {
            for (Tag t : m.getTags()) {
                Element tagElement = new Element(TaskHeaderFeedModule.ELEMENT_TAG, NS);
                List<Attribute> tagAttributeList = new ArrayList<Attribute>();
                
                Attribute tagId = 
                    new Attribute(
                        TaskHeaderFeedModule.ATRB_ID, String.valueOf(t.getId()));
                tagAttributeList.add(tagId);
                
                Attribute tagName =
                    new Attribute(
                        TaskHeaderFeedModule.ATRB_NAME, t.getValue());
                tagAttributeList.add(tagName);
                
                Attribute tagNumberOfCount = 
                    new Attribute(
                        TaskHeaderFeedModule.ATRB_NUMBER_OF_COUNT, String.valueOf(t.getNumberOfCount()));
                tagAttributeList.add(tagNumberOfCount);
                
                tagElement.setAttributes(tagAttributeList);
                headerElement.addContent(tagElement);
            }
        }
        element.addContent(headerElement);
    }

    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#getNamespaceUri()
     */
    public String getNamespaceUri() {
        return TaskHeaderFeedModule.URI;
    }

    /*
     * (non-Javadoc)
     * @see com.sun.syndication.io.ModuleGenerator#getNamespaces()
     */
    public Set<Namespace> getNamespaces() {
        return NAMESPACES;
    }

}
