/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.feeds.rss;

import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.seasar.tugboat.gtd.container.exception.SystemUnexpectedRuntimeException;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;

import com.sun.syndication.feed.rss.Channel;
import com.sun.syndication.feed.rss.Description;
import com.sun.syndication.feed.rss.Item;
import com.sun.syndication.io.WireFeedOutput;

/**
 * task rss publish service implements.
 * @author yosuke
 *
 */
public class TaskRssPublishServiceImpl implements TaskRssPublishService {

    /* task service */
    TaskService taskService_;
    
    public void setTaskService(TaskService taskService){
        this.taskService_ = taskService;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.rss.service.TaskRssPublishService#publish(int, int, limit)
     */
    public String publish(int activityId, int userId, int limit) {
        TaskRelationDto[] tasks = 
            this.taskService_.getTasksByActivityAndUserId(activityId, userId, limit);
        
        return this.getRSS(tasks);
    }

    /**
     * get rss.
     * 
     * @param tasks
     * @return
     */
    String getRSS(TaskRelationDto[] tasks) {
        Channel channel = new Channel();  
        channel.setFeedType("rss_2.0");
        channel.setLanguage("ja-jp");                
        channel.setTitle("tugboat.GTD - Task Feeds");         
        channel.setDescription("tugboat.GTD - Task Feeds");  
        channel.setLink("http://tugboat-gtd.sandbox.seasar.org/users/1");                                           
        channel.setPubDate(new Date()); 
        
        List<Item> items = new ArrayList<Item>();
        if (tasks == null || tasks.length == 9) {
        } else {
            for (TaskRelationDto t : tasks) {
                Item item1 = new Item();
                item1.setTitle(t.getTitle());
                item1.setLink("http://10.3.1.203:8080/tugboatGTD/browse/tasks/" + t.getTaskId());
                item1.setPubDate(new Date());   
                item1.setAuthor(t.getUserLastName() + "," + t.getUserFirstName());
                
                Description desc = new Description();
                desc.setValue(t.getDescription());
                item1.setDescription(desc); 
                items.add(item1); 
            }
        }
        
        // generate.
        channel.setItems(items);  
        try {
            WireFeedOutput out = new WireFeedOutput();
            
            StringWriter bw = new StringWriter();
            out.output(channel, bw);
            String result = bw.toString();
            System.out.println(result);
            return result;
            
        } catch (Exception e) {
            e.printStackTrace();
            throw new SystemUnexpectedRuntimeException(e);
        }
    }
    
}
