/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.service.impl;

import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.annotation.RestMethod;
import org.seasar.tugboat.gtd.container.constant.HttpMethod;
import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dto.TaskBodyDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskDto;
import org.seasar.tugboat.gtd.contents.task.dto.TaskRelationDto;
import org.seasar.tugboat.gtd.contents.task.service.TaskService;
import org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;
import org.seasar.tugboat.gtd.webservices.service.TaskWebService;

/**
 * task webservice impl.
 * 
 * @author yosuke
 * 
 */
public class TaskRESTWebServiceImpl extends DefaultRESTfulWebService implements TaskWebService {
    /* logger */
    Logger logger_ = Logger.getLogger(TaskRESTWebServiceImpl.class);
    
    public TaskRESTWebServiceImpl() {
    }
    
    /* task service */
    TaskService taskService_;
    
    public void setTaskService(TaskService taskService) {
        this.taskService_ = taskService;
    }
    
    /* task web service helper */
    WebServiceHelper webServiceHelper_;
    
    public void setWebServiceHelper(WebServiceHelper webServiceHelper) {
        this.webServiceHelper_ = webServiceHelper;
    }
    
    /* id (task_id) */
    int id_;
    public void setId(int id) {
        this.id_ = id;
    }
    
    /* title */
    String title_;
    public void setTitle(String title) {
        this.title_ = title;
    }
    
    /* description */
    String description_;
    public void setDescription(String description) {
        this.description_ = description;
    }
    
    /* due */
    String due_;
    public void setDue(String due) {
        this.due_ = due;
    }
    
    /* (selected) folder */
    int folder_;
    public void setSelectFolder(int folder) {
        this.folder_ = folder;
    }
    
    /* status */
    int status_;
    public void setStatus(int status) {
        this.status_ = status;
    }
    
    /* statuses */
    Integer[] statuses_;
    public void setStatuses(Integer[] statuses) {
        this.statuses_ = statuses;
    }
    public Integer[] getStatuses() {
        return (
            this.statuses_==null || (this.statuses_.length > 0 && this.statuses_[0].intValue() == 0) 
            ? null : this.statuses_);
    }
    
    /* classification */
    int classification_;
    public void setClassification(int classification) {
        this.classification_ = classification;
    }
    
    /* classes */
    Integer[] classes_;
    public void setClasses(Integer[] classes) {
        this.classes_ = classes;
    }
    public Integer[] getClasses() {
        return (
            this.classes_==null || (this.classes_.length > 0 && this.classes_[0].intValue() == 0) 
            ? null : this.classes_);
    }
    
    /* activity */
    int activity_;
    public void setActivity(int activity) {
        this.activity_ = activity;
    }
    
    /* activities */
    Integer[] activities_;
    public void setActivities(Integer[] activities) {
        this.activities_ = activities;
    }
    public Integer[] getActivities() {
        return (
            this.activities_==null || (this.activities_.length > 0 && this.activities_[0].intValue() == 0) 
            ? null : this.activities_);
    }
    
    /* person in charge */
    int personInCharge_;
    public void setPersonInCharge(int personInCharge) {
        this.personInCharge_ = personInCharge;
    }
    
    /* people */
    Integer[] people_;
    public void setPeople(Integer[] people) {
        this.people_ = people;
    }
    public Integer[] getPeople() {
        return (
            this.people_==null || (this.people_.length > 0 && this.people_[0].intValue() == 0) 
            ? null : this.people_);
    }
    
    /* tags */
    String[] tags_;
    public void setTags(String[] tags) {
        this.tags_ = tags;
    }
    
    /* offset */
    int offset_;
    public void setOffset(int offset) {
        this.offset_ = offset;
    }
    
    /* limit */
    int limit_;
    public void setLimit(int limit) {
        this.limit_ = limit;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#create()
     */
    @RestMethod(methods = HttpMethod.POST)
    public String create() {
        logger_.debug("#create:[" + this.activity_ + "," + this.title_ + "," + this.description_);
        
        TaskDto taskDto = new TaskDto();
        taskDto.setTitle(this.title_);
        taskDto.setUserId(this.userId_);

        TaskBodyDto bodyDto = new TaskBodyDto();
        bodyDto.setBody(this.description_);

        TaskRelationDto relationsDto = new TaskRelationDto();
        relationsDto.setActivityId(this.activity_);
        relationsDto.setStatusId(TaskRelationDto.STATUS_TODO);
        relationsDto.setClassificationId(Classification.INBOX);
        relationsDto.setPersonInChargeId(this.userId_);
        relationsDto.setUserId(1);
        
        int taskId = this.taskService_.addTask(taskDto, bodyDto, relationsDto);
        return String.valueOf(taskId);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#delete()
     */
    @RestMethod(methods = {HttpMethod.POST, HttpMethod.DELETE})
    public boolean delete() {
        return false;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#find()
     */
    @RestMethod(methods = HttpMethod.GET, parameters = "id")
    public String find() {
        TaskRelationDto dto = this.taskService_.getTaskById(this.id_, this.userId_);
        return this.webServiceHelper_.createContentForTask(this.acceptType_, dto);
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.contents.task.webservice.TaskWebService#findAll()
     */
    @RestMethod(methods = HttpMethod.GET)
    public String findAll() {
        logger_.debug("#findAll");
        return "[OK]:findAll";
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#findByQueryParams()
     */
    @RestMethod(
        methods = HttpMethod.GET, parameters = {"folder","statuses","classes","activities","people","offset","limit"})
    public String findByQueryParams() {
        logger_.debug(this.toString());
        
        TaskDto taskDto = new TaskDto();
        taskDto.setUserId(this.userId_);
        taskDto.setQueryStatusIds(this.getStatuses());
        taskDto.setQueryClassificationIds(this.getClasses());
        taskDto.setQueryActivityIds(this.getActivities());  
        taskDto.setQueryPersonIds(this.getPeople());  
        taskDto.setOffset(this.offset_);
        taskDto.setLimit(this.limit_);
        
        TaskRelationDto[] tasks = 
            this.taskService_.getTasksByQueryParams(taskDto, this.folder_, false);
        
        return this.webServiceHelper_.createContentForTask(this.acceptType_, tasks);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#findByTags()
     */
    @RestMethod(
        methods = HttpMethod.GET, parameters = {"tags","offset","limit"})
    public String findByTags() {
        logger_.debug("***** findByTags *****");
        logger_.debug(this.toString());
        
        return null;
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#getFolderRecords()
     */
    @RestMethod(methods = HttpMethod.GET)
    public String getFolderRecords() {
        TaskRelationDto task =this.taskService_.getDefaultFoloderTasks(this.userId_);
        
        return this.webServiceHelper_.createFolderRecords(this.acceptType_, task);
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.TaskWebService#update()
     */
    @RestMethod(methods = {HttpMethod.POST, HttpMethod.PUT})
    public boolean update() {
        return false;
    }
    
    /*
     * (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }

}
