/*
 * Copyright 2004-2007 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.tugboat.gtd.webservices.service.impl;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang.RandomStringUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.builder.ReflectionToStringBuilder;
import org.apache.commons.lang.builder.ToStringStyle;
import org.seasar.framework.log.Logger;
import org.seasar.tugboat.gtd.container.annotation.RestMethod;
import org.seasar.tugboat.gtd.container.constant.HttpMethod;
import org.seasar.tugboat.gtd.container.entity.WebServiceSecurity;
import org.seasar.tugboat.gtd.container.exception.InvalidSecureTokenRuntimeException;
import org.seasar.tugboat.gtd.container.exception.UnauthorizedRuntimeException;
import org.seasar.tugboat.gtd.contents.login.service.LoginService;
import org.seasar.tugboat.gtd.contents.login.service.WebServiceAuthService;
import org.seasar.tugboat.gtd.contents.user.dto.UserDto;
import org.seasar.tugboat.gtd.webservices.helper.WebServiceHelper;
import org.seasar.tugboat.gtd.webservices.service.DefaultRESTfulWebService;
import org.seasar.tugboat.gtd.webservices.service.WebServiceAuthorize;

/**
 * webservice authorize impl.
 * 
 * @author yosuke
 * 
 */
public class WebServiceAuthorizeImpl extends DefaultRESTfulWebService implements WebServiceAuthorize {
    Logger logger_ = Logger.getLogger(WebServiceAuthorizeImpl.class);

    /* login service */
    LoginService loginService_;
    
    public void setLoginService(LoginService loginService) {
    	this.loginService_ = loginService;
    }
    
    /* web service security service */
    WebServiceAuthService webServiceAuthService_;
    
    public void setWebServiceAuthService(WebServiceAuthService webServiceAuthService) {
    	this.webServiceAuthService_ = webServiceAuthService;
    }
    
    /* webservice helper */
    WebServiceHelper webserviceHelper_;
    
    public void setWebServiceHelper(WebServiceHelper webserviceHelper) {
        this.webserviceHelper_ = webserviceHelper;
    }
    
    /* wsse */
    String wsse_;
    public String getWsse() {
        if (this.wsse_ != null && !"".equals(this.wsse_)) return this.wsse_;
        
        StringBuilder buf = new StringBuilder();
        buf.append("UsernameToken Username=\"#{userName}\",");
        buf.append("PasswordDigest=\"#{passwordDigest}\",");
        buf.append("Nonce=\"#{nonce}\",");
        buf.append("Created=\"#{created}\"");
        
        wsse_ = buf.toString();
        wsse_ = StringUtils.replace(wsse_, "#{userName}",       this.userName_);
        wsse_ = StringUtils.replace(wsse_, "#{passwordDigest}", this.passwordDigest_);
        wsse_ = StringUtils.replace(wsse_, "#{nonce}",          this.nonce_);
        wsse_ = StringUtils.replace(wsse_, "#{created}",        this.created_);
        return wsse_;
    }
    public void setWsse(String wsse) {
        this.wsse_ = wsse;
    }
    
    /* user id */
    String userName_;
    public void setUserName(String userName) {
        this.userName_ = userName;
    }
    
    /* password digest */
    String passwordDigest_;
    public void setPasswordDigest(String passwordDigest) {
        this.passwordDigest_ = passwordDigest;
    }
    
    /* nonce */
    String nonce_;
    public void setNonce(String nonce) {
        this.nonce_ = nonce;
    }
    
    /* created at */
    String created_;
    public void setCreated(String created) {
        this.created_ = created;
    }

    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.container.service.WebServiceAuthorize#getSecureToken()
     */
    @RestMethod(methods = {HttpMethod.GET})
    public String getSecureToken() {
        DateFormat formatter = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");

        WebServiceSecurity entity = new WebServiceSecurity();
        entity.setClientAddress("http://");
        entity.setNonce(RandomStringUtils.randomAlphanumeric(20));
        entity.setCreatedAt(formatter.format(new Date()));

        this.webServiceAuthService_.add(entity);
        
        return this.webserviceHelper_.createSecureInfo(
                this.acceptType_, entity.getNonce(), entity.getCreatedAt());
    }
    
    /*
     * (non-Javadoc)
     * @see org.seasar.tugboat.gtd.webservices.service.WebServiceAuthorize#authorize(java.lang.String)
     */
    @RestMethod(methods = {HttpMethod.POST}, parameters = {"userName", "passwordDigest", "nonce", "created"})
    public int authorize() throws UnauthorizedRuntimeException, InvalidSecureTokenRuntimeException {
        
        if (this.getWsse() == null || "".equals(this.getWsse().trim())) {
            throw new InvalidSecureTokenRuntimeException();
        }
        final String TOKEN_USER_NAME       = "Username=\"";
        final String TOKEN_PASSWORD_DIGEST = "PasswordDigest=\"";
        final String TOKEN_NONCE           = "Nonce=\"";
        final String TOKEN_CREATED         = "Created=\"";

        String loginId = 
            StringUtils.substringBetween(this.wsse_, TOKEN_USER_NAME, "\",");
        String passwordDigest = 
            StringUtils.substringBetween(this.wsse_, TOKEN_PASSWORD_DIGEST, "\",");
        String nonce = StringUtils.substringBetween(this.wsse_, TOKEN_NONCE, "\",");
        String created = 
            StringUtils.substringBetween(this.wsse_, TOKEN_CREATED, "\"");

        UserDto user = this.loginService_.findByLoginId(loginId);
        if (user == null) {
            throw new UnauthorizedRuntimeException();
        }
        String compPasswordDigest = 
            new String(
                new Base64().encode(
                    DigestUtils.sha(nonce + created + user.getPassword()))).trim();

        WebServiceSecurity security = 
            this.webServiceAuthService_.findByNonceAndCreated(nonce, created);

        int result = 
            loginId.equals(user.getLoginId())
         && passwordDigest.equals(compPasswordDigest)
         && security != null
         ?  user.getUserId()
         :  -1;
         if (result == -1) {
             throw new UnauthorizedRuntimeException();
         }
        return result;
    }
    
    /*
     * (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return ReflectionToStringBuilder.toString(this,
                ToStringStyle.MULTI_LINE_STYLE);
    }
    
}
