package org.seasar.tugboat.gtd.contents.meeting.dao;

import org.seasar.dao.unit.S2DaoTestCase;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBasic;
import org.seasar.tugboat.gtd.contents.meeting.entity.MeetingBrowsePageCondition;

/**
 * test meeting basics dao.
 * 
 * @author yosuke
 * 
 */
public class TestMeetingBasicsDao extends S2DaoTestCase {
    MeetingBasicsDao meetingBasicsDao_;
    
    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() {
        this.include("dicon/contents/meeting/meetingDao.dicon");
    }
    
    /**
     * test #insert.
     *
     */
    public void testInsertTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        int newId = this.meetingBasicsDao_.findNewId().getId();
        MeetingBasic basic = new MeetingBasic();
        basic.setId(newId);
        basic.setActivityId(9990);
        basic.setStatusId(1);
        basic.setPostedUserId(9990);
        basic.setTitle("meeting eleven.");
        
        this.meetingBasicsDao_.insert(basic);
        
        super.assertEquals(10000, newId);
    }
    
    /**
     * test #update.
     *
     */
    public void testUpdateTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        int newId = this.meetingBasicsDao_.findNewId().getId();
        MeetingBasic basic = new MeetingBasic();
        basic.setId(newId);
        basic.setActivityId(9990);
        basic.setStatusId(1);
        basic.setPostedUserId(9990);
        basic.setTitle("meeting eleven.");
        
        this.meetingBasicsDao_.insert(basic);
        super.assertEquals(10000, newId);
        
        basic.setTitle("meeting eleven part2.");
        this.meetingBasicsDao_.update(basic);
    }
    
    /**
     * test #updateStatus.
     *
     */
    public void testUpdateStatusTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        MeetingBasic basic = new MeetingBasic();
        basic.setId(9990);
        basic.setStatusId(1);
        basic.setPostedUserId(9990);
        
        this.meetingBasicsDao_.updateStatus(basic);
    }
    
    /**
     * test #findById.
     *
     */
    public void testFindByIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        MeetingBasic entity = 
            this.meetingBasicsDao_.findById(9990, 9990);
        
        super.assertEquals(9990, entity.getId());
        super.assertEquals(9990, entity.getActivityId());
        super.assertEquals(true, "Test Meeting 1".equals(entity.getTitle()));
        super.assertEquals(true, "20070501".equals(entity.getStartDate()));
        super.assertEquals(true, "t1".equals(entity.getFirstName()));
        super.assertEquals(true, "test1".equals(entity.getLastName()));
    }
    
    /**
     * test #findNewId.
     *
     */
    public void testFindNewIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        MeetingBasic entity = 
            this.meetingBasicsDao_.findNewId();
        
        super.assertEquals(10000, entity.getId());
    }
    
    /**
     * test #findByUserId.
     *
     */
    public void testFindByUserIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        // case 1.
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(9990);
        condition.setLimit(20);
        condition.setOffset(0);
        
        MeetingBasic[] entities = this.meetingBasicsDao_.findByUserId(condition);
        super.assertEquals(9, entities.length);
        
        // case 2.
        condition.setQueryStatusIds(new Integer[]{1,2});
        entities = this.meetingBasicsDao_.findByUserId(condition);
        super.assertEquals(7, entities.length);
        
        // case 3.
        condition.setQueryActivityIdsIds(new Integer[]{9990,9991});
        entities = this.meetingBasicsDao_.findByUserId(condition);
        super.assertEquals(2, entities.length);
        
        // case 4.
        condition.setQueryRegistrationIds(new Integer[]{9990,9991});
        entities = this.meetingBasicsDao_.findByUserId(condition);
        super.assertEquals(2, entities.length);
        
    }
    
    /**
     * test #findCountByQueryParams.
     *
     */
    public void testFindCountByQueryParamsTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        // case 1.
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(9990);
        condition.setLimit(20);
        condition.setOffset(0);
        
        MeetingBasic entity = this.meetingBasicsDao_.findCountByQueryParams(condition);
        super.assertEquals(9, entity.getRowCount());
        
        // case 2.
        condition.setQueryStatusIds(new Integer[]{1,2});
        entity = this.meetingBasicsDao_.findCountByQueryParams(condition);
        super.assertEquals(7, entity.getRowCount());
        
        // case 3.
        condition.setQueryActivityIdsIds(new Integer[]{9990,9991});
        entity = this.meetingBasicsDao_.findCountByQueryParams(condition);
        super.assertEquals(2, entity.getRowCount());
        
        // case 4.
        condition.setQueryRegistrationIds(new Integer[]{9990,9991});
        entity = this.meetingBasicsDao_.findCountByQueryParams(condition);
        super.assertEquals(2, entity.getRowCount());
    }
    
    /**
     * test #findDefaultCountByFolder.
     *
     */
    public void testFindDefaultCountByFolderTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        MeetingBrowsePageCondition condition = new MeetingBrowsePageCondition();
        condition.setUserId(9990);
        MeetingBasic entity = 
            this.meetingBasicsDao_.findDefaultCountByFolder(condition);
        
        super.assertEquals(7, entity.getLibraryCount());
        super.assertEquals(2, entity.getArchivesCount());
    }
    
    /**
     * test #findWithTaskById.
     *
     */
    public void testFindWithTaskByIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        MeetingBasic[] entities = 
            this.meetingBasicsDao_.findWithTaskById(9990, 9990);
        
        super.assertEquals(1, entities.length);
        super.assertEquals(9990, entities[0].getActivityId());
        super.assertEquals(3, entities[0].getStatusId());
        super.assertEquals(9990, entities[0].getPostedUserId());
        super.assertEquals(true, "Test Meeting 1".equals(entities[0].getTitle()));
        super.assertEquals(true, "Meeting body 1".equals(entities[0].getBody()));
        super.assertEquals(true, "t1".equals(entities[0].getFirstName()));
        super.assertEquals(true, "test1".equals(entities[0].getLastName()));
    }
    
    /**
     * test #findByUserIdForTimeline.
     *
     */
    public void testFindByUserIdForTimelineTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        MeetingBasic[] entities = 
            this.meetingBasicsDao_.findByUserIdForTimeline(9990);
        
        super.assertEquals(9, entities.length);
    }
    
    /**
     * test #getCountByActivityId.
     *
     */
    public void testGetCountByActivityIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        int result = this.meetingBasicsDao_.getCountByActivityId(9990);
        super.assertEquals(3, result);
    }
    
    /**
     * test #getCountByActivityIdAndPostedUserId.
     *
     */
    public void testGetCountByActivityIdAndPostedUserIdTx() {
        super.readXlsWriteDb("meeting_basics.xls");
        
        int result = 
            this.meetingBasicsDao_.getCountByActivityIdAndPostedUserId(9990, 9990);
        super.assertEquals(3, result);
    }
}
