package org.seasar.tugboat.gtd.webservices.helper;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import junit.framework.TestCase;

import org.seasar.tugboat.gtd.contents.common.entity.Classification;
import org.seasar.tugboat.gtd.contents.task.dto.TaskTagDto;
import org.seasar.tugboat.gtd.contents.task.entity.Tag;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskDetailItemModule;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskDetailItemModuleImpl;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskHeaderFeedModule;
import org.seasar.tugboat.gtd.webservices.feeds.atom.TaskHeaderFeedModuleImpl;

import com.sun.syndication.feed.atom.Content;
import com.sun.syndication.feed.atom.Entry;
import com.sun.syndication.feed.atom.Feed;
import com.sun.syndication.feed.atom.Link;
import com.sun.syndication.feed.atom.Person;
import com.sun.syndication.feed.rss.Channel;
import com.sun.syndication.feed.rss.Description;
import com.sun.syndication.feed.rss.Item;
import com.sun.syndication.io.WireFeedOutput;

/**
 * test generate atom feed.
 * 
 * @author yosuke
 * 
 */
public class TestGenerateAtomFeed extends TestCase {

    /*
     * (non-Javadoc)
     * 
     * @see junit.framework.TestCase#setUp()
     */
    public void setUp() {
    }

    /**
     * test generate atom feed.
     *
     */
    public void testGenerateATOM() {
        Feed feed = new Feed();  
        feed.setFeedType("atom_1.0");
        feed.setId("http://tugboat-gtd.sandbox.seasar.org/users/1");                
        feed.setUpdated(new Date());
        feed.setTitle("tugbaot.GTD - Task feeds");    
        
        Content subtitle = new Content();
        subtitle.setValue("tugbaot.GTD - Task feeds");
        feed.setSubtitle(subtitle);
        
        Link selfLink = new Link();
        selfLink.setHref("http://tugboat-gtd.sandbox.seasar.org/users/1"); 
        selfLink.setRel("self");
        feed.setOtherLinks(Collections.singletonList(selfLink));
        
        // added TaskSecurity module.
        TaskHeaderFeedModule exUserElement = new TaskHeaderFeedModuleImpl();
        exUserElement.setCreated("2007-08-20T00:00+00:00");
        exUserElement.setNonce("qwerty");
        
        List<Classification> classes = new ArrayList<Classification>();
        classes.add(new Classification(1,"Today",1));
        classes.add(new Classification(2,"Week",2));
        exUserElement.setClassifications(classes.toArray(new Classification[classes.size()]));
        
        List<Tag> tags = new ArrayList<Tag>();
        tags.add(new Tag(1,"Agile",5));
        tags.add(new Tag(2,"Java",10));
        exUserElement.setTags(tags.toArray(new Tag[tags.size()]));
        
        feed.getModules().add(exUserElement);
        
        //
        // Entries.
        //
        List<Entry> entries = new ArrayList<Entry>();
        
        Entry entry1 = new Entry();
        entry1.setTitle("$TITLE");
        entry1.setId("http://tugboat-gtd.sandbox.saesar.org/browse/tasks/$TASK_ID");  
        
        Link altLink = new Link();
        altLink.setHref("http://tugboat-gtd.sandbox.saesar.org/browse/tasks/$TASK_ID");
        entry1.setAlternateLinks(Collections.singletonList(altLink));
        
        entry1.setPublished(new Date());         
        entry1.setUpdated(new Date());         
        Person author = new Person();
        author.setName("$POSTED_USER");
        entry1.setAuthors(Collections.singletonList(author));        

        Content content1 = new Content(); 
        content1.setValue("$DESCRIPTION");
        entry1.setContents(Collections.singletonList(content1));   
        
        // tgb
        TaskDetailItemModule exDetailElement = new TaskDetailItemModuleImpl();
        exDetailElement.setStatusId(1);
        exDetailElement.setActivityId(2);
        exDetailElement.setActivityName("Personal");
        exDetailElement.setActivityColor("#c0c0c0");
        exDetailElement.setClassificationId(3);
        exDetailElement.setPersonInChargeId(4);
        exDetailElement.setPersonInChargeFirstName("Yosuke");
        exDetailElement.setPersonInChargeLastName("Hara");
        
        List<TaskTagDto> taskTags = new ArrayList<TaskTagDto>();
        taskTags.add(new TaskTagDto(1,"Java"));
        taskTags.add(new TaskTagDto(2,"JavaScript"));
        taskTags.add(new TaskTagDto(3,"0.8.0"));
        exDetailElement.setTags(taskTags.toArray(new TaskTagDto[taskTags.size()]));
        
        entry1.getModules().add(exDetailElement);
        
        entries.add(entry1);
        feed.setEntries(entries);
        
        try {
            WireFeedOutput out = new WireFeedOutput();
            out.output(feed, new PrintWriter(System.out));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    /**
     * test generate atom feed.
     *
     */
    public void testGenerateRSS() {
        Channel channel = new Channel();  
        channel.setFeedType("rss_2.0");
        channel.setLanguage("ja-jp");                
        channel.setTitle("tugboat.GTD - Task Feeds");         
        channel.setDescription("tugboat.GTD - Task Feeds");  
        channel.setLink("http://tugboat-gtd.sandbox.seasar.org/users/1");                                           
        channel.setPubDate(new Date()); 
        
        List<Item> items = new ArrayList<Item>();
        
        // #1
        Item item1 = new Item();
        item1.setTitle("$TITILE");
        item1.setLink("http://localhost:8080/tugboat-gtd/browse/tasks/1");
        item1.setPubDate(new Date());   
        item1.setAuthor("$POSTED_USER");
        
        Description desc = new Description();
        desc.setValue("$ACTIVITY_NAME\n$STATUS_NAME\n$CLASSIFICATION_NAME\n$DUE\n$DESCRIPTION");
        item1.setDescription(desc); 
        items.add(item1); 
        
        // #2
        item1 = new Item();
        item1.setTitle("$TITILE");
        item1.setLink("http://localhost:8080/tugboat-gtd/browse/tasks/2");
        item1.setPubDate(new Date());   
        item1.setAuthor("$POSTED_USER");
        
        desc = new Description();
        desc.setValue("$ACTIVITY_NAME\n$STATUS_NAME\n$CLASSIFICATION_NAME\n$DUE\n$DESCRIPTION");
        item1.setDescription(desc); 
        items.add(item1); 
        
        // generate.
        channel.setItems(items);  
        try {
            WireFeedOutput out = new WireFeedOutput();
            out.output(channel, new PrintWriter(System.out));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
